library(shiny)
library(ggplot2)
library(pbapply)
library(reshape2)
library(dplyr)
library(DT)
library(gridExtra)
library(rhandsontable)
library(Cairo)
library(broom)
library(shinyjs)
library(formattable)
library(markdown)
library(rmarkdown)
library(XML)
library(corrplot)
library(scales)
library(caret)
library(randomForest)
library(DescTools)
pdf(NULL)


options(shiny.maxRequestSize=9000000*1024^2)

options(warn=-1)
assign("last.warning", NULL, envir = baseenv())

shinyServer(function(input, output, session) {
    
    
    
    

    
    output$filegrab <- renderUI({
        
        if(input$filetype=="CSV") {
            fileInput('file1', 'Choose CSV', multiple=TRUE,
            accept=c(".csv"))
        } else if(input$filetype=="TXT") {
            fileInput('file1', 'Choose TXT', multiple=TRUE,
            accept=c(".txt"))
        } else if(input$filetype=="Net") {
            fileInput('file1', 'Choose Net Counts', multiple=TRUE,
            accept=c(".csv"))
        } else if(input$filetype=="Elio") {
            fileInput('file1', 'Choose Elio Spectra', multiple=TRUE,
            accept=c(".spt"))
        } else if(input$filetype=="MCA") {
            fileInput('file1', 'Choose MCA File', multiple=TRUE,
            accept=c(".mca"))
        } else if(input$filetype=="SPX") {
            fileInput('file1', 'Choose Artax File', multiple=TRUE,
            accept=c(".spx"))
        } else if(input$filetype=="PDZ") {
            fileInput('file1', 'Choose PDZ File', multiple=TRUE,
            accept=c(".pdz"))
        }
        
    })
    
    
    output$gainshiftui <- renderUI({
        
        if(input$advanced==TRUE){
            numericInput('gainshift', "Gain Shift (keV)", min=-1, max=1, value=0)
        } else {
            p()
        }
        
    })
    
    
    output$binaryui <- renderUI({
        
        if(input$advanced==TRUE && input$filetype=="PDZ"){
            numericInput('binaryshift', "Binary Shift (bits)", min=0, max=1000, value=0)
        } else {
            p()
        }
        
    })
    
    
    binaryHold <- reactive({
        
        if(input$advanced==TRUE){
            input$binaryshift
        } else if(input$advanced==FALSE){
            500
        }
        
    })
    
    
    gainshiftHold <- reactive({
        
        if(input$advanced==TRUE){
            input$gainshift
        } else if(input$advanced==FALSE){
            0
        }
        
    })
    
    
    fullSpectraDataTable <- reactive({
        
        
        withProgress(message = 'Processing Data', value = 0, {
            
            inFile <- input$file1
            if (is.null(inFile)) return(NULL)
            temp = inFile$name
            temp <- gsub(".csv", "", temp)
            id.seq <- seq(1, 2048,1)
            
            n <- length(temp)*id.seq
            
            myfiles.x = pblapply(inFile$datapath, read_csv_filename_x)
            
            
            
            myfiles.y = pblapply(inFile$datapath, read_csv_filename_y)
            
            
            
            
            xrf.x <- data.frame(id.seq, myfiles.x)
            colnames(xrf.x) <- c("ID", temp)
            xrf.y <- data.frame(id.seq, myfiles.y)
            colnames(xrf.y) <- c("ID", temp)
            
            
            xrf.x <- data.table(xrf.x)
            xrf.y <- data.table(xrf.y)
            
            
            energy.m <- xrf.x[, list(variable = names(.SD), value = unlist(.SD, use.names = F)), by = ID]
            cps.m <- xrf.y[, list(variable = names(.SD), value = unlist(.SD, use.names = F)), by = ID]
            
            
            spectra.frame <- data.frame(energy.m$value, cps.m$value, cps.m$variable)
            colnames(spectra.frame) <- c("Energy", "CPS", "Spectrum")
            data <- spectra.frame
            
            
            incProgress(1/n)
            Sys.sleep(0.1)
        })
        
        data$Energy <- data$Energy + gainshiftHold()
        
        data
    })
    
    
    
    fullSpectra <- reactive({
        
        
        withProgress(message = 'Processing Data', value = 0, {
            
            inFile <- input$file1
            if (is.null(inFile)) return(NULL)
            temp = inFile$name
            temp <- gsub(".csv", "", temp)
            id.seq <- seq(1, 2048,1)
            
            n <- length(temp)*id.seq
            
            n.seq <- seq(1, length(inFile$name), 1)
            
            
            data <- pblapply(n.seq, function(x) csvFrame(filepath=inFile$datapath[x], filename=inFile$name[x]))
            data <- do.call("rbind", data)
            
            
            incProgress(1/n)
            Sys.sleep(0.1)
        })
        
        data$Energy <- data$Energy + gainshiftHold()
        
        data
    })
    
    
    netCounts <- reactive({
        
        withProgress(message = 'Processing Data', value = 0, {
            
            
            inFile <- input$file1
            if (is.null(inFile)) return(NULL)
            
            #inName <- inFile$name
            #inPath <- inFile$datapath
            
            #inList <- list(inName, inPath)
            #names(inList) <- c("inName", "inPath")
            
            
            n <- length(inFile$name)
            net.names <- gsub("\\@.*","",inFile$name)
            
            myfiles = pblapply(inFile$datapath,  read_csv_net)
            
            
            myfiles.frame.list <- pblapply(myfiles, data.frame, stringsAsFactors=FALSE)
            nms = unique(unlist(pblapply(myfiles.frame.list, names)))
            myfiles.frame <- as.data.frame(do.call(rbind, lapply(myfiles.frame.list, "[", nms)))
            myfiles.frame <- as.data.frame(sapply(myfiles.frame, as.numeric))
            
            
            #myfiles.frame$Spectrum <- net.names
            
            united.frame <- data.frame(net.names, myfiles.frame)
            colnames(united.frame) <- c("Spectrum", names(myfiles.frame))
            #united.frame$None <- rep(1, length(united.frame$Spectrum))
            
            
            incProgress(1/n)
            Sys.sleep(0.1)
        })
        
        united.frame <- as.data.frame(united.frame)
        united.frame
        
    })
    
    
    readTXT <- reactive({
        
        withProgress(message = 'Processing Data', value = 0, {
            
            inFile <- input$file1
            if (is.null(inFile)) return(NULL)
            
            n <- length(inFile$datapath)
            names <- inFile$name
            
            myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readTXTData(filepath=inFile$datapath[x], filename=inFile$name[x]))))
            
            incProgress(1/n)
            Sys.sleep(0.1)
        })
        
        myfiles.frame$Energy <- myfiles.frame$Energy + gainshiftHold()
        
        myfiles.frame
        
    })
    
    readElio <- reactive({
        
        withProgress(message = 'Processing Data', value = 0, {
            
            inFile <- input$file1
            if (is.null(inFile)) return(NULL)
            
            n <- length(inFile$datapath)
            names <- inFile$name
            
            myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readSPTData(filepath=inFile$datapath[x], filename=inFile$name[x]))))
            
            
            incProgress(1/n)
            Sys.sleep(0.1)
        })
        
        myfiles.frame$Energy <- myfiles.frame$Energy + gainshiftHold()
        
        myfiles.frame
        
        
    })
    
    
    readMCA <- reactive({
        
        withProgress(message = 'Processing Data', value = 0, {
            
            inFile <- input$file1
            if (is.null(inFile)) return(NULL)
            
            n <- length(inFile$datapath)
            names <- inFile$name
            
            myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readMCAData(filepath=inFile$datapath[x], filename=inFile$name[x]))))
            
            
            incProgress(1/n)
            Sys.sleep(0.1)
        })
        
        myfiles.frame$Energy <- myfiles.frame$Energy + gainshiftHold()
        
        myfiles.frame
        
        
    })
    
    
    readSPX <- reactive({
        
        withProgress(message = 'Processing Data', value = 0, {
            
            inFile <- input$file1
            if (is.null(inFile)) return(NULL)
            
            n <- length(inFile$datapath)
            names <- inFile$name
            
            myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readSPXData(filepath=inFile$datapath[x], filename=inFile$name[x]))))
            
            
            incProgress(1/n)
            Sys.sleep(0.1)
        })
        
        myfiles.frame$Energy <- myfiles.frame$Energy + gainshiftHold()
        
        myfiles.frame
        
        
    })
    
    
    readPDZ <- reactive({
        
        withProgress(message = 'Processing Data', value = 0, {
            
            inFile <- input$file1
            if (is.null(inFile)) return(NULL)
            
            n <- length(inFile$datapath)
            names <- inFile$name
            
            if(input$advanced==FALSE){
                myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readPDZData(filepath=inFile$datapath[x], filename=inFile$name[x]))))
            } else if(input$advanced==TRUE){
                myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readPDZ25DataManual(filepath=inFile$datapath[x], filename=inFile$name[x], binaryshift=binaryHold()))))
                
            }
            
            
            incProgress(1/n)
            Sys.sleep(0.1)
        })
        
        myfiles.frame$Energy <- myfiles.frame$Energy + gainshiftHold()
        
        myfiles.frame
        
        
    })
    
    
    
    observeEvent(input$actionprocess, {
      
      blankNotes <- reactive({
          
          "Add your notes here"
          
      })
      
      calNotes <- reactive({
          
          if(!is.null(calFileContents()[["Notes"]])){
              calFileContents()[["Notes"]]
          } else if(is.null(calFileContents()[["Notes"]])){
              blankNotes()
          }
          
      })
      
      
      
      
      notesObject <- reactive({
          
          if(input$usecalfile==FALSE){
              blankNotes()
          } else if(input$usecalfile==TRUE){
              calNotes()
          }
          
      })
      
      output$notesui <- renderUI({
          
          
          textAreaInput('notes', "Notes", placeholder=notesObject(), width="600px", height="500px")
          
          
      })
        
        myData <- reactive({
            
            data <- if(input$filetype=="CSV"){
                fullSpectra()
            } else if(input$filetype=="TXT"){
                readTXT()
            } else if(input$filetype=="Net"){
                netCounts()
            } else if(input$filetype=="Elio"){
                readElio()
            }  else if(input$filetype=="MCA"){
                readMCA()
            }  else if(input$filetype=="SPX"){
                readSPX()
            }  else if(input$filetype=="PDZ"){
                readPDZ()
            }
            
            data
            
            
        })
        
        
        calFileContents <- reactive({
            
            existingCalFile <- input$calfileinput
            
            if (is.null(existingCalFile)) return(NULL)
            
            
            Calibration <- readRDS(existingCalFile$datapath)
            
            
            
            Calibration[["Values"]]$Spectrum <- gsub(".spx", "", Calibration[["Values"]]$Spectrum)
            Calibration[["Values"]]$Spectrum <- gsub(".pdz", "", Calibration[["Values"]]$Spectrum)
            Calibration[["Values"]]$Spectrum <- gsub(".CSV", "", Calibration[["Values"]]$Spectrum)
            Calibration[["Values"]]$Spectrum <- gsub(".csv", "", Calibration[["Values"]]$Spectrum)
            Calibration[["Values"]]$Spectrum <- gsub(".spt", "", Calibration[["Values"]]$Spectrum)
            Calibration[["Values"]]$Spectrum <- gsub(".mca", "", Calibration[["Values"]]$Spectrum)
            
            Calibration[["Spectra"]]$Spectrum <- gsub(".spx", "", Calibration[["Spectra"]]$Spectrum)
            Calibration[["Spectra"]]$Spectrum <- gsub(".pdz", "", Calibration[["Spectra"]]$Spectrum)
            Calibration[["Spectra"]]$Spectrum <- gsub(".CSV", "", Calibration[["Spectra"]]$Spectrum)
            Calibration[["Spectra"]]$Spectrum <- gsub(".csv", "", Calibration[["Spectra"]]$Spectrum)
            Calibration[["Spectra"]]$Spectrum <- gsub(".spt", "", Calibration[["Spectra"]]$Spectrum)
            Calibration[["Spectra"]]$Spectrum <- gsub(".mca", "", Calibration[["Spectra"]]$Spectrum)
            
            Calibration
            
        })
        
        dataHold <- reactive({
            data <- if(input$usecalfile==FALSE){
                myData()
            } else if(input$usecalfile==TRUE && input$usespectravalues==FALSE){
                calFileContents()$Spectra
            } else if (input$usecalfile==TRUE && input$usespectravalues==TRUE){
                myData()
            }
            
            data <- data[order(as.character(data$Spectrum)),]
            
            data$Spectrum <- gsub(".pdz", "", data$Spectrum)
            data$Spectrum <- gsub(".csv", "", data$Spectrum)
            data$Spectrum <- gsub(".CSV", "", data$Spectrum)
            data$Spectrum <- gsub(".spt", "", data$Spectrum)
            data$Spectrum <- gsub(".mca", "", data$Spectrum)
            data$Spectrum <- gsub(".spx", "", data$Spectrum)
            
            data
            
        })
        
        
        dataCount <- reactive({
            inFile <- input$file1
            
            if(input$usecalfile==FALSE){
                length(inFile$datapath)
            }else if(input$usecalfile==TRUE && input$usespectravalues==FALSE){
                length(calFileContents()$Intensities)
            }else if(input$usecalfile==TRUE && input$usespectravalues==TRUE){
                length(inFile$datapath)
            }
        })
        
        
        
        
        # Return the requested dataset
        datasetInput <- reactive({
            switch(input$element,
            "H.table" = H.table,
            "He.table" = He.table,
            "Li.table" = Li.table,
            "Be.table" = Be.table,
            "B.table" = B.table,
            "C.table" = C.table,
            "N.table" = N.table,
            "O.table" = O.table,
            "F.table" = F.table,
            "Ne.table" = Ne.table,
            "Na.table" = Na.table,
            "Mg.table" = Mg.table,
            "Al.table" = Al.table,
            "Si.table" = Si.table,
            "P.table" = P.table,
            "S.table" = S.table,
            "Cl.table" = Cl.table,
            "Ar.table" = Ar.table,
            "K.table" = K.table,
            "Ca.table" = Ca.table,
            "Sc.table" = Sc.table,
            "Ti.table" = Ti.table,
            "V.table" = V.table,
            "Cr.table" = Cr.table,
            "Mn.table" = Mn.table,
            "Fe.table" = Fe.table,
            "Co.table" = Co.table,
            "Ni.table" = Ni.table,
            "Cu.table" = Cu.table,
            "Zn.table" = Zn.table,
            "Ga.table" = Ga.table,
            "Ge.table" = Ge.table,
            "As.table" = As.table,
            "Se.table" = Se.table,
            "Br.table" = Br.table,
            "Kr.table" = Kr.table,
            "Rb.table" = Rb.table,
            "Sr.table" = Sr.table,
            "Y.table" = Y.table,
            "Zr.table" = Zr.table,
            "Nb.table" = Nb.table,
            "Mo.table" = Mo.table,
            "Tc.table" = Tc.table,
            "Ru.table" = Ru.table,
            "Rh.table" = Rh.table,
            "Pd.table" = Pd.table,
            "Ag.table" = Ag.table,
            "Cd.table" = Cd.table,
            "In.table" = In.table,
            "Sn.table" = Sn.table,
            "Sb.table" = Sb.table,
            "Te.table" = Te.table,
            "I.table" = I.table,
            "Xe.table" = Xe.table,
            "Cs.table" = Cs.table,
            "Ba.table" = Ba.table,
            "La.table" = La.table,
            "Ce.table" = Ce.table,
            "Pr.table" = Pr.table,
            "Nd.table" = Nd.table,
            "Pm.table" = Pm.table,
            "Sm.table" = Sm.table,
            "Eu.table" = Eu.table,
            "Gd.table" = Gd.table,
            "Tb.table" = Tb.table,
            "Dy.table" = Dy.table,
            "Ho.table" = Ho.table,
            "Er.table" = Er.table,
            "Tm.table" = Tm.table,
            "Yb.table" = Yb.table,
            "Lu.table" = Lu.table,
            "Hf.table" = Hf.table,
            "Ta.table" = Ta.table,
            "W.table" = W.table,
            "Re.table" = Re.table,
            "Os.table" = Os.table,
            "Ir.table" = Ir.table,
            "Pt.table" = Pt.table,
            "Au.table" = Au.table,
            "Hg.table" = Hg.table,
            "Tl.table" = Tl.table,
            "Pb.table" = Pb.table,
            "Bi.table" = Bi.table,
            "Po.table" = Po.table,
            "At.table" = At.table,
            "Rn.table" = Rn.table,
            "Fr.table" = Fr.table,
            "Ra.table" = Ra.table,
            "Ac.table" = Ac.table,
            "Th.table" = Th.table,
            "Pa.table" = Pa.table,
            "U.table" = U.table)
        })
        
        observeEvent(input$actionplot, {
            
            # Expression that generates a histogram. The expression is
            # wrapped in a call to renderPlot to indicate that:
            #
            #  1) It is "reactive" and therefore should re-execute automatically
            #     when inputs change
            #  2) Its output type is a plot
            ranges <- reactiveValues(x = NULL, y = NULL)
            
            
            
            plotInput <- reactive({
                
                data <- dataHold()
                
                
                
                id.seq <- seq(1, 2048,1)
                
                n <- length(data$Energy)
                
                element <- datasetInput()
                intensity.norm <- (element$Intensity/max(element$Intensity))*max(data$CPS)
                intensity.base <- (element$Intensity/max(element$Intensity))
                
                
                
                spectral.plot <- qplot(data$Energy, data$CPS, xlab = "Energy (keV)", ylab = "Counts per Second", geom="line", colour=data$Spectrum) +
                theme_light()+
                theme(legend.position="bottom") +
                geom_segment(aes(x=element$Line, xend=element$Line, y = 0, yend=intensity.norm), colour="grey50", linetype=2)  +
                scale_colour_discrete("Spectrum") +
                coord_cartesian(xlim = ranges$x, ylim = ranges$y)
                
                spectral.plot
                
            })
            
            
            output$distPlot <- renderPlot({
                
                plotInput()
                
                
            })
            
            # When a double-click happens, check if there's a brush on the plot.
            # If so, zoom to the brush bounds; if not, reset the zoom.
            observeEvent(input$cropspectra, {
                data <- dataHold()
                brush <- input$plot1_brush
                if (!is.null(brush)) {
                    ranges$x <- c(brush$xmin, brush$xmax)
                    ranges$y <- c(brush$ymin, brush$ymax)
                    
                } else {
                    ranges$x <- NULL
                    ranges$y <- NULL
                }
                
                
                
            })
            
            output$downloadPlot <- downloadHandler(
            filename = function() { paste(input$dataset, '.png', sep='') },
            content = function(file) {
                ggsave(file,plotInput(), width=10, height=10)
            }
            )
            
            
            
            
            
            
            
            
        })
        
        
        
        
        
        
        
        standardElements <- reactive({
            
            spectra.line.table <- dataHold()
            
            
            if(input$usecalfile==FALSE && input$filetype=="CSV"){
                standard
            } else if(input$usecalfile==FALSE && input$filetype=="TXT"){
                standard
            } else if(input$usecalfile==FALSE && input$filetype=="Elio"){
                standard
            }  else if(input$usecalfile==FALSE && input$filetype=="MCA"){
                standard
            }  else if(input$usecalfile==FALSE && input$filetype=="SPX"){
                standard
            }  else if(input$usecalfile==FALSE && input$filetype=="PDZ"){
                standard
            } else if(input$usecalfile==FALSE && input$filetype=="Net"){
                colnames(spectra.line.table[2:4])
            } else if(input$usecalfile==TRUE){
                ls(calFileContents()$Intensities)
            }
            
        })
        
        standardLines <- reactive({
            
            spectra.line.table <- dataHold()
            
            n <- length(names(spectra.line.table))
            
            
            choices <- if(input$filetype=="CSV"){
                spectralLines
            } else if(input$filetype=="TXT"){
                spectralLines
            } else if(input$filetype=="Elio"){
                spectralLines
            }  else if(input$filetype=="MCA"){
                spectralLines
            }  else if(input$filetype=="SPX"){
                spectralLines
            }  else if(input$filetype=="PDZ"){
                spectralLines
            } else if(input$filetype=="Net"){
                colnames(spectra.line.table[2:n])
            }
            
            choices
            
            
        })
        
        
        
        
        
        selectedKalpha <- reactive({
            
            if(input$usecalfile==FALSE){
                c("Ca.K.alpha", "Ti.K.alpha", "Fe.K.alpha")
            } else if(input$usecalfile==TRUE){
                as.vector(subset(selectedElementsCalpre(), Orbital=="K" & Line=="alpha")$ElementLine)
            }
            
        })
        
        
        selectedKbeta <- reactive({
            
            if(input$usecalfile==FALSE){
                NULL
            } else if(input$usecalfile==TRUE){
                as.vector(subset(selectedElementsCalpre(), Orbital=="K" & Line=="beta")$ElementLine)
            }
            
        })
        
        selectedLalpha <- reactive({
            
            if(input$usecalfile==FALSE){
                "Rh.L.alpha"
            } else if(input$usecalfile==TRUE){
                as.vector(subset(selectedElementsCalpre(), Orbital=="L" & Line=="alpha")$ElementLine)
            }
            
        })
        
        selectedLbeta <- reactive({
            
            if(input$usecalfile==FALSE){
                "Pb.L.beta"
            } else if(input$usecalfile==TRUE){
                as.vector(subset(selectedElementsCalpre(), Orbital=="L" & Line=="beta")$ElementLine)
            }
            
        })
        
        selectedMLine <- reactive({
            
            if(input$usecalfile==FALSE){
                NULL
            } else if(input$usecalfile==TRUE){
                as.vector(subset(selectedElementsCalpre(), Orbital=="M" & Line=="line")$Element)
            }
            
        })
        
        output$checkboxElements <-  renderUI({
            
            checkboxGroupInput("show_vars", label="Elemental lines to show:",
            choices = standardLines(), selected = selectedElements())
            
        })
        
        output$checkboxElementsKalpha <-  renderUI({
            
            selectInput("show_vars_k_alpha", label="K-alpha",
            choices = kalphaLines, selected = selectedKalpha() , multiple=TRUE)
            
        })
        
        output$checkboxElementsKbeta <-  renderUI({
            
            selectInput("show_vars_k_beta", label="K-beta",
            choices = kbetaLines, selected = selectedKbeta(), multiple=TRUE)
            
        })
        
        output$checkboxElementsLalpha <-  renderUI({
            
            selectInput("show_vars_l_alpha", label="L-alpha",
            choices = lalphaLines, selected = selectedLalpha(), multiple=TRUE)
            
        })
        
        output$checkboxElementsLbeta <-  renderUI({
            
            selectInput("show_vars_l_beta", label="L-beta",
            choices = lbetaLines, selected = selectedLbeta(), multiple=TRUE)
            
        })
        
        output$checkboxElementsM <-  renderUI({
            
            selectInput("show_vars_m", label="M",
            choices = mLines, selected = selectedMLine(), multiple=TRUE)
            
        })
        
        
        
        
        linevalues <- reactiveValues()
        
        
        lineInput <- reactive({
            
            blank.frame <- data.frame(
            Name=as.vector(as.character(rep("", 25))),
            EnergyMin=as.numeric(rep("", 25)),
            EnergyMax=as.numeric(rep("", 25)),
            stringsAsFactors = FALSE
            )
            
            blank.frame
            
        })
        
        
        lineInputCal <- reactive({
            
            
            calFileContents()[["Definitions"]]
            
            
        })
        
        lineTableInput <- reactive({
            
            
            if(input$usecalfile==FALSE){
                lineInput()
            } else if(input$usecalfile==TRUE && "Definitions" %in% ls(calFileContents())){
                lineInputCal()
            } else if(input$usecalfile==TRUE && !"Definitions" %in% ls(calFileContents())){
                lineInput()
            }
            
        })
        
        observe({
            if (!is.null(input$hotline)) {
                DF <- hot_to_r(input$hotline)
            } else {
                DF <- lineTableInput()
            }
            linevalues[["DF"]] <- DF
        })
        
        eventReactive(input$linecommit,{
            
            linevalues[["DF"]] <- lineTableInput()
            
        })
        
        
        ## Handsontable
        
        output$hotline <- renderRHandsontable({
            
            DF <- linevalues[["DF"]]
            
            
            
            
            rhandsontable(DF) %>% hot_col(1:length(DF), renderer=htmlwidgets::JS("safeHtmlRenderer"))
            
            
        })
        
        
        observeEvent(input$resethotableline, {
            
            linevalues[["DF"]] <- NULL
            
            linevalues[["DF"]] <- lineTableInput()
            
            
        })
        
        
        
        # randomInterList <- reactive({
        #   if (is.null(input$intercept_vars))
        #   paste(,2)
        #   else
        #   input$intercept_vars
        #})
        
        
        #randomSlopeList <- reactive({
        #   if (is.null(input$intercept_vars))
        #   paste(,2)
        #   else
        #   input$slope_vars
        #})
        
        #output$nullintercept <- randomInterList()
        
        #output$nullslope <- randomSlopeList()
        
        lineSubset <- reactive({
            
            xrf_parse(range.table = linevalues[["DF"]], data=dataHold())
            
        })
        
        
        output$LineValues <- renderDataTable({
            lineSubset()
        })
        
        
        
        
        
        testfFrame <- reactive({
            
            k.alpha.lines <- kalphaLines[input$show_vars_k_alpha]
            k.beta.lines <- kbetaLines[input$show_vars_k_beta]
            l.alpha.lines <- lalphaLines[input$show_vars_l_alpha]
            l.beta.lines <- lbetaLines[input$show_vars_l_beta]
            m.lines <- mLines[input$show_vars_m]
            
            k.alpha.lines <- k.alpha.lines[!is.na(k.alpha.lines)]
            k.beta.lines <- k.beta.lines[!is.na(k.beta.lines)]
            l.alpha.lines <- l.alpha.lines[!is.na(l.alpha.lines)]
            l.beta.lines <- l.beta.lines[!is.na(l.beta.lines)]
            m.lines <- m.lines[!is.na(m.lines)]
            
            choosen.elements <- c(input$show_vars_k_alpha, input$show_vars_k_beta, input$show_vars_l_alpha, input$show_vars_l_beta, input$show_vars_m)
            
            
            element.frame <- data.frame(elements=choosen.elements, order=atomic_order_vector(choosen.elements))
            
            element.frame
        })
        
        selectedElements <- reactive({
            
            k.alpha.lines <- kalphaLines[input$show_vars_k_alpha]
            k.beta.lines <- kbetaLines[input$show_vars_k_beta]
            l.alpha.lines <- lalphaLines[input$show_vars_l_alpha]
            l.beta.lines <- lbetaLines[input$show_vars_l_beta]
            m.lines <- mLines[input$show_vars_m]
            
            k.alpha.lines <- k.alpha.lines[!is.na(k.alpha.lines)]
            k.beta.lines <- k.beta.lines[!is.na(k.beta.lines)]
            l.alpha.lines <- l.alpha.lines[!is.na(l.alpha.lines)]
            l.beta.lines <- l.beta.lines[!is.na(l.beta.lines)]
            m.lines <- m.lines[!is.na(m.lines)]
            
            choosen.elements <- c(input$show_vars_k_alpha, input$show_vars_k_beta, input$show_vars_l_alpha, input$show_vars_l_beta, input$show_vars_m)
            
            element.frame <- data.frame(elements=choosen.elements, order=atomic_order_vector(choosen.elements))
            as.vector(element.frame[order(element.frame$order),]$elements)
            
            
        })
        
        selectedElementsCalpre <- reactive({
            
            element.lines <- ls(calFileContents()$Intensities)
            
            
            do.call("rbind", lapply(element.lines, element_line_pull))
            
        })
        
        selectedElementsCalpost <- reactive({
            table <- linevalues[["DF"]]
            table <- table[complete.cases(table),]
            
            element.lines <- ls(calFileContents()$Intensities)
            
            
            c(do.call("rbind", lapply(element.lines, element_line_pull)),  as.vector(table$Name))
            
        })
        
        
        selectedElementsCal <- reactive({
            
            table <- linevalues[["DF"]]
            table <- table[complete.cases(table),]
            
            if(length(table[,1])==0){
                selectedElementsCalpre()
            } else if(length(table[,1])!=0){
                selectedElementsCalpost()
            }
            
        })
        
        elementallinestousepre <- reactive({
            
            table <- linevalues[["DF"]]
            table <- table[complete.cases(table),]
            
            #c(selectedElements()_k_alpha, selectedElements()_k_beta, selectedElements()_l_alpha, selectedElements()_l_beta, selectedElements()_m)
            
            selectedElements()
            
            
        })
        
        elementallinestousepost <- reactive({
            table <- linevalues[["DF"]]
            table <- table[complete.cases(table),]
            
            
            
            c(selectedElements(),  as.vector(table$Name))
            
        })
        
        
        elementallinestouse <- reactive({
            table <- linevalues[["DF"]]
            table <- table[complete.cases(table),]
            
            if(length(table[,1])==0){
                elementallinestousepre()
            } else if(length(table[,1])!=0){
                elementallinestousepost()
            }
            
            
        })
        
        
        
        spectraData <- reactive({
            
            line.data <- elementFrame(data=dataHold(), elements=elementallinestousepre())
            
            table <- linevalues[["DF"]]
            table <- table[complete.cases(table),]
            
            if(length(table[,1])==0){
                line.data
            } else if(length(table[,1])!=0){
                merge(line.data, lineSubset(), by="Spectrum")
            }
            
        })
        
        netData <- reactive({
            
            net.data <- dataHold()
            
            elements <- elementallinestouse()
            
            
            net.data.partial <- net.data[,elements]
            net.data <- data.frame(net.data$Spectrum ,net.data.partial)
            colnames(net.data) <- c("Spectrum", elements)
            net.data <- net.data[order(as.character(net.data$Spectrum)),]
            
            net.data$Spectrum <- gsub(".csv", "", net.data$Spectrum)
            net.data$Spectrum <- gsub(".CSV", "", net.data$Spectrum)
            
            net.data
            
        })
        
        
        
        
        calDataType <- reactive({
            
            select.line.table <- if(input$filetype=="CSV"){
                spectraData()
            } else if(input$filetype=="TXT"){
                spectraData()
            } else if(input$filetype=="Elio"){
                spectraData()
            }  else if(input$filetype=="MCA"){
                spectraData()
            }  else if(input$filetype=="SPX"){
                spectraData()
            }  else if(input$filetype=="PDZ"){
                spectraData()
            } else if(input$filetype=="Net"){
                netData()
            }
            
        })
        
        
        
        tableInput <- reactive({
            
            elements <- elementallinestouse()
            
            
            select.line.table <- if(input$filetype=="CSV"){
                spectraData()
            } else if(input$filetype=="TXT"){
                spectraData()
            } else if(input$filetype=="Elio"){
                spectraData()
            }  else if(input$filetype=="MCA"){
                spectraData()
            }  else if(input$filetype=="SPX"){
                spectraData()
            }  else if(input$filetype=="PDZ"){
                spectraData()
            } else if(input$filetype=="Net"){
                netData()
            }
            
            rounded <- round(select.line.table[,elements], digits=0)
            full <- data.frame(select.line.table$Spectrum, rounded)
            colnames(full) <- c("Spectrum", elements)
            
            full
        })
        
        
        output$mytable1 <- renderDataTable({
            
            base.table <- tableInput()[,-1]
            rownames(base.table) <- tableInput()$Spectrum
            base.table
            
        })
        
        output$covarianceplot <- renderPlot({
            
            data.table <- tableInput()
            correlations <- cor(data.table[,-1])
            corrplot(correlations, method="circle")
            
        })
        
        
        
        
        output$downloadData <- downloadHandler(
        filename = function() { paste(input$dataset, '.csv', sep=',') },
        content = function(file
        ) {
            write.csv(spectraData(), file)
        }
        )
        
        observeEvent(input$hotableprocess1, {
        })
        
        
        
        hotableInputBlank <- reactive({
            
            elements <- elementallinestouse()
            
            
            
            
            spectra.line.table <- if(input$filetype=="CSV"){
                spectraData()
            } else if(input$filetype=="TXT"){
                spectraData()
            } else if(input$filetype=="Elio"){
                spectraData()
            }  else if(input$filetype=="MCA"){
                spectraData()
            }  else if(input$filetype=="SPX"){
                spectraData()
            }  else if(input$filetype=="PDZ"){
                spectraData()
            } else if(input$filetype=="Net"){
                dataHold()
            }
            
            empty.line.table <- spectra.line.table[,elements] * 0.0000
            
            #empty.line.table$Spectrum <- spectra.line.table$Spectrum
            
            hold.frame <- data.frame(spectra.line.table$Spectrum, empty.line.table)
            colnames(hold.frame) <- c("Spectrum", elements)
            
            hold.frame <- as.data.frame(hold.frame)
            
            
            
            hold.frame
            
            
        })
        
        hotableInputCal <- reactive({
            
            elements <- elementallinestouse()
            
            
            
            
            spectra.line.table <- if(input$filetype=="CSV"){
                spectraData()
            } else if(input$filetype=="TXT"){
                spectraData()
            } else if(input$filetype=="Elio"){
                spectraData()
            }  else if(input$filetype=="MCA"){
                spectraData()
            }  else if(input$filetype=="SPX"){
                spectraData()
            }  else if(input$filetype=="PDZ"){
                spectraData()
            } else if(input$filetype=="Net"){
                dataHold()
            }
            
            empty.line.table <- spectra.line.table[,elements] * 0.0000
            
            #empty.line.table$Spectrum <- spectra.line.table$Spectrum
            
            hold.frame <- data.frame(spectra.line.table$Spectrum, empty.line.table)
            colnames(hold.frame) <- c("Spectrum", elements)
            
            hold.frame <- as.data.frame(hold.frame)
            
            
            value.frame <- calFileContents()$Values
            
            #anna <- rbind(hold.frame, value.frame)
            
            #temp.table <- data.table(anna)[,list(result = sum(result)), elements]
            
            #as.data.frame(temp.table)
            
            #element.matches <- elements[elements %in% ls(value.frame)]
            
            #merge_Sum(.df1=hold.frame, .df2=value.frame, .id_Columns="Spectrum",  .match_Columns=element.matches)
            
            
            #data.frame(calFileContents()$Values, hold.frame[,! names(hold.frame) %in% names(calFileContents()$Values)])
            
            hold.frame.reduced <- hold.frame[2:length(hold.frame)]
            value.frame.reduced <- if(colnames(calFileContents()$Values)[1]=="Spectrum"){
                value.frame[2:length(value.frame)]
            }else if(colnames(calFileContents()$Values)[1]=="Include"){
                value.frame[3:length(value.frame)]
            }
            
            rownames(hold.frame.reduced) <- hold.frame$Spectrum
            rownames(value.frame.reduced) <- value.frame$Spectrum
            
            
            hotable.new = hold.frame.reduced %>% add_rownames %>%
            full_join(value.frame.reduced %>% add_rownames) %>%
            group_by(rowname) %>%
            summarise_all(funs(sum(., na.rm = FALSE)))
            
            colnames(hotable.new)[1] <- "Spectrum"
            
            hotable.new$Spectrum <- gsub(".pdz", "", hotable.new$Spectrum)
            hotable.new$Spectrum <- gsub(".csv", "", hotable.new$Spectrum)
            hotable.new$Spectrum <- gsub(".CSV", "", hotable.new$Spectrum)
            hotable.new$Spectrum <- gsub(".spt", "", hotable.new$Spectrum)
            hotable.new$Spectrum <- gsub(".mca", "", hotable.new$Spectrum)
            hotable.new$Spectrum <- gsub(".spx", "", hotable.new$Spectrum)
            
            hotable.new
            
            
        })
        
        hotableInput <- reactive({
            
            
            hotable.data <- if(input$usecalfile==FALSE){
                hotableInputBlank()
            }else if(input$usecalfile==TRUE){
                hotableInputCal()
            }
            
            
            
            hotable.new <- if(input$usecalfile==FALSE){
                data.frame(Include=rep(TRUE, length(hotable.data$Spectrum)), hotable.data)
            }else if(input$usecalfile==TRUE && colnames(calFileContents()$Values)[1]=="Spectrum"){
                data.frame(Include=rep(TRUE, length(hotable.data$Spectrum)), hotable.data)
            }else if(input$usecalfile==TRUE && colnames(calFileContents()$Values)[1]=="Include"){
                data.frame(Include=calFileContents()$Values[,1], hotable.data)
            }
            
            
            
        })
        
        
        
        
        
        values <- reactiveValues()
        
        
        
        
        
        observe({
            if (!is.null(input$hot)) {
                DF <- hot_to_r(input$hot)
            } else {
                if (input$linecommit)
                DF <- hotableInput()
                else
                DF <- values[["DF"]]
            }
            values[["DF"]] <- DF
        })
        
        eventReactive(input$linecommit,{
            
            values[["DF"]] <- hotableInput()
            
        })
        
        
        ## Handsontable
        
        output$hot <- renderRHandsontable({
            
            DF <- values[["DF"]]
            
            DF <- DF[order(as.character(DF$Spectrum)),]
            
            
            
            if (!is.null(DF))
            rhandsontable(DF) %>% hot_col(2:length(DF), renderer=htmlwidgets::JS("safeHtmlRenderer"))
            
            
        })
        
        
        observeEvent(input$resethotable, {
            
            values[["DF"]] <- NULL
            
            values[["DF"]] <- hotableInput()
            
            
        })
        
        output$covarianceplotvalues <- renderPlot({
            
            data.table <- values[["DF"]]
            correlations <- cor(data.table[,3:length(data.table)], use="pairwise.complete.obs")
            corrplot(correlations, method="circle")
            
        })
        
        
        
        
        
        
        
        keepRowsFrame <- reactive({
            
            spectra.stuff <- values[["DF"]]
            rows <- vals$keeprows
            
            the.frame <- data.frame(Spectrum=spectra.stuff$Spectrum, Standards=rows)
            the.frame
            
        })
        
        
        output$whichrowstokeep <- renderRHandsontable({
            
            DF <- keepRowsFrame()
            
            DF <- DF[order(as.character(DF$Spectrum)),]
            
            
            
            if (!is.null(DF))
            rhandsontable(DF) %>% hot_col(2:length(DF), renderer=htmlwidgets::JS("safeHtmlRenderer"))
            
            
        })
        
        
        
        
        #if(input$hotableprocess2){vals$keeprows <- vals$keeprows[dropStandard()]}
        
        
        output$temp <- renderTable({
            
            as.data.frame(vals$keeprows)
            
        })
        
        
        dataType <- reactive({
            if(input$filetype=="CSV"){
                "Spectra"
            } else if(input$filetype=="TXT"){
                "Spectra"
            } else if(input$filetype=="Elio"){
                "Spectra"
            }  else if(input$filetype=="MCA"){
                "Spectra"
            }  else if(input$filetype=="SPX"){
                "Spectra"
            }  else if(input$filetype=="PDZ"){
                "Spectra"
            } else if (input$filetype=="Net"){
                "Net"
            }
            
        })
        
        
        
        concentrationTable <- reactive({
            
            concentration.table <- as.data.frame(values[["DF"]], stringsAsFactors=FALSE)
            concentration.table[concentration.table==""] <- NA
            concentration.table[values[["DF"]]$Include,]
            
        })
        
        spectraLineTable <- reactive({
            
            spectra.line.table <- if(dataType()=="Spectra"){
                spectraData()[values[["DF"]]$Include,]
            }else if(dataType()=="Net"){
                dataHold()[values[["DF"]]$Include,]
            }
            
            
            spectra.line.table <- spectra.line.table[order(as.character(spectra.line.table$Spectrum)),]
            spectra.line.table
            
            
            
        })
        
        
        holdFrame <- reactive({
            
            spectra.line.table <- spectraLineTable()
            
            concentration <- as.vector(as.numeric(unlist(concentrationTable()[,input$calcurveelement])))
            
            
            
            intensity <- as.vector(as.numeric(unlist(spectraLineTable()[,input$calcurveelement])))
            
            spectra.names <- spectra.line.table$Spectrum
            
            hold.frame <- data.frame(spectra.names, concentration, intensity)
            colnames(hold.frame) <- c("Spectrum", "Concentration", "Intensity")
            hold.frame <- na.omit(hold.frame)
            
            hold.frame <- hold.frame[order(as.character(hold.frame$Spectrum)),]
            
            
            hold.frame[complete.cases(hold.frame),]
            
            
        })
        
        dataNorm <- reactive({
            
            data <- dataHold()
            data[data$Spectrum %in% holdFrame()$Spectrum, ]
            
            
        })
        
        # randomInterList <- reactive({
        #   if (is.null(input$intercept_vars))
        #   paste(,2)
        #   else
        #   input$intercept_vars
        #})
        
        
        #randomSlopeList <- reactive({
        #   if (is.null(input$intercept_vars))
        #   paste(,2)
        #   else
        #   input$slope_vars
        #})
        
        #output$nullintercept <- randomInterList()
        
        #output$nullslope <- randomSlopeList()
        
        
        
        
        
        #####Set Defaults
        
        
        calConditons <- reactiveValues()
        calList <- reactiveValues()
        calList <- NULL
        
        observeEvent(input$hotableprocess2, {
            
            cal.condition <- 3
            norm.condition <- 1
            
            norm.min <- 18.5
            norm.max <- 19.5
            
            cal.table <- data.frame(cal.condition, norm.condition, norm.min, norm.max)
            colnames(cal.table) <- c("CalType", "NormType", "Min", "Max")
            
            slope.corrections <- input$slope_vars
            intercept.corrections <- input$intercept_vars
            
            standards.used <- vals$keeprows
            
            cal.mode.list <- list(cal.table, slope.corrections, intercept.corrections, standards.used)
            names(cal.mode.list) <- c("CalTable", "Slope", "Intercept", "StandardsUsed")
            
            calConditons <<- cal.mode.list
            
        })
        
        
        outVar <- reactive({
            input$hotableprocess2
            
            myelements <- elementallinestouse()
            
            result <- if(is.null(myelements)){
                "Ca.K.alpha"
            }else{
                myelements
            }
            
            result
            
            
        })
        
        outVaralt <- reactive({
            input$hotableprocess2
            
            
            myelements <- c(elementallinestouse())
            
            
            if(is.null(myelements)){
                paste("Ca.K.alpha")
            }else{
                myelements
            }
            
        })
        
        outVaralt2 <- reactive({
            input$hotableprocess2
            
            
            myelements <- c(elementallinestouse())
            
            
            if(is.null(myelements)){
                paste("Ca.K.alpha")
            }else{
                myelements[! myelements %in% c(input$calcurveelement)]
            }
            
        })
        
        output$inVar2 <- renderUI({
            selectInput(inputId = "calcurveelement", label = h4("Element"), choices =  outVar())
        })
        
        inVar3Selectedpre <- reactive({
            
            hold <- values[["DF"]]
            
            optionhold <- if(is.null(input$calcurveelement)){
                ls(hold)[2]
            }else{
                input$calcurveelement
            }
            
            if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==TRUE){
                optionhold
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE  && is.null(calFileContents()$calList[[optionhold]])==FALSE){
                calFileContents()$calList[[optionhold]][[1]]$Intercept
            } else if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$Intercept
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$Intercept
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE && is.null(calFileContents()$calList[[optionhold]])==TRUE){
                optionhold
            }
            
            
        })
        
        
        
        
        inVar4Selectedpre <- reactive({
            
            hold <- values[["DF"]]
            
            optionhold <- if(is.null(input$calcurveelement)){
                ls(hold)[2]
            }else{
                input$calcurveelement
            }
            
            
            if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==TRUE){
                optionhold
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE && is.null(calFileContents()$calList[[optionhold]])==FALSE){
                calFileContents()$calList[[optionhold]][[1]]$Slope
            } else if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$Slope
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$Slope
            }  else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE && is.null(calFileContents()$calList[[optionhold]])==TRUE){
                optionhold
            }
        })
        
        
        
        
        ########Machine Learning: Normalization
        
        normMinPre <- reactive({
            
            hold <- values[["DF"]]
            
            optionhold <- if(is.null(input$calcurveelement)){
                ls(hold)[2]
            }else{
                input$calcurveelement
            }
            
            if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==TRUE){
                calConditons[["CalTable"]][["Min"]]
            }else if(input$usecalfile==TRUE && is.null(calFileContents()$calList[[optionhold]])==FALSE && is.null(calList[[optionhold]])==TRUE){
                calFileContents()$calList[[optionhold]][[1]]$CalTable$Min
            } else if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$CalTable$Min
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$CalTable$Min
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE && is.null(calFileContents()$calList[[optionhold]])==FALSE){
                calConditons[["CalTable"]][["Min"]]
            }
            
        })
        
        normMaxPre <- reactive({
            
            hold <- values[["DF"]]
            
            optionhold <- if(is.null(input$calcurveelement)){
                ls(hold)[2]
            }else{
                input$calcurveelement
            }
            
            if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==TRUE){
                calConditons[["CalTable"]][["Max"]]
            }else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE && is.null(calFileContents()$calList[[optionhold]])==FALSE){
                calFileContents()$calList[[optionhold]][[1]]$CalTable$Max
            } else if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$CalTable$Max
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$CalTable$Max
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE && is.null(calFileContents()$calList[[optionhold]])==TRUE){
                calConditons[["CalTable"]][["Max"]]
            }
        })
        
        
        
        planktonVector <- reactive({
            
            #0.7, 0.9
            #2.5, 2.8
            #11.0, 11.2
            #18.4, 19.4
            #19.5, 22
            #21, 22
            #30, 35
            #35, 40
            
            mins <- c(0.7, 2.5, 11.0, 18.4, 19.5, 21, 30, 35)
            maxs <- c(0.9, 2.8, 11.2, 19.4, 22, 22, 35, 40)
            
            norm.list <- list(mins, maxs)
            names(norm.list) <- c("Min", "Max")
            norm.list
            
            
        })
        
        bestNormVars <- reactive({
            
            norm.list <- planktonVector()
            
            element <- input$calcurveelement
            
            choices <- elementallinestouse()
            spectra.line.table <- spectraLineTable()
            data <- dataNorm()
            concentration.table <- concentrationTable()
            
            index <- seq(1, length(norm.list[[1]]), 1)
            
            
            concentration.table <- concentration.table[complete.cases(concentration.table[,input$calcurveelement]),]
            
            
            
            spectra.line.table <- spectraLineTable()[spectraLineTable()$Spectrum %in% holdFrame()$Spectrum, ]
            
            #spectra.line.table <- spectra.line.table[spectra.line.table$Spectrum %in% concentration.table$Spectrum, ]
            
            spectra.line.table <- spectra.line.table[complete.cases(concentration.table[, element]),]
            
            data <- data[data$Spectrum %in% concentration.table$Spectrum, ]
            
            
            time.bic <- if(dataType()=="Spectra"){
                extractAIC(lm(concentration.table[, input$calcurveelement]~general_prep_xrf(spectra.line.table, input$calcurveelement)$Intensity, na.action=na.exclude), k=log(length(1)))[2]
            } else if(dataType()=="Net"){
                extractAIC(lm(concentration.table[, input$calcurveelement]~general_prep_xrf_net(spectra.line.table, input$calcurveelement)$Intensity, na.action=na.exclude), k=log(length(1)))[2]
            }
            
            tc.bic <- if(dataType()=="Spectra"){
                extractAIC(lm(concentration.table[, input$calcurveelement]~simple_tc_prep_xrf(data, spectra.line.table, input$calcurveelement)$Intensity, na.action=na.exclude), k=log(length(1)))[2]
            } else if(dataType()=="Net"){
                extractAIC(lm(concentration.table[, input$calcurveelement]~simple_tc_prep_xrf_net(data, spectra.line.table, input$calcurveelement)$Intensity, na.action=na.exclude), k=log(length(1)))[2]
            }
            
            comp.bic <- if(dataType()=="Spectra"){
                optimal_norm_chain_xrf(data=data, element=element, spectra.line.table=spectra.line.table, values=concentration.table, possible.mins=norm.list[["Min"]], possible.maxs=norm.list[["Max"]])
            } else if(dataType()=="Net"){
                time.bic
            }
            
            norm.chain <- c(time.bic, tc.bic, comp.bic)
            type.chain <- c(1, 2, 3)
            
            best <- index[[which.min(unlist(norm.chain))]]
            best.comp <- c(planktonVector()[["Min"]][best], planktonVector()[["Max"]][best])
            best.type <- type.chain[which.min(unlist(norm.chain))]
            result.list <- list(best.type, best.comp)
            names(result.list) <- c("Type", "Compton")
            result.list
        })
        
        
        normhold <- reactiveValues()
        
        observeEvent(input$hotableprocess2, {
            normhold$norms <- c(normMinPre(), normMaxPre())
            normhold$normtype <- calNormSelectionpre()
        })
        
        
        observeEvent(input$trainslopes, {
            
            isolate(normhold$norms[1] <- bestNormVars()[["Compton"]][1])
            isolate(normhold$norms[2] <- bestNormVars()[["Compton"]][2])
            isolate(normhold$normtype <- bestNormVars()[["Type"]])
            
        })
        
        calNormSelection <- reactive({
            normhold$normtype
        })
        
        normMinSelection <- reactive({
            normhold$norms[1]
        })
        
        normMaxSelection <- reactive({
            normhold$norms[2]
        })
        
        
        
        
        calNormSelectionpre <- reactive({
            
            hold <- values[["DF"]]
            
            optionhold <- if(is.null(input$calcurveelement)){
                ls(hold)[2]
            }else{
                input$calcurveelement
            }
            
            if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==TRUE){
                calConditons[["CalTable"]][["NormType"]]
            }else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE && is.null(calFileContents()$calList[[optionhold]])==FALSE){
                calFileContents()$calList[[optionhold]][[1]]$CalTable$NormType
            } else if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$CalTable$NormType
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$CalTable$NormType
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE && is.null(calFileContents()$calList[[optionhold]])==TRUE){
                calConditons[["CalTable"]][["NormType"]]
            }
            
        })
        
        
        output$normTypeInput <- renderUI({
            
            selectInput("normcal", label = "Normalization",
            choices = list("Time" = 1, "Total Counts" = 2, "Compton" = 3),
            selected = calNormSelection())
            
            
        })
        
        
        output$comptonMinInput <- renderUI({
            
            numericInput('comptonmin', label=h6("Min"), step=0.001, value=normMinSelection(), min=0, max=50, width='30%')
            
        })
        
        output$comptonMaxInput <- renderUI({
            
            numericInput('comptonmax', label=h6("Max"), step=0.001, value=normMaxSelection(), min=0, max=50, width='30%')
            
        })
        
        
        #####Machine Learning: Intercepts
        
        
        
        cephlopodVector <- reactive({
            
            combos_mod.xrf <- function(a.vector){
                
                so <- seq(from=1, to=length(a.vector), by=1)
                
                long <- pblapply(so, function(x) gRbase::combnPrim(x=a.vector, m=x), cl=6L)
                and <- pblapply(long, function(x) plyr::alply(x, 2), cl=6L)
                thanks.for.all.the.fish <- do.call(list, unlist(and, recursive=FALSE))
                
                thanks.for.all.the.fish
                
            }
            
            if(!is.null(likely_intercepts(input$calcurveelement))){
                combos_mod.xrf(likely_intercepts(input$calcurveelement))
            } else if(is.null(likely_intercepts(input$calcurveelement))){
                c("Rh.K.alpha", "Rh.L.alpha")
            }
            
        })
        
        
        bestInterceptVars <- reactive({
            
            element <- input$calcurveelement
            
            choices <- elementallinestouse()
            
            spectra.line.table <- if(all(cephlopodVector() %in% colnames(spectraLineTable()))==TRUE){
                spectraLineTable()
            } else if(all(cephlopodVector() %in% colnames(spectraLineTable()))==FALSE){
                merge(spectraLineTable(), elementFrame(data=dataHold(), elements=cephlopodVector()[cephlopodVector() %in% colnames(spectraLineTable())]))
            }
            
            data <- dataNorm()
            concentration.table <- concentrationTable()
            
            
            spectra.line.table <- spectra.line.table[spectra.line.table$Spectrum %in% holdFrame()$Spectrum, ]
            
            
            predict.intensity.list <- if(input$normcal==1){
                pblapply(cephlopodVector(), function(x) lucas_simp_prep_xrf(spectra.line.table=spectra.line.table, element.line=element, slope.element.lines=element, intercept.element.lines=c(element, x)))
            } else if(input$normcal==2){
                pblapply(cephlopodVector(), function(x) lucas_tc_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=element, slope.element.lines=element, intercept.element.lines=c(element, x)))
            } else if(input$normcal==3){
                pblapply(cephlopodVector(), function(x) lucas_comp_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=element, slope.element.lines=element, intercept.element.lines=c(element, x), norm.min=input$comptonmin, norm.max=input$comptonmax))
            }
            
            optimal_intercept_chain_xrf(element=element, intensities=predict.intensity.list, values=concentration.table, keep=vals$keeprows)
            
            
        })
        
        
        intercepthold <- reactiveValues()
        intercepthold$intercepts <- NULL
        
        observeEvent(input$calcurveelement, {
            
            isolate(intercepthold$intercepts <- inVar3Selectedpre())
            
        })
        
        
        #observeEvent(input$trainslopes, {
        
        #    isolate(intercepthold$intercepts <- bestInterceptVars())
        
        #})
        
        
        inVar3Selected <- reactive({
            
            intercepthold$intercepts
            
            
        })
        
        output$inVar3 <- renderUI({
            
            selectInput(inputId = "intercept_vars", label = h4("Intercept"), choices =  outVaralt2(), selected=inVar3Selected(), multiple=TRUE)
        })
        
        
        
        ####Machine Learning: Slopes
        
        caretSlope <- reactive({
            
            element <- input$calcurveelement
            
            # prepare simple test suite
            control <- trainControl(method="cv", number=5)
            seed <- 7
            metric <- "RMSE"
            set.seed(seed)
            
            data <- dataNorm()
            concentration.table <- concentrationTable()
            
            concentration.table <- concentration.table[complete.cases(concentration.table[,input$calcurveelement]),]
            
            
            
            #spectra.line.table <- spectraLineTable()[spectraLineTable()$Spectrum %in% holdFrame()$Spectrum, ]
            
            spectra.line.table <- spectraLineTable()[spectraLineTable()$Spectrum %in% concentration.table$Spectrum, ]
            
            #spectra.line.table <- spectraLineTable()[complete.cases(concentration.table[, element]),]
            
            data <- data[data$Spectrum %in% concentration.table$Spectrum, ]
            
            
            cal.table <- if(dataType()=="Spectra"){
                if(input$normcal==1){
                    lucas_simp_prep_xrf(spectra.line.table=spectra.line.table, element.line=input$calcurveelemenet, slope.element.lines=colnames(spectra.line.table[,-1]), intercept.element.lines=input$intercept_vars)
                } else if(input$normcal==2){
                    lucas_tc_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=colnames(spectra.line.table[,-1]), intercept.element.lines=input$intercept_vars)
                } else if(input$normcal==3){
                    lucas_comp_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=colnames(spectra.line.table[,-1]), intercept.element.lines=input$intercept_vars, norm.min=input$comptonmin, norm.max=input$comptonmax)
                }
            } else if(dataType()=="Net"){
                if(input$normcal==1){
                    lucas_simp_prep_xrf_net(spectra.line.table=spectra.line.table, element.line=input$calcurveelemenet,slope.element.lines=colnames(spectra.line.table[,-1]), intercept.element.lines=input$intercept_vars)
                } else if(input$normcal==2){
                    lucas_tc_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=colnames(spectra.line.table[,-1]), intercept.element.lines=input$intercept_vars)
                } else if(input$normcal==3){
                    lucas_comp_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=colnames(spectra.line.table[,-1]), intercept.element.lines=input$intercept_vars, norm.min=input$comptonmin, norm.max=input$comptonmax)
                }
            }
            
            #cal.table <- cal.table[,!colnames(cal.table) %in% "Intensity"]
            cal.table$Concentration <- concentration.table[,input$calcurveelement]
            
            
            train(Concentration~., data=cal.table[,-1], method="lm", metric=metric, preProc=c("center", "scale"), trControl=control)
            
            
        })
        
        
        
        slopeImportance <- reactive({
            
            varImp(caretSlope(), scale=FALSE)
            
        })
        
        
        rainForestImportance <- reactive({
            
            
            as.data.frame(importance(elementModel()))
            
        })
        
        
        importanceFrame <- reactive({
            
            importance.frame <- rainForestImportance()
            colnames(importance.frame) <- c("NodePurity")
            importance.frame$Energy <- as.numeric(gsub("X", "", rownames(importance.frame)))
            importance.frame
            
        })
        
        
        
        importanceranges <- reactiveValues(x = NULL, y = NULL)
        
        
        
        
        
        
        
        rainForestImportancePlot <- reactive({
            
            importance.frame <- importanceFrame()
            
            
            element <- datasetInputVar()
            intensity.norm <- (element$Intensity/max(element$Intensity))*max(importance.frame$NodePurity)
            intensity.base <- (element$Intensity/max(element$Intensity))
            
            ggplot(importance.frame) +
            geom_area(aes(Energy, NodePurity), fill="grey80", alpha=0.8) +
            geom_line(aes(Energy, NodePurity)) +
            geom_segment(data=element, aes(x=Line, xend=Line, y = 0, yend=intensity.norm), colour="grey50", linetype=2)  +
            theme_light() +
            scale_x_continuous("Energy (keV)") +
            coord_cartesian(xlim = importanceranges$x, ylim = importanceranges$y, expand = TRUE)
            
            
        })
        
        # When a double-click happens, check if there's a brush on the plot.
        # If so, zoom to the brush bounds; if not, reset the zoom.
        observeEvent(input$cropvar, {
            data <- dataHold()
            brush <- input$plot_var_brush
            if (!is.null(brush)) {
                importanceranges$x <- c(brush$xmin, brush$xmax)
                importanceranges$y <- c(brush$ymin, brush$ymax)
                
            } else {
                importanceranges$x <- NULL
                importanceranges$y <- NULL
            }
            
            
            
        })
        
        variablesPlot <- reactive({
            
            if(calType()!=5){
                plot(slopeImportance())
            } else if(calType()==5){
                rainForestImportancePlot()
            }
            
        })
        
        
        
        
        output$varelementui <- renderUI({
            
            selectInput(
            "elementvar", "Element:",
            choices=c("(Ne) Neon" = "Ne",
            "(Na) Sodium" = "Na",
            "(Mg) Magnesium" = "Mg",
            "(Al) Aluminum" = "Al",
            "(Si) Silicon" = "Si",
            "(P)  Phosphorous" = "P",
            "(S)  Sulfur" = "S",
            "(Cl) Chlorine" = "Cl",
            "(Ar) Argon" = "Ar",
            "(K)  Potassium" = "K",
            "(Ca) Calcium" = "Ca",
            "(Sc) Scandium" = "Sc",
            "(Ti) Titanium" = "Ti",
            "(Cr) Chromium" = "Cr",
            "(Mn) Manganese" = "Mn",
            "(Fe) Iron" = "Fe",
            "(Co) Cobalt" = "Co",
            "(Ni) Nickel" = "Ni",
            "(Cu) Copper" = "Cu",
            "(Zn) Zinc"= "Zn",
            "(Ga) Gallium" = "Ga",
            "(Ge) Germanium" = "Ge",
            "(As) Arsenic" = "As",
            "(Se) Selenium" = "Se",
            "(Br) Bromium" = "Br",
            "(Kr) Krypton" = "Kr",
            "(Rb) Rubidium" = "Rb",
            "(Sr) Strontium" = "Sr",
            "(Y)  Yttrium" = "Y",
            "(Zr) Zirconium" = "Zr",
            "(Nb) Niobium" = "Nb",
            "(Mo) Molybdenum" = "Mo",
            "(Tc) Technicium" = "Tc",
            "(Ru) Ruthenium" = "Ru",
            "(Rh) Rhodium" = "Rh",
            "(Pd) Paladium" = "Pd",
            "(Ag) Silver" = "Ag",
            "(Cd) Cadmium" = "Cd",
            "(In) Indium" = "In",
            "(Sn) Tin" = "Sn",
            "(Sb) Antimony" = "Sb",
            "(Te) Tellerium" = "Te",
            "(I) Iodine" = "I",
            "(Xe) Xenon" = "Xe",
            "(Cs) Cesium" = "Cs",
            "(Ba) Barium" = "Ba",
            "(Ce) Cerium" = "Ce",
            "(Pr) Praeseodymeum" = "Pr",
            "(Nd) Neodymeum" = "Nd",
            "(Pr) Promethium" = "Pr",
            "(Sm) Samarium" = "Sm",
            "(Eu) Europium" = "Eu",
            "(Gd) Gadolinium" = "Gd",
            "(Tb) Terbium" = "Tb",
            "(Dy) Dysprosium" = "Dy",
            "(Ho) Holmium" = "Ho",
            "(Er) Erbium" = "Er",
            "(Tm) Thullium" = "Tm",
            "(Yb) Ytterbium" = "Yb",
            "(Lu) Lutetium" = "Lu",
            "(Hf) Halfnium" = "Hf",
            "(Ta) Tantalum" = "Ta",
            "(W)  Tungsten" = "W",
            "(Re) Rhenium" = "Re",
            "(Os) Osmium" = "Os",
            "(Ir) Irridium" = "Ir",
            "(Pt) Platinum" = "Pt",
            "(Au) Gold" = "Au",
            "(Tl) Thallium" = "Tl",
            "(Pb) Lead" = "Pb",
            "(Bi) Bismuth" = "Bi",
            "(Po) Polonium" = "Po",
            "(At) Astatine" = "At",
            "(Rn) Radon" = "Rn",
            "(Fr) Francium" = "Fr",
            "(Ra) Radium" = "Ra",
            "(Ac) Actinum" = "Ac",
            "(Th) Thorium" = "Th",
            "(Pa) Proactinum" = "Pa",
            "(U)  Uranium" = "U"),
            selected=strsplit(x=input$calcurveelement, split="\\.")[[1]][1])
            
        })
        
        
        
        # Return the requested dataset
        datasetInputVar <- reactive({
            switch(input$elementvar,
            "H" = H.table,
            "He" = He.table,
            "Li" = Li.table,
            "Be" = Be.table,
            "B" = B.table,
            "C" = C.table,
            "N" = N.table,
            "O" = O.table,
            "F" = F.table,
            "Ne" = Ne.table,
            "Na" = Na.table,
            "Mg" = Mg.table,
            "Al" = Al.table,
            "Si" = Si.table,
            "P" = P.table,
            "S" = S.table,
            "Cl" = Cl.table,
            "Ar" = Ar.table,
            "K" = K.table,
            "Ca" = Ca.table,
            "Sc" = Sc.table,
            "Ti" = Ti.table,
            "V" = V.table,
            "Cr" = Cr.table,
            "Mn" = Mn.table,
            "Fe" = Fe.table,
            "Co" = Co.table,
            "Ni" = Ni.table,
            "Cu" = Cu.table,
            "Zn" = Zn.table,
            "Ga" = Ga.table,
            "Ge" = Ge.table,
            "As" = As.table,
            "Se" = Se.table,
            "Br" = Br.table,
            "Kr" = Kr.table,
            "Rb" = Rb.table,
            "Sr" = Sr.table,
            "Y" = Y.table,
            "Zr" = Zr.table,
            "Nb" = Nb.table,
            "Mo" = Mo.table,
            "Tc" = Tc.table,
            "Ru" = Ru.table,
            "Rh" = Rh.table,
            "Pd" = Pd.table,
            "Ag" = Ag.table,
            "Cd" = Cd.table,
            "In" = In.table,
            "Sn" = Sn.table,
            "Sb" = Sb.table,
            "Te" = Te.table,
            "I" = I.table,
            "Xe" = Xe.table,
            "Cs" = Cs.table,
            "Ba" = Ba.table,
            "La" = La.table,
            "Ce" = Ce.table,
            "Pr" = Pr.table,
            "Nd" = Nd.table,
            "Pm" = Pm.table,
            "Sm" = Sm.table,
            "Eu" = Eu.table,
            "Gd" = Gd.table,
            "Tb" = Tb.table,
            "Dy" = Dy.table,
            "Ho" = Ho.table,
            "Er" = Er.table,
            "Tm" = Tm.table,
            "Yb" = Yb.table,
            "Lu" = Lu.table,
            "Hf" = Hf.table,
            "Ta" = Ta.table,
            "W" = W.table,
            "Re" = Re.table,
            "Os" = Os.table,
            "Ir" = Ir.table,
            "Pt" = Pt.table,
            "Au" = Au.table,
            "Hg" = Hg.table,
            "Tl" = Tl.table,
            "Pb" = Pb.table,
            "Bi" = Bi.table,
            "Po" = Po.table,
            "At" = At.table,
            "Rn" = Rn.table,
            "Fr" = Fr.table,
            "Ra" = Ra.table,
            "Ac" = Ac.table,
            "Th" = Th.table,
            "Pa" = Pa.table,
            "U" = U.table)
        })
        
        
        output$importanceplot <- renderPlot({
            
            variablesPlot()
            
        })
        
        
        output$hover_info_variable <- renderUI({
            if(calType()==5){
                
                point.table <- importanceFrame()
                
                hover <- input$plot_hover_variable
                point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
                if (nrow(point) == 0) return(NULL)
                
                # calculate point position INSIDE the image as percent of total dimensions
                # from left (horizontal) and from top (vertical)
                left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
                top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
                
                # calculate distance from left and bottom side of the picture in pixels
                left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
                top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
                
                
                # create style property fot tooltip
                # background color is set so tooltip is a bit transparent
                # z-index is set so we are sure are tooltip will be on top
                style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
                "left:", left_px + 2, "px; top:", top_px + 2, "px;")
                
                # actual tooltip created as wellPanel
                wellPanel(
                style = style,
                p(HTML(paste0("Energy:", " ", round(point$Energy, 0)))),
                p(HTML(paste0("NodePurity:", " ", round(point$NodePurity, 1))))
                )
            } else if(calType()!=5){
                NULL
            }
        })
        
        output$variablePlot <- downloadHandler(
        filename = function() { paste0(input$calname, "_", input$calcurveelement, '_Variables', '.tiff', sep='') },
        content = function(file) {
            ggsave(file,variablesPlot(), width=14, height=8, device="tiff", compression="lzw", type="cairo", dpi=300)
        }
        )
        
        
        
        
        
        fishVector <- reactive({
            
            combos_mod_xrf <- function(a.vector){
                
                so <- seq(from=2, to=input$nvariables, by=1)
                
                long <- pblapply(so, function(x) gRbase::combnPrim(x=a.vector, m=x), cl=6L)
                and <- pblapply(long, function(x) plyr::alply(x, 2), cl=6L)
                thanks.for.all.the.fish <- do.call(list, unlist(and, recursive=FALSE))
                thanks.for.all.the.fish <- pblapply(thanks.for.all.the.fish, function(x) c(input$calcurveelement, x))
                
                thanks.for.all.the.fish
                
            }
            
            fit.lm <- caretSlope()
            
            first.combos <- c(elementallinestouse()[!elementallinestouse() %in% input$calcurveelement])
            
            coef.frame <- as.data.frame(summary(fit.lm)$coefficients)
            sig.frame <- subset(coef.frame, coef.frame[,4] < 0.05)
            
            second.combos <- first.combos[c(first.combos %in% rownames(sig.frame)[c(!rownames(sig.frame) %in% "(Intercept)")])]
            
            
            
            combos_mod(second.combos)
            
            
        })
        
        
        bestSlopeVars <- reactive({
            
            element <- input$calcurveelement
            
            choices <- elementallinestouse()
            spectra.line.table <- spectraLineTable()
            data <- dataNorm()
            concentration.table <- concentrationTable()
            
            #concentration.table[complete.cases(concentration.table[,input$calcurveelement]),]
            
            #index <- complete.cases(concentration.table[,input$calcurveelement])
            
            
            spectra.line.table <- spectraLineTable()[spectraLineTable()$Spectrum %in% holdFrame()$Spectrum, ]
            
            spectra.line.table <- spectra.line.table[spectra.line.table$Spectrum %in% concentration.table$Spectrum, ]
            
            #spectra.line.table <- spectra.line.table[complete.cases(concentration.table[, element]),]
            
            data <- data[data$Spectrum %in% concentration.table$Spectrum, ]
            
            
            
            
            predict.intensity <- if(input$normcal==1){
                if(dataType()=="Spectra"){
                    lucas_simp_prep_xrf(spectra.line.table=spectra.line.table, element.line=element, slope.element.lines=choices, intercept.element.lines=input$intercept_vars)
                } else if(dataType()=="Net"){
                    lucas_simp_prep_xrf_net(spectra.line.table=spectra.line.table, element.line=element, slope.element.lines=choices, intercept.element.lines=input$intercept_vars)
                }
            } else if(input$normcal==2){
                predict.intensity <- if(dataType()=="Spectra"){
                    lucas_tc_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=element, slope.element.lines=choices, intercept.element.lines=input$intercept_vars)
                } else if(dataType()=="Net"){
                    lucas_tc_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=element, slope.element.lines=choices, intercept.element.lines=input$intercept_vars)
                }
            } else if(input$normcal==3){
                predict.intensity <- if(dataType()=="Spectra"){
                    lucas_comp_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=element, slope.element.lines=choices, intercept.element.lines=input$intercept_vars, norm.min=input$comptonmin, norm.max=input$comptonmax)
                } else if(dataType()=="Net"){
                    lucas_comp_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=element, slope.element.lines=choices, intercept.element.lines=input$intercept_vars, norm.min=input$comptonmin, norm.max=input$comptonmax)
                }
            }
            
            
            
            #optimal_r_chain(element=element, intensities=predict.intensity, values= concentration.table, possible.slopes=fishVector(), keep=vals$keeprows)
            
            results <- variable_select_short_xrf(slopeImportance())
            
            c(input$calcurveelement, results[!results %in% input$calcurveelement])
            
            
        })
        
        slopehold <- reactiveValues()
        slopehold$slopes <- NULL
        
        observeEvent(input$calcurveelement, {
            
            isolate(slopehold$slopes <- inVar4Selectedpre())
            
        })
        observeEvent(input$trainslopes, {
            
            isolate(slopehold$slopes <- bestSlopeVars())
            
        })
        
        
        inVar4Selected <- reactive({
            
            slopehold$slopes
            
            
        })
        
        
        
        output$inVar4 <- renderUI({
            
            selectInput(inputId = "slope_vars", label = h4("Slope"), choices =  outVaralt(), selected=inVar4Selected(), multiple=TRUE)
        })
        
        
        
        ####Set up Standards
        elementHold <- reactive({
            
            if(is.null(input$calcurveelement)==TRUE){
                ls(dataHold())[1]
            } else if(!is.null(input$calcurveelement)==TRUE){
                input$calcurveelement
            }
            
        })
        
        
        
        
        calFileStandards <- reactive({
            
            
            
            if(input$usecalfile==TRUE && is.null(calList[[elementHold()]])==TRUE && is.null(calFileContents()$calList[[elementHold()]])==FALSE){
                calFileContents()$calList[[elementHold()]][[1]][["StandardsUsed"]]
            } else if(input$usecalfile==FALSE && is.null(calList[[elementHold()]])==TRUE && is.null(calFileContents()$calList[[elementHold()]])==TRUE){
                rep(TRUE, dataCount())
            } else if(input$usecalfile==TRUE && is.null(calList[[elementHold()]])==FALSE && is.null(calFileContents()$calList[[elementHold()]])==TRUE){
                calList[[elementHold()]][[1]][["StandardsUsed"]]
            } else if(input$usecalfile==TRUE && is.null(calList[[elementHold()]])==FALSE && is.null(calFileContents()$calList[[elementHold()]])==FALSE){
                calList[[elementHold()]][[1]][["StandardsUsed"]]
            } else if(input$usecalfile==FALSE && is.null(calList[[elementHold()]])==FALSE && is.null(calFileContents()$calList[[elementHold()]])==TRUE){
                calList[[elementHold()]][[1]][["StandardsUsed"]]
            } else if(input$usecalfile==FALSE && is.null(calList[[elementHold()]])==FALSE && is.null(calFileContents()$calList[[elementHold()]])==FALSE){
                calList[[elementHold()]][[1]][["StandardsUsed"]]
            } else if(input$usecalfile==TRUE && is.null(calList[[elementHold()]])==TRUE && is.null(calFileContents()$calList[[elementHold()]])==TRUE){
                rep(TRUE, dataCount())
            }
            
            
            
            
        })
        
        
        
        
        vals <- reactiveValues()
        
        
        vals$keeprows <- if(input$usecalfile==TRUE){
            calFileStandards()
        }else{
            rep(TRUE, dataCount())
        }
        
        
        
        #####Machine Learning: Cal Type
        
        
        calTypeSelectionPre <- reactive({
            
            hold <- values[["DF"]]
            
            optionhold <- if(is.null(input$calcurveelement)){
                ls(hold)[2]
            }else{
                input$calcurveelement
            }
            
            if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==TRUE){
                calConditons[["CalTable"]][["CalType"]]
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE && is.null(calFileContents()$calList[[optionhold]])==FALSE){
                calFileContents()$calList[[optionhold]][[1]]$CalTable$CalType
            } else if(input$usecalfile==FALSE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$CalTable$CalType
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==FALSE){
                calList[[optionhold]][[1]]$CalTable$CalType
            } else if(input$usecalfile==TRUE && is.null(calList[[optionhold]])==TRUE && is.null(calFileContents()$calList[[optionhold]])==TRUE){
                calConditons[["CalTable"]][["CalType"]]
            }
            
        })
        
        
        predictIntensitySimp <- reactive({
            
            concentration.table <- concentrationTable()
            data <- dataNorm()
            spectra.line.table <- spectraLineTable()
            
            if(input$normcal==1){
                if(dataType()=="Spectra"){
                    general_prep_xrf(spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                } else if(dataType()=="Net"){
                    general_prep_xrf_net(spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                }
            } else if(input$normcal==2){
                predict.intensity <- if(dataType()=="Spectra"){
                    simple_tc_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                } else if(dataType()=="Net"){
                    simple_tc_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                }
            } else if(input$normcal==3){
                predict.intensity <- if(dataType()=="Spectra"){
                    simple_comp_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, norm.min=input$comptonmin, norm.max=input$comptonmax)
                } else if(dataType()=="Net"){
                    simple_comp_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, norm.min=input$comptonmin, norm.max=input$comptonmax)
                }
            }
            
        })
        
        
        predictFrameSimp <- reactive({
            
            concentration.table <- concentrationTable()
            data <- dataNorm()
            spectra.line.table <- spectraLineTable()
            
            predict.intensity.simp <- predictIntensitySimp()
            
            predict.frame.simp <- data.frame(predict.intensity.simp, concentration.table[,input$calcurveelement])
            predict.frame.simp <- predict.frame.simp[complete.cases(predict.frame.simp),]
            predict.frame.simp <- predict.frame.simp[vals$keeprows,]
            colnames(predict.frame.simp) <- c(names(predict.intensity.simp), "Concentration")
            predict.frame.simp <- predict.frame.simp[complete.cases(predict.frame.simp$Concentration),]
            
            predict.frame.simp
            
        })
        
        simpleLinearModel <- reactive({
            
            lm(Concentration~Intensity, data=predictFrameSimp(), na.action=na.exclude)
            
            
        })
        
        nonLinearModel <- reactive({
            
            lm(Concentration~Intensity + I(Intensity^2), data=predictFrameSimp(), na.action=na.exclude)
            
        })
        
        predictIntensityForest <- reactive({
            
            concentration.table <- concentrationTable()
            data <- dataNorm()
            spectra.line.table <- spectraLineTable()
            
            
            if(input$normcal==1){
                if(dataType()=="Spectra"){
                    lucas_simp_prep_xrf(spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars)
                } else if(dataType()=="Net"){
                    lucas_simp_prep_xrf_net(spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars)
                }
            } else if(input$normcal==2){
                predict.intensity <- if(dataType()=="Spectra"){
                    lucas_tc_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars)
                } else if(dataType()=="Net"){
                    lucas_tc_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars)
                }
            } else if(input$normcal==3){
                predict.intensity <- if(dataType()=="Spectra"){
                    lucas_comp_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars, norm.min=input$comptonmin, norm.max=input$comptonmax)
                } else if(dataType()=="Net"){
                    lucas_comp_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars, norm.min=input$comptonmin, norm.max=input$comptonmax)
                }
            }
            
            
        })
        
        
        predictFrameForest <- reactive({
            
            concentration.table <- concentrationTable()
            data <- dataNorm()
            spectra.line.table <- spectraLineTable()
            
            
            predict.intensity.forest <- predictIntensityForest()
            
            predict.frame.forest <- data.frame(predict.intensity.forest, Concentration=concentration.table[,input$calcurveelement])
            predict.frame.forest <- predict.frame.forest[complete.cases(predict.frame.forest),]
            predict.frame.forest <- predict.frame.forest[vals$keeprows,]
            predict.frame.forest <- predict.frame.forest[complete.cases(predict.frame.forest$Concentration),]
            
            predict.frame.forest
            
        })
        
        forestModel <- reactive({
            
            randomForest(Concentration~., data=predictFrameForest(), na.action=na.omit, ntree=1000, nPerm=100)
            
        })
        
        
        predictIntensityLuc <- reactive({
            
            
            predictIntensityForest()[,c("Intensity", input$slope_vars)]
            
        })
        
        predictFrameLuc <- reactive({
            
            concentration.table <- concentrationTable()
            data <- dataNorm()
            spectra.line.table <- spectraLineTable()
            
            
            predict.intensity.luc <- predictIntensityLuc()
            
            predict.frame.luc <- data.frame(predict.intensity.luc, concentration.table[,input$calcurveelement])
            predict.frame.luc <- predict.frame.luc[complete.cases(predict.frame.luc),]
            predict.frame.luc <- predict.frame.luc[vals$keeprows,]
            colnames(predict.frame.luc) <- c(names(predict.intensity.luc), "Concentration")
            predict.frame.luc <- predict.frame.luc[complete.cases(predict.frame.luc$Concentration),]
            
            predict.frame.luc
            
        })
        
        
        lucasToothModel <- reactive({
            
            lm(Concentration~., data=predictFrameLuc(), na.action=na.exclude)
        })
        
        
        rainforestData <- reactive({
            
            concentration.table <- concentrationTable()
            data <- dataNorm()
            spectra.line.table <- spectraLineTable()
            
            spectra.data <- if(input$normcal==1){
                if(dataType()=="Spectra"){
                    spectra_simp_prep_xrf(spectra=data)[,-1]
                } else if(dataType()=="Net"){
                    NULL
                }
            } else if(input$normcal==2){
                if(dataType()=="Spectra"){
                    spectra_tc_prep_xrf(spectra=data)[,-1]
                } else if(dataType()=="Net"){
                    NULL
                }
            } else if(input$normcal==3){
                if(dataType()=="Spectra"){
                    spectra_comp_prep_xrf(spectra=data, norm.min=input$comptonmin, norm.max=input$comptonmax)[,-1]
                } else if(dataType()=="Net"){
                    NULL
                }
            }
            
            spectra.data$Concentration <- concentration.table[complete.cases(concentration.table[,input$calcurveelement]),input$calcurveelement]
            spectra.data <- spectra.data[complete.cases(spectra.data$Concentration),]
            spectra.data <- spectra.data[vals$keeprows,]
            
            spectra.data
            
        })
        
        
        rainforestModel <- reactive({
            
            randomForest(Concentration~., data=rainforestData(), na.action=na.omit, ntree=1000, nPerm=100)
            
        })
        
        
        
        
        
        bestCalTypeFrame <- reactive({
            
            concentration.table <- concentrationTable()
            data <- dataNorm()
            spectra.line.table <- spectraLineTable()
            
            
            #concentration.table <- concentration.table[complete.cases(concentration.table[, input$calcurveelement]),]
            
            #spectra.line.table <- spectra.line.table[complete.cases(concentration.table[, input$calcurveelement]),]
            #data2 <- data[data$Spectrum %in% concentration.table$Spectrum, ]
            
            predict.intensity.simp <- predictIntensitySimp()
            
            predict.intensity.luc <- predictIntensityLuc()
            
            predict.intensity.forest <- predictIntensityForest()
            
            spectra.data <- rainforestData()
            spectra.data <- spectra.data[complete.cases(spectra.data),]
            
            spectra.data <- spectra.data[ , !(names(spectra.data) %in% "Concentration")]
            
            
            
            predict.frame.simp <- predictFrameSimp()
            predict.frame.forest <- predictFrameForest()
            predict.frame.luc <- predictFrameLuc()
            predict.frame.rainforest <- rainforestData()
            
            cal.lm.simp <- simpleLinearModel()
            lm.predict <- predict(cal.lm.simp, newdata=predict.frame.simp)
            lm.sum <- summary(lm(predict.frame.simp$Concentration~lm.predict, na.action=na.exclude))
            
            cal.lm.two <- nonLinearModel()
            lm2.predict <- predict(cal.lm.two, newdata=predict.frame.simp)
            lm2.sum <- summary(lm(predict.frame.simp$Concentration~lm2.predict, na.action=na.exclude))
            
            cal.lm.luc <- lucasToothModel()
            lucas.predict <- predict(cal.lm.luc, newdata=predict.frame.luc)
            lucas.sum <- summary(lm(predict.frame.luc$Concentration~lucas.predict, na.action=na.exclude))
            
            cal.lm.forest <- forestModel()
            forest.predict <- predict(cal.lm.forest, newdata=predict.frame.forest)
            forest.sum <- summary(lm(predict.frame.forest$Concentration~forest.predict, na.action=na.exclude))
            
            cal.lm.rainforest <- rainforestModel()
            rainforest.predict <- predict(cal.lm.rainforest, newdata=predict.frame.rainforest)
            rainforest.sum <- summary(lm(predict.frame.rainforest$Concentration~rainforest.predict, na.action=na.exclude))
            
            
            model.frame <- data.frame(Model = c("Linear", "Non-Linear", "Lucas-Tooth", "Forest", "Rainforest"),
            valSlope = round(c(lm.sum$coef[2], lm2.sum$coef[2], lucas.sum$coef[2], forest.sum$coef[2], rainforest.sum$coef[2]), 2),
            R2 = round(c(lm.sum$adj.r.squared, lm2.sum$adj.r.squared, lucas.sum$adj.r.squared, forest.sum$adj.r.squared, rainforest.sum$adj.r.squared), 2),
            Score = round(c(lm.sum$adj.r.squared*lm.sum$coef[2], lm2.sum$adj.r.squared*lm2.sum$coef[2], lucas.sum$adj.r.squared*lucas.sum$coef[2], forest.sum$adj.r.squared*forest.sum$coef[2], rainforest.sum$adj.r.squared*rainforest.sum$coef[2]), 2),
            Rank = round(abs(1-c(lm.sum$adj.r.squared*lm.sum$coef[2], lm2.sum$adj.r.squared*lm2.sum$coef[2], lucas.sum$adj.r.squared*lucas.sum$coef[2], forest.sum$adj.r.squared*forest.sum$coef[2], rainforest.sum$adj.r.squared*rainforest.sum$coef[2])), 2),
            Code=c(1, 2, 3, 4, 5))
            
            
            model.frame <- model.frame %>%  arrange(Rank)
            
            #model.frame[order(model.frame, model.frame$Rank),]
            model.frame
            
        })
        
        
        bestCalType <- reactive({
            
            model.frame <- bestCalTypeFrame()
            
            model.frame <- subset(model.frame, !(model.frame$valSlope < 0.8 | model.frame$valSlope > 1.2))
            model.frame[Closest(model.frame$Score, 1, which=TRUE),6]
            
        })
        
        
        output$models <- renderDataTable({
            
            bestCalTypeFrame()
            
        })
        
        
        testing2 <- reactive({
            
            predict.frame <- predictFrame()
            predict.frame <- predict.frame[complete.cases(predict.frame$Concentration),]
            
            predict.frame
            
        })
        
        
        testing <- reactive({
            predict.frame <- predictFrame()[complete.cases(predictFrame()$Concentration),]
            cal.lm.forest <- randomForest(Concentration~., data=predict.frame, na.action=na.omit)
            
            forest.predict <- predict(cal.lm.forest, new.data=predict.frame, proximity=FALSE)
            as.data.frame(forest.predict)
            
        })
        
        output$testtable <- renderDataTable({
            
            predictFramePre()
            
        })
        
        output$testtable2 <- renderDataTable({
            
            testing2()
            
        })
        
        
        calhold <- reactiveValues()
        
        observeEvent(input$calcurveelement, {
            calhold$caltype <- calTypeSelectionPre()
        })
        
        
        observeEvent(input$trainslopes, {
            
            isolate(calhold$caltype <- bestCalType())
            
        })
        
        
        
        calTypeSelection <- reactive({
            calhold$caltype
        })
        
        
        output$calTypeInput <- renderUI({
            
            selectInput("radiocal", label = "Calibration Curve",
            choices = list("Linear" = 1, "Non-Linear" = 2, "Lucas-Tooth" = 3, "Forest" = 4, "Rainforest"=5),
            selected = calTypeSelection())
            
            
        })
        
        
        
        
        
        
        predictFramePre <- reactive({
            
            intensity <- holdFrame()$Intensity
            
            concentration <- holdFrame()$Concentration
            
            predict.frame <- data.frame(concentration, intensity)
            colnames(predict.frame) <- c("Concentration", "Intensity")
            
            
            predict.frame[complete.cases(predict.frame),]
            
            
        })
        
        
        
        predictIntensity <- reactive({
            
            spectra.line.table <- spectraLineTable()
            data <- dataNorm()
            
            
            spectra.line.table <- spectraLineTable()[spectraLineTable()$Spectrum %in% holdFrame()$Spectrum, ]
            
            
            if (input$radiocal==1){
                
                if(input$normcal==1){
                    predict.intensity <- if(dataType()=="Spectra"){
                        general_prep_xrf(spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                    } else if(dataType()=="Net"){
                        general_prep_xrf_net(spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                    }
                }
                
                if(input$normcal==2){
                    predict.intensity <- if(dataType()=="Spectra"){
                        simple_tc_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                    } else if(dataType()=="Net"){
                        simple_tc_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                    }
                }
                
                if(input$normcal==3){
                    predict.intensity <- if(dataType()=="Spectra"){
                        simple_comp_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, norm.min=input$comptonmin, norm.max=input$comptonmax)
                    } else if(dataType()=="Net"){
                        simple_comp_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, norm.min=input$comptonmin, norm.max=input$comptonmax)
                    }
                }
                
            }
            
            if (input$radiocal==2){
                
                if(input$normcal==1){
                    predict.intensity <- if(dataType()=="Spectra"){
                        general_prep_xrf(spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                    } else if(dataType()=="Net"){
                        general_prep_xrf_net(spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                    }
                }
                
                if(input$normcal==2){
                    predict.intensity <- if(dataType()=="Spectra"){
                        simple_tc_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                    } else if(dataType()=="Net"){
                        simple_tc_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement)
                    }
                }
                
                if(input$normcal==3){
                    predict.intensity <- if(dataType()=="Spectra"){
                        simple_comp_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, norm.min=input$comptonmin, norm.max=input$comptonmax)
                    } else if(dataType()=="Net"){
                        simple_comp_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, norm.min=input$comptonmin, norm.max=input$comptonmax)
                    }
                }
                
            }
            
            
            if (input$radiocal==3){
                
                if(input$normcal==1){
                    predict.intensity <- if(dataType()=="Spectra"){
                        lucas_simp_prep_xrf(spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=input$slope_vars, intercept.element.lines=input$intercept_vars)
                    } else if(dataType()=="Net"){
                        lucas_simp_prep_xrf_net(spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=input$slope_vars, intercept.element.lines=input$intercept_vars)
                    }
                }
                
                if(input$normcal==2){
                    predict.intensity <- if(dataType()=="Spectra"){
                        lucas_tc_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=input$slope_vars, intercept.element.lines=input$intercept_vars)
                    } else if(dataType()=="Net"){
                        lucas_tc_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=input$slope_vars, intercept.element.lines=input$intercept_vars)
                    }
                }
                
                if(input$normcal==3){
                    predict.intensity <- if(dataType()=="Spectra"){
                        lucas_comp_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=input$slope_vars, intercept.element.lines=input$intercept_vars, norm.min=input$comptonmin, norm.max=input$comptonmax)
                    } else if(dataType()=="Net"){
                        lucas_comp_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=input$slope_vars, intercept.element.lines=input$intercept_vars, norm.min=input$comptonmin, norm.max=input$comptonmax)
                    }
                }
                
            }
            
            if (input$radiocal==4){
                concentration.table <- concentrationTable()
                concentration.table <- concentration.table[complete.cases(concentration.table[, input$calcurveelement]),]
                spectra.line.table <- spectra.line.table[complete.cases(concentration.table[, input$calcurveelement]),]
                data <- data[data$Spectrum %in% concentration.table$Spectrum, ]
                
                predict.intensity <- if(input$normcal==1){
                    if(dataType()=="Spectra"){
                        lucas_simp_prep_xrf(spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars)
                    } else if(dataType()=="Net"){
                        lucas_simp_prep_xrf_net(spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars)
                    }
                } else if(input$normcal==2){
                    predict.intensity <- if(dataType()=="Spectra"){
                        lucas_tc_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars)
                    } else if(dataType()=="Net"){
                        lucas_tc_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars)
                    }
                } else if(input$normcal==3){
                    predict.intensity <- if(dataType()=="Spectra"){
                        lucas_comp_prep_xrf(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars, norm.min=input$comptonmin, norm.max=input$comptonmax)
                    } else if(dataType()=="Net"){
                        lucas_comp_prep_xrf_net(data=data, spectra.line.table=spectra.line.table, element.line=input$calcurveelement, slope.element.lines=elementallinestouse(), intercept.element.lines=input$intercept_vars, norm.min=input$comptonmin, norm.max=input$comptonmax)
                    }
                }
                
            }
            
            if (input$radiocal==5){
                if(input$normcal==1){
                    predict.intensity <- if(dataType()=="Spectra"){
                        spectra_simp_prep_xrf(spectra=data)[,-1]
                    } else if(dataType()=="Net"){
                        NULL
                    }
                } else if(input$normcal==2){
                    predict.intensity <- if(dataType()=="Spectra"){
                        spectra_tc_prep_xrf(spectra=data)[,-1]
                    } else if(dataType()=="Net"){
                        NULL
                    }
                } else if(input$normcal==3){
                    predict.intensity <- if(dataType()=="Spectra"){
                        spectra_comp_prep_xrf(spectra=data, norm.min=input$comptonmin, norm.max=input$comptonmax)[,-1]
                    } else if(dataType()=="Net"){
                        NULL
                    }
                }
            }
            
            
            predict.intensity
            
            
        })
        
        
        output$testingagain <- renderDataTable({
            predictIntensity()
            
        })
        
        
        predictFrame <- reactive({
            
            predict.frame <- predictFramePre()
            predict.intensity <- predictIntensity()
            
            
            
            predict.frame <- data.frame(predict.intensity, predict.frame$Concentration)
            colnames(predict.frame) <- c(names(predict.intensity), "Concentration")
            
            
            
            predict.frame
            
            
        })
        
        
        predictFrameName <- reactive({
            
            predict.frame <- predictFrame()[ vals$keeprows, , drop = FALSE]
            spectra.line.table <- spectraLineTable()[ vals$keeprows, , drop = FALSE]
            
            predict.frame.name <- data.frame(spectra.line.table$Spectrum, predict.frame)
            colnames(predict.frame.name) <- c("Spectrum", names(predict.frame))
            predict.frame.name
            
        })
        
        
        
        calCurveFrame <- reactive({
            
            predictFrame()
            
        })
        
        
        elementModel <- reactive({
            
            
            
            if (input$radiocal==1){
                simpleLinearModel()
            } else if (input$radiocal==2){
                nonLinearModel()
            } else if (input$radiocal==3){
                lucasToothModel()
            } else if (input$radiocal==4){
                forestModel()
            } else if (input$radiocal==5){
                rainforestModel()
            }
            
            
            
        })
        
        
        valFrame <- reactive({
            
            predict.intensity <- predictIntensity()
            predict.frame <- predictFrame()
            element.model <- elementModel()
            
            
            if (input$radiocal==1){
                cal.est.conc.pred <- predict(object=element.model, newdata=predict.intensity, interval='confidence')
                cal.est.conc.tab <- data.frame(cal.est.conc.pred)
                cal.est.conc <- cal.est.conc.tab$fit
                
                val.frame <- data.frame(predict.frame$Concentration, cal.est.conc)
                colnames(val.frame) <- c("Concentration", "Prediction")
            }
            
            if (input$radiocal==2){
                cal.est.conc.pred <- predict(object=element.model, newdata=predict.intensity, interval='confidence')
                cal.est.conc.tab <- data.frame(cal.est.conc.pred)
                cal.est.conc <- cal.est.conc.tab$fit
                
                val.frame <- data.frame(predict.frame$Concentration, cal.est.conc)
                colnames(val.frame) <- c("Concentration", "Prediction")
            }
            
            if (input$radiocal==3){
                
                xmin = 0; xmax=10
                N = length(predict.frame$Concentration)
                means = colMeans(predict.frame)
                dummyDF = t(as.data.frame(means))
                for(i in 2:N){dummyDF=rbind(dummyDF,means)}
                xv=seq(xmin,xmax, length.out=N)
                dummyDF$Concentration = xv
                yv=predict(element.model, newdata=predict.intensity)
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- predict(object=element.model , newdata=predict.intensity, interval='confidence')
                cal.est.conc.tab <- data.frame(cal.est.conc.pred.luc)
                cal.est.conc.luc <- cal.est.conc.tab$fit
                cal.est.conc.luc.up <- cal.est.conc.tab$upr
                cal.est.conc.luc.low <- cal.est.conc.tab$lwr
                
                
                val.frame <- data.frame(predict.frame$Concentration, predict.intensity$Intensity, lucas.x, cal.est.conc.luc, cal.est.conc.luc.up, cal.est.conc.luc.low)
                colnames(val.frame) <- c("Concentration", "Intensity", "IntensityNorm", "Prediction", "Upper", "Lower")
            }
            
            if (input$radiocal==4){
                
                xmin = 0; xmax=10
                N = length(predict.frame$Concentration)
                means = colMeans(predict.frame)
                dummyDF = t(as.data.frame(means))
                for(i in 2:N){dummyDF=rbind(dummyDF,means)}
                xv=seq(xmin,xmax, length.out=N)
                dummyDF$Concentration = xv
                yv=predict(element.model, newdata=predict.intensity)
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- predict(object=element.model , newdata=predict.intensity, interval='confidence')
                #cal.est.conc.tab <- data.frame(cal.est.conc.pred.luc)
                #cal.est.conc.luc <- cal.est.conc.tab$fit
                #cal.est.conc.luc.up <- cal.est.conc.tab$upr
                #cal.est.conc.luc.low <- cal.est.conc.tab$lwr
                
                
                val.frame <- data.frame(predict.frame$Concentration, predict.intensity$Intensity, lucas.x, as.vector(cal.est.conc.pred.luc))
                colnames(val.frame) <- c("Concentration", "Intensity", "IntensityNorm", "Prediction")
            }
            
            
            if (input$radiocal==5){
                
                xmin = 0; xmax=10
                N = length(predict.frame$Concentration)
                means = colMeans(predict.frame)
                dummyDF = t(as.data.frame(means))
                for(i in 2:N){dummyDF=rbind(dummyDF,means)}
                xv=seq(xmin,xmax, length.out=N)
                dummyDF$Concentration = xv
                yv=predict(element.model, newdata=predict.intensity)
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- predict(object=element.model , newdata=predict.intensity, interval='confidence')
                #cal.est.conc.tab <- data.frame(cal.est.conc.pred.luc)
                #cal.est.conc.luc <- cal.est.conc.tab$fit
                #cal.est.conc.luc.up <- cal.est.conc.tab$upr
                #cal.est.conc.luc.low <- cal.est.conc.tab$lwr
                
                
                val.frame <- data.frame(predict.frame$Concentration, lucas.x, as.vector(cal.est.conc.pred.luc))
                colnames(val.frame) <- c("Concentration",  "IntensityNorm", "Prediction")
            }
            
            
            
            
            val.frame
            
        })
        
        calValFrame <- reactive({
            
            valFrame()
            
        })
        
        
        calType <- reactive({
            
            if(input$radiocal==1){
                1
            } else if(input$radiocal==2){
                2
            } else if(input$radiocal==3){
                3
            } else if(input$radiocal==4){
                3
            } else if(input$radiocal==5){
                5
            }
            
        })
        
        rangescalcurve <- reactiveValues(x = NULL, y = NULL)
        
        
        
        calCurvePlot <- reactive({
            
            predict.frame <- predictFrame()
            element.model <- elementModel()
            val.frame <- valFrame()
            
            element.name <- if(input$calcurveelement %in% spectralLines){
                gsub("[.]", "", substr(input$calcurveelement, 1, 2))
            } else {
                input$calcurveelement
            }
            intens <- " Counts per Second"
            norma <- " Normalized"
            norma.comp <- " Compton Normalized"
            norma.tc <- " Valid Counts Normalized"
            conen <- " (%)"
            predi <- " Estimate (%)"
            log <- "Log "
            
            
            intensity.name <- c(element.name, intens)
            concentration.name <- c(element.name, conen)
            prediction.name <- c(element.name, predi)
            
            
            if(input$radiocal==1){
                calcurve.plot <- ggplot(data=predict.frame[ vals$keeprows, , drop = FALSE], aes(Intensity, Concentration)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~Intensity, predict.frame[ vals$keeprows, , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(data = predict.frame[!vals$keeprows, , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
                stat_smooth(method="lm", fullrange = TRUE) +
                scale_x_continuous(paste(element.name, intens)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurve$x, ylim = rangescalcurve$y, expand = TRUE)
                
            }
            
            if(input$radiocal==2){
                calcurve.plot <- ggplot(data=predict.frame[ vals$keeprows, , drop = FALSE], aes(Intensity, Concentration)) +
                theme_light() +
                annotate("text", label=lm_eqn_poly(lm(Concentration~Intensity + I(Intensity^2), predict.frame[ vals$keeprows, , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(data = predict.frame[!vals$keeprows, , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
                stat_smooth(method="lm", formula=y~poly(x,2)) +
                scale_x_continuous(paste(element.name, intens)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurve$x, ylim = rangescalcurve$y, expand = TRUE)
                
            }
            
            if(input$radiocal==3){
                calcurve.plot <- ggplot(data=val.frame[ vals$keeprows, , drop = FALSE], aes(IntensityNorm, Concentration)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~., val.frame[ vals$keeprows, , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(aes(IntensityNorm, Concentration), data = val.frame[!vals$keeprows, , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
                geom_smooth(aes(x=IntensityNorm, y=Concentration, ymin = Lower, ymax = Upper)) +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurve$x, ylim = rangescalcurve$y, expand = TRUE)
                
            }
            
            if(input$radiocal==4){
                calcurve.plot <- ggplot(data=val.frame[ vals$keeprows, , drop = FALSE], aes(IntensityNorm, Concentration)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~., val.frame[ vals$keeprows, , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(aes(IntensityNorm, Concentration), data = val.frame[!vals$keeprows, , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
                geom_smooth() +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurve$x, ylim = rangescalcurve$y, expand = TRUE)
                
            }
            
            if(input$radiocal==5){
                calcurve.plot <- ggplot(data=val.frame[ vals$keeprows, , drop = FALSE], aes(IntensityNorm, Concentration)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~., val.frame[ vals$keeprows, , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(aes(IntensityNorm, Concentration), data = val.frame[!vals$keeprows, , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
                geom_smooth() +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurve$x, ylim = rangescalcurve$y, expand = TRUE)
                
            }
            
            
            
            calcurve.plot
            
            
        })
        
        observeEvent(input$cropcal, {
            brush <- input$plot_cal_brush
            if (!is.null(brush)) {
                rangescalcurve$x <- c(brush$xmin, brush$xmax)
                rangescalcurve$y <- c(brush$ymin, brush$ymax)
                
            } else {
                rangescalcurve$x <- NULL
                rangescalcurve$y <- NULL
            }
        })
        
        output$calcurveplots <- renderPlot({
            calCurvePlot()
        })
        
        
        rangesvalcurve <- reactiveValues(x = NULL, y = NULL)
        
        
        valCurvePlot <- reactive({
            
            predict.frame <- predictFrame()
            element.model <- elementModel()
            
            
            
            element.name <- if(input$calcurveelement %in% spectralLines){
                gsub("[.]", "", substr(input$calcurveelement, 1, 2))
            } else {
                input$calcurveelement
            }
            intens <- " Counts per Second"
            norma <- " Normalized"
            norma.comp <- " Compton Normalized"
            norma.tc <- " Valid Counts Normalized"
            conen <- " (%)"
            predi <- " Estimate (%)"
            log <- "Log "
            
            intensity.name <- c(element.name, intens)
            concentration.name <- c(element.name, conen)
            prediction.name <- c(element.name, predi)
            val.frame <- valFrame()
            
            
            valcurve.plot <- ggplot(data=val.frame[ vals$keeprows, , drop = FALSE], aes(Prediction, Concentration)) +
            theme_bw() +
            annotate("text", label=lm_eqn_val(lm(Concentration~Prediction, val.frame[ vals$keeprows, , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
            geom_abline(intercept=0, slope=1, lty=2) +
            stat_smooth(method="lm") +
            geom_point() +
            geom_point(aes(Prediction, Concentration),  data = val.frame[!vals$keeprows, , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
            scale_x_continuous(paste(element.name, predi)) +
            scale_y_continuous(paste(element.name, conen)) +
            coord_cartesian(xlim = rangesvalcurve$x, ylim = rangesvalcurve$y, expand = TRUE)
            
            
            
            
            
            valcurve.plot
            
        })
        
        observeEvent(input$cropval, {
            brush <- input$plot_val_brush
            if (!is.null(brush)) {
                rangesvalcurve$x <- c(brush$xmin, brush$xmax)
                rangesvalcurve$y <- c(brush$ymin, brush$ymax)
                
            } else {
                rangesvalcurve$x <- NULL
                rangesvalcurve$y <- NULL
            }
        })
        
        
        output$valcurveplots <- renderPlot({
            valCurvePlot()
        })
        
        
        
        
        
        calPlotDownload <- reactive({
            
            grid.arrange(calCurvePlot(), valCurvePlot(), ncol=2)
            
        })
        
        
        output$downloadcloudplot <- downloadHandler(
        filename = function() { paste(paste(c(input$calname, "_", input$calcurveelement), collapse=''), '.tiff',  sep='') },
        content = function(file) {
            ggsave(file,calPlotDownload(), device="tiff", compression="lzw", type="cairo", dpi=300, width=12, height=7)
        }
        )
        
        
        calValTable <- reactive({
            
            standard.table <- valFrame()
            hold.frame <- holdFrame()
            
            standard.table.summary <- data.frame(hold.frame$Spectrum, standard.table$Concentration, standard.table$Prediction, standard.table$Concentration-standard.table$Prediction, ((standard.table$Concentration-standard.table$Prediction)/standard.table$Concentration))
            colnames(standard.table.summary) <- c("Standard", "Concentration", "Prediction", "Difference", "Relative")
            
            standard.table.summary[,-1] <-round(standard.table.summary[,-1],4)
            standard.table.summary[,5] <- as.character(percent(standard.table.summary[,5]))
            
            this.table <- standard.table.summary
            this.table
            
        })
        
        
        output$standardsperformance <- DT::renderDataTable({
            
            
            standard.table <- calValTable()
            standard.table
            
        }, options =list(aoColumnDefs = list(list(sClass="alignRight",aTargets=c(list(2), list(3),list(4),list(5))))  ))
        
        
        randomizeData <- reactive({
            
            cal.frame <- concentrationTable()
            cal.frame <- cal.frame[ vals$keeprows, , drop = FALSE]
            total.number <- length(cal.frame[,1])
            sample.number <- total.number-round(input$percentrandom*total.number, 0)
            
            hold <- cal.frame[sample(nrow(cal.frame), sample.number),]
            cal.frame$Spectrum %in% hold$Spectrum
            
        })
        
        
        
        calCurveFrameRandomized <- reactive({
            
            predict.frame <- predictFrame()
            predict.frame <- predict.frame[ vals$keeprows, , drop = FALSE]
            
            predict.frame[randomizeData(),]
            
        })
        
        
        elementModelRandom <- reactive({
            
            predict.frame <- calCurveFrameRandomized()
            
            
            if (input$radiocal==1){
                cal.lm <- lm(Concentration~Intensity, data=predict.frame)
            }
            
            
            if (input$radiocal==2){
                cal.lm <- lm(Concentration~Intensity + I(Intensity^2), data=predict.frame)
            }
            
            if (input$radiocal==3){
                cal.lm <- lm(Concentration~., data=predict.frame)
            }
            
            if (input$radiocal==4){
                cal.lm <- randomForest(Concentration~., data=predict.frame, na.action=na.omit)
            }
            
            if (input$radiocal==5){
                cal.lm <- randomForest(Concentration~., data=predict.frame, na.action=na.omit)
            }
            
            cal.lm
            
        })
        
        
        valFrameRandomized <- reactive({
            
            predict.intensity <- predictIntensity()[ vals$keeprows, , drop = FALSE]
            predict.frame <- predictFrame()[ vals$keeprows, , drop = FALSE]
            
            predict.intensity <- predict.intensity[!(randomizeData()), , drop = FALSE]
            predict.frame <- predict.frame[!(randomizeData()), , drop = FALSE]
            element.model <- elementModelRandom()
            
            
            
            if (input$radiocal==1){
                cal.est.conc.pred <- predict(object=element.model, newdata=predict.intensity, interval='confidence')
                cal.est.conc.tab <- data.frame(cal.est.conc.pred)
                cal.est.conc <- cal.est.conc.tab$fit
                
                val.frame <- data.frame(predict.frame$Concentration, cal.est.conc)
                colnames(val.frame) <- c("Concentration", "Prediction")
            }
            
            if (input$radiocal==2){
                cal.est.conc.pred <- predict(object=element.model, newdata=predict.intensity, interval='confidence')
                cal.est.conc.tab <- data.frame(cal.est.conc.pred)
                cal.est.conc <- cal.est.conc.tab$fit
                
                val.frame <- data.frame(predict.frame$Concentration, cal.est.conc)
                colnames(val.frame) <- c("Concentration", "Prediction")
            }
            
            if (input$radiocal==3){
                
                xmin = 0; xmax=10
                N = length(predict.frame$Concentration)
                means = colMeans(predict.frame)
                dummyDF = t(as.data.frame(means))
                for(i in 2:N){dummyDF=rbind(dummyDF,means)}
                xv=seq(xmin,xmax, length.out=N)
                dummyDF$Concentration = xv
                yv=predict(element.model, newdata=predict.intensity)
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- predict(object=element.model , newdata=predict.intensity, interval='confidence')
                cal.est.conc.tab <- data.frame(cal.est.conc.pred.luc)
                cal.est.conc.luc <- cal.est.conc.tab$fit
                cal.est.conc.luc.up <- cal.est.conc.tab$upr
                cal.est.conc.luc.low <- cal.est.conc.tab$lwr
                
                
                val.frame <- data.frame(predict.frame$Concentration, predict.intensity$Intensity, lucas.x, cal.est.conc.luc, cal.est.conc.luc.up, cal.est.conc.luc.low)
                colnames(val.frame) <- c("Concentration", "Intensity", "IntensityNorm", "Prediction", "Upper", "Lower")
            }
            
            if (input$radiocal==4){
                
                xmin = 0; xmax=10
                N = length(predict.frame$Concentration)
                means = colMeans(predict.frame)
                dummyDF = t(as.data.frame(means))
                for(i in 2:N){dummyDF=rbind(dummyDF,means)}
                xv=seq(xmin,xmax, length.out=N)
                dummyDF$Concentration = xv
                yv=predict(element.model, newdata=predict.intensity)
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- predict(object=element.model , newdata=predict.intensity, interval='confidence')
                #cal.est.conc.tab <- data.frame(cal.est.conc.pred.luc)
                #cal.est.conc.luc <- cal.est.conc.tab$fit
                #cal.est.conc.luc.up <- cal.est.conc.tab$upr
                #cal.est.conc.luc.low <- cal.est.conc.tab$lwr
                
                
                val.frame <- data.frame(predict.frame$Concentration, predict.intensity$Intensity, lucas.x, as.vector(cal.est.conc.pred.luc))
                colnames(val.frame) <- c("Concentration", "Intensity", "IntensityNorm", "Prediction")
            }
            
            if (input$radiocal==5){
                
                xmin = 0; xmax=10
                N = length(predict.frame$Concentration)
                means = colMeans(predict.frame)
                dummyDF = t(as.data.frame(means))
                for(i in 2:N){dummyDF=rbind(dummyDF,means)}
                xv=seq(xmin,xmax, length.out=N)
                dummyDF$Concentration = xv
                yv=predict(element.model, newdata=predict.intensity)
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- predict(object=element.model , newdata=predict.intensity, interval='confidence')
                #cal.est.conc.tab <- data.frame(cal.est.conc.pred.luc)
                #cal.est.conc.luc <- cal.est.conc.tab$fit
                #cal.est.conc.luc.up <- cal.est.conc.tab$upr
                #cal.est.conc.luc.low <- cal.est.conc.tab$lwr
                
                
                val.frame <- data.frame(predict.frame$Concentration, lucas.x, as.vector(cal.est.conc.pred.luc))
                colnames(val.frame) <- c("Concentration", "IntensityNorm", "Prediction")
            }
            
            
            
            
            val.frame
            
        })
        
        
        valFrameRandomizedRev <- reactive({
            
            predict.intensity <- predictIntensity()[ vals$keeprows, , drop = FALSE]
            predict.frame <- predictFrame()[ vals$keeprows, , drop = FALSE]
            
            predict.intensity <- predict.intensity[(randomizeData()), ]
            predict.frame <- predict.frame[(randomizeData()), ]
            element.model <- elementModelRandom()
            
            
            
            if (input$radiocal==1){
                cal.est.conc.pred <- predict(object=element.model, newdata=predict.intensity, interval='confidence')
                cal.est.conc.tab <- data.frame(cal.est.conc.pred)
                cal.est.conc <- cal.est.conc.tab$fit
                
                val.frame <- data.frame(predict.frame$Concentration, cal.est.conc)
                colnames(val.frame) <- c("Concentration", "Prediction")
            }
            
            if (input$radiocal==2){
                cal.est.conc.pred <- predict(object=element.model, newdata=predict.intensity, interval='confidence')
                cal.est.conc.tab <- data.frame(cal.est.conc.pred)
                cal.est.conc <- cal.est.conc.tab$fit
                
                val.frame <- data.frame(predict.frame$Concentration, cal.est.conc)
                colnames(val.frame) <- c("Concentration", "Prediction")
            }
            
            if (input$radiocal==3){
                
                xmin = 0; xmax=10
                N = length(predict.frame$Concentration)
                means = colMeans(predict.frame)
                dummyDF = t(as.data.frame(means))
                for(i in 2:N){dummyDF=rbind(dummyDF,means)}
                xv=seq(xmin,xmax, length.out=N)
                dummyDF$Concentration = xv
                yv=predict(element.model, newdata=predict.intensity)
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- predict(object=element.model , newdata=predict.intensity, interval='confidence')
                cal.est.conc.tab <- data.frame(cal.est.conc.pred.luc)
                cal.est.conc.luc <- cal.est.conc.tab$fit
                cal.est.conc.luc.up <- cal.est.conc.tab$upr
                cal.est.conc.luc.low <- cal.est.conc.tab$lwr
                
                
                val.frame <- data.frame(predict.frame$Concentration, predict.intensity$Intensity, lucas.x, cal.est.conc.luc, cal.est.conc.luc.up, cal.est.conc.luc.low)
                colnames(val.frame) <- c("Concentration", "Intensity", "IntensityNorm", "Prediction", "Upper", "Lower")
            }
            
            if (input$radiocal==4){
                
                xmin = 0; xmax=10
                N = length(predict.frame$Concentration)
                means = colMeans(predict.frame)
                dummyDF = t(as.data.frame(means))
                for(i in 2:N){dummyDF=rbind(dummyDF,means)}
                xv=seq(xmin,xmax, length.out=N)
                dummyDF$Concentration = xv
                yv=predict(element.model, newdata=predict.intensity)
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- predict(object=element.model , newdata=predict.intensity, interval='confidence')
                
                
                
                val.frame <- data.frame(predict.frame$Concentration, predict.intensity$Intensity, lucas.x, as.vector(cal.est.conc.pred.luc))
                colnames(val.frame) <- c("Concentration", "Intensity", "IntensityNorm", "Prediction")
            }
            
            if (input$radiocal==5){
                
                xmin = 0; xmax=10
                N = length(predict.frame$Concentration)
                means = colMeans(predict.frame)
                dummyDF = t(as.data.frame(means))
                for(i in 2:N){dummyDF=rbind(dummyDF,means)}
                xv=seq(xmin,xmax, length.out=N)
                dummyDF$Concentration = xv
                yv=predict(element.model, newdata=predict.intensity)
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- predict(object=element.model , newdata=predict.intensity, interval='confidence')
                
                
                
                val.frame <- data.frame(predict.frame$Concentration, lucas.x, as.vector(cal.est.conc.pred.luc))
                colnames(val.frame) <- c("Concentration", "IntensityNorm", "Prediction")
            }
            
            
            
            
            val.frame
            
        })
        
        rangescalcurverandom <- reactiveValues(x = NULL, y = NULL)
        
        
        calCurvePlotRandom <- reactive({
            
            predict.frame <- calCurveFrameRandomized()
            element.model <- elementModelRandom()
            
            
            element.name <- if(input$calcurveelement %in% spectralLines){
                gsub("[.]", "", substr(input$calcurveelement, 1, 2))
            } else {
                input$calcurveelement
            }
            intens <- " Counts per Second"
            norma <- " Normalized"
            norma.comp <- " Compton Normalized"
            norma.tc <- " Valid Counts Normalized"
            conen <- " (%)"
            predi <- " Estimate (%)"
            
            intensity.name <- c(element.name, intens)
            concentration.name <- c(element.name, conen)
            prediction.name <- c(element.name, predi)
            
            
            if(input$radiocal==1){
                calcurve.plot <- ggplot(data=predict.frame, aes(Intensity, Concentration)) +
                theme_light() +
                annotate("text", label=lm_eqn(element.model), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(data = predict.frame, shape = 21, fill = "red", color = "black", alpha = 0.25) +
                stat_smooth(method="lm", fullrange = TRUE) +
                scale_x_continuous(paste(element.name, intens)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurverandom$x, ylim = rangescalcurverandom$y, expand = TRUE)
                
            }
            
            if(input$radiocal==2){
                
                calcurve.plot <- ggplot(data=predict.frame, aes(Intensity, Concentration)) +
                theme_light() +
                annotate("text", label=lm_eqn_poly(element.model), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(data = predict.frame, shape = 21, fill = "red", color = "black", alpha = 0.25) +
                stat_smooth(method="lm", formula=y~poly(x,2)) +
                scale_x_continuous(paste(element.name, intens)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurverandom$x, ylim = rangescalcurverandom$y, expand = TRUE)
            }
            
            if(input$radiocal==3){
                val.frame <- valFrameRandomizedRev()
                
                calcurve.plot <- ggplot(data=val.frame, aes(IntensityNorm, Concentration)) +
                theme_light() +
                annotate("text", label=lm_eqn(element.model), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(aes(IntensityNorm, Concentration), data = val.frame, shape = 21, fill = "red", color = "black", alpha = 0.25) +
                geom_smooth(aes(x=IntensityNorm, y=Concentration, ymin = Lower, ymax = Upper)) +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurverandom$x, ylim = rangescalcurverandom$y, expand = TRUE)
            }
            
            if(input$radiocal==4){
                val.frame <- valFrameRandomizedRev()
                
                calcurve.plot <- ggplot(data=val.frame, aes(IntensityNorm, Concentration)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~., val.frame)), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(aes(IntensityNorm, Concentration), data = val.frame, shape = 21, fill = "red", color = "black", alpha = 0.25) +
                geom_smooth() +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurverandom$x, ylim = rangescalcurverandom$y, expand = TRUE)
            }
            
            if(input$radiocal==5){
                val.frame <- valFrameRandomizedRev()
                
                calcurve.plot <- ggplot(data=val.frame, aes(IntensityNorm, Concentration)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~., val.frame)), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(aes(IntensityNorm, Concentration), data = val.frame, shape = 21, fill = "red", color = "black", alpha = 0.25) +
                geom_smooth() +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurverandom$x, ylim = rangescalcurverandom$y, expand = TRUE)
            }
            
            calcurve.plot
            
            
        })
        
        observeEvent(input$cropcalrandom, {
            brush <- input$plot_cal_brush_random
            if (!is.null(brush)) {
                rangescalcurverandom$x <- c(brush$xmin, brush$xmax)
                rangescalcurverandom$y <- c(brush$ymin, brush$ymax)
                
            } else {
                rangescalcurverandom$x <- NULL
                rangescalcurverandom$y <- NULL
            }
        })
        
        
        
        output$calcurveplotsrandom <- renderPlot({
            calCurvePlotRandom()
        })
        
        
        rangesvalcurverandom <- reactiveValues(x = NULL, y = NULL)
        
        valCurvePlotRandom <- reactive({
            
            
            
            #element.name <- gsub("[.]", "", substr(input$calcurveelement, 1, 2))
            
            element.name <- if(input$calcurveelement %in% spectralLines){
                gsub("[.]", "", substr(input$calcurveelement, 1, 2))
            } else {
                input$calcurveelement
            }

            intens <- " Counts per Second"
            norma <- " Normalized"
            norma.comp <- " Compton Normalized"
            norma.tc <- " Valid Counts Normalized"
            conen <- " (%)"
            predi <- " Estimate (%)"
            
            intensity.name <- c(element.name, intens)
            concentration.name <- c(element.name, conen)
            prediction.name <- c(element.name, predi)
            
            val.frame <- valFrameRandomized()
            
            valcurve.plot <- ggplot(data=val.frame, aes(Prediction, Concentration)) +
            theme_bw() +
            annotate("text", label=lm_eqn_val(lm(Concentration~Prediction, val.frame)), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
            geom_abline(intercept=0, slope=1, lty=2) +
            stat_smooth(method="lm") +
            geom_point() +
            geom_point(aes(Prediction, Concentration),  data = val.frame, shape = 21, fill = "red", color = "black", alpha = 0.25) +
            scale_x_continuous(paste(element.name, predi)) +
            scale_y_continuous(paste(element.name, conen)) +
            coord_cartesian(xlim = rangesvalcurverandom$x, ylim = rangesvalcurverandom$y, expand = TRUE)
            
            valcurve.plot
            
        })
        
        observeEvent(input$cropvalrandom, {
            brush <- input$plot_val_brush_random
            if (!is.null(brush)) {
                rangesvalcurverandom$x <- c(brush$xmin, brush$xmax)
                rangesvalcurverandom$y <- c(brush$ymin, brush$ymax)
                
            } else {
                rangesvalcurverandom$x <- NULL
                rangesvalcurverandom$y <- NULL
            }
        })
        
        output$valcurveplotsrandom <- renderPlot({
            valCurvePlotRandom()
        })
        
        
        ####CalCurves
        
        # Float over info
        output$hover_infocal <- renderUI({
            
            point.table <- if(calType()==1){
                calCurveFrame()
            } else if(calType()==2){
                calCurveFrame()
            } else if(calType()==3) {
                calValFrame()
            } else if(calType()==5) {
                calValFrame()
            }
            
            concentration.table <- concentrationTable()
            hold.table <- concentration.table[,c("Spectrum", input$calcurveelement)]
            colnames(hold.table) <- c("Spectrum", "Selection")
            hold.table$Selection[hold.table$Selection==""] <- NA
            hold.table <- hold.table[complete.cases(hold.table), ]
            
            point.table$Spectrum <- hold.table["Spectrum"]
            
            hover <- input$plot_hovercal
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Spectrum
            
            )))
            )
        })
        
        
        # Float over info
        output$hover_infocal_random <- renderUI({
            
            point.table <- if(calType()==1){
                calCurveFrame()
            } else if(calType()==2){
                calCurveFrame()
            } else if(calType()==3) {
                calValFrame()
            } else if(calType()==5) {
                calValFrame()
            }
            
            randomized <- randomizeData()
            
            
            point.table <- point.table[ vals$keeprows, , drop = FALSE]
            point.table <- point.table[randomized,]
            
            
            concentration.table <- concentrationTable()
            
            concentration.table <- concentration.table[ vals$keeprows, , drop = FALSE]
            concentration.table <- concentration.table[randomized,]
            
            hold.table <- concentration.table[,c("Spectrum", input$calcurveelement)]
            colnames(hold.table) <- c("Spectrum", "Selection")
            
            
            point.table$Spectrum <- hold.table["Spectrum"]
            
            hover <- input$plot_hovercal_random
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Spectrum
            
            )))
            )
        })
        
        
        # Toggle points that are clicked
        observeEvent(input$plot_cal_click, {
            
            predict.frame <- if(calType()==1){
                calCurveFrame()
            } else if(calType()==2){
                calCurveFrame()
            } else if(calType()==3) {
                calValFrame()
            } else if(calType()==5) {
                calValFrame()
            }
            
            res <- nearPoints(predict.frame, input$plot_cal_click, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        # Toggle points that are brushed, when button is clicked
        observeEvent(input$exclude_toggle, {
            
            predict.frame <- if(calType()==3){
                calCurveFrame()
            } else if(calType()==2){
                calCurveFrame()
            } else if(calType()==3) {
                calValFrame()
            } else if(calType()==5) {
                calValFrame()
            }
            res <- brushedPoints(predict.frame, input$plot_cal_brush, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        # Reset all points
        observeEvent(input$exclude_reset, {
            
            predict.frame <- if(calType()==1){
                calCurveFrame()
            } else if(calType()==2){
                calCurveFrame()
            } else if(calType()==3) {
                calValFrame()
            } else if(calType()==5) {
                calValFrame()
            }
            vals$keeprows <- rep(TRUE, nrow(predict.frame))
        })
        
        # Reset all points on element change
        observeEvent(input$calcurveelement, {
            
            
            
            vals$keeprows <- calFileStandards()
            
            
            
        })
        
        
        
        
        ####ValCurves
        
        
        
        
        # Float over info
        output$hover_infoval <- renderUI({
            
            point.table <- calValFrame()
            concentration.table <- concentrationTable()
            hold.table <- concentration.table[,c("Spectrum", input$calcurveelement)]
            colnames(hold.table) <- c("Spectrum", "Selection")
            hold.table$Selection[hold.table$Selection==""] <- NA
            hold.table <- hold.table[complete.cases(hold.table), ]
            
            point.table$Spectrum <- hold.table["Spectrum"]
            
            
            hover <- input$plot_hoverval
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Spectrum
            
            )))
            )
        })
        
        
        output$hover_infoval_random <- renderUI({
            
            point.table <- calValFrame()
            
            randomized <- randomizeData()
            
            
            point.table <- point.table[ vals$keeprows, , drop = FALSE]
            point.table <- point.table[!(randomized),]
            
            concentration.table <- concentrationTable()
            
            concentration.table <- concentration.table[ vals$keeprows, , drop = FALSE]
            concentration.table <- concentration.table[!(randomized),]
            concentration.table.rev <- concentration.table[(randomized),]
            
            hold.table <- concentration.table[,c("Spectrum", input$calcurveelement)]
            colnames(hold.table) <- c("Spectrum", "Selection")
            
            
            point.table$Spectrum <- hold.table["Spectrum"]
            
            
            point.table <- point.table[point.table$Concentration > min(concentration.table.rev[,input$calcurveelement], na.rm = TRUE) & point.table$Concentration < max(concentration.table.rev[,input$calcurveelement], na.rm = TRUE), ]
            
            
            
            hover <- input$plot_hoverval_random
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Spectrum
            
            )))
            )
        })
        
        # Toggle points that are clicked
        observeEvent(input$plot_val_click, {
            
            predict.frame <- calValFrame()
            
            res <- nearPoints(predict.frame, input$plot_val_click, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        # Toggle points that are brushed, when button is clicked
        observeEvent(input$exclude_toggle, {
            predict.frame <- calValFrame()
            
            res <- brushedPoints(predict.frame, input$plot_val_brush, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        # Reset all points
        observeEvent(input$exclude_reset, {
            predict.frame <- calValFrame()
            
            vals$keeprows <- rep(TRUE, nrow(predict.frame))
        })
        
        
        normalLM <- reactive({
            
            
            model <- elementModel()
            
            model.frame <- as.data.frame(augment(model))
            
            model.frame$qq <- qqnorm(model.frame$.std.resid)[[1]]
            
            model.frame$sqrt.std.resid <- sqrt(abs(model.frame$.std.resid))
            
            model.frame$seq.cooksd <- seq_along(model.frame$.cooksd)
            
            #model.frame$Spectrum <- predictFrameName()$Spectrum
            
            
            
            model.frame
            
        })
        
        
        forestLM <- reactive({
            
            
            model <- lm(Concentration~Prediction, data=as.data.frame(calValTable()))
            
            model.frame <- as.data.frame(augment(model))
            
            model.frame$qq <- qqnorm(model.frame$.std.resid)[[1]]
            
            model.frame$sqrt.std.resid <- sqrt(abs(model.frame$.std.resid))
            
            model.frame$seq.cooksd <- seq_along(model.frame$.cooksd)
            
            #model.frame$Spectrum <- predictFrameName()$Spectrum
            
            
            
            model.frame
            
            
        })
        
        
        modelFrame <- reactive({
            
            if(input$radiocal==1){
                normalLM()
            } else if(input$radiocal==2){
                normalLM()
            } else if(input$radiocal==3){
                normalLM()
            } else if(input$radiocal==4){
                forestLM()
            } else if(input$radiocal==5){
                forestLM()
            }
            
            
        })
        
        
        
        
        
        diagResidualsFitted <- reactive({
            
            model <- modelFrame()
            
            p1 <- ggplot(model[ vals$keeprows, , drop = FALSE], aes(.fitted, .resid)) +
            stat_smooth(method="loess") +
            geom_hline(yintercept=0, col="red", linetype="dashed") +
            xlab("Fitted values") +
            ylab("Residuals") +
            ggtitle("Residual vs Fitted Plot") +
            theme_light() +
            geom_point() +
            geom_point(data=model[ !vals$keeprows, , drop = FALSE], aes(.fitted, .resid), shape = 21, fill = "red", color = "black", alpha = 0.25)
            
            p1
            
        })
        
        output$residualsfitted <- renderPlot({
            
            diagResidualsFitted()
            
        })
        
        
        diagQQ <- reactive({
            
            model <- modelFrame()
            
            p2 <- ggplot(model[ vals$keeprows, , drop = FALSE], aes(qq, .std.resid))+geom_point(na.rm = TRUE) +
            geom_abline() +
            xlab("Theoretical Quantiles") +
            ylab("Standardized Residuals") +
            ggtitle("Normal Q-Q") +
            theme_light() +
            geom_point(data=model[ !vals$keeprows, , drop = FALSE], aes(qq, .std.resid), shape = 21, fill = "red", color = "black", alpha = 0.25)
            
            
            p2
            
        })
        
        
        output$qq <- renderPlot({
            
            diagQQ()
            
        })
        
        diagScaleLocation <- reactive({
            
            model <- modelFrame()
            
            
            p3 <- ggplot(model[ vals$keeprows, , drop = FALSE], aes(.fitted, sqrt.std.resid)) +
            stat_smooth(method="loess", na.rm = TRUE) +
            xlab("Fitted Value") +
            ylab(expression(sqrt("|Standardized residuals|"))) +
            ggtitle("Scale-Location") +
            theme_light() +
            geom_point(na.rm=TRUE) +
            geom_point(data=model[ !vals$keeprows, , drop = FALSE], aes(.fitted, sqrt.std.resid), shape = 21, fill = "red", color = "black", alpha = 0.25)
            
            
            p3
            
            
        })
        
        
        output$scalelocation <- renderPlot({
            
            diagScaleLocation()
            
        })
        
        
        diagCooksDistance <- reactive({
            
            model <- modelFrame()
            
            p4 <- ggplot(model, aes(seq.cooksd, .cooksd)) +
            geom_bar(stat="identity", position="identity") +
            xlab("Obs. Number") +
            ylab("Cook's distance") +
            ggtitle("Cook's distance") +
            theme_light()
            
            p4
            
        })
        
        output$cooksdistance <- renderPlot({
            
            diagCooksDistance()
            
        })
        
        
        diagResidualLeverage <- reactive({
            
            model <- modelFrame()
            
            
            p5<-ggplot(model[ vals$keeprows, , drop = FALSE], aes(.hat, .std.resid))+
            geom_point(aes(size=.cooksd), na.rm=TRUE) +
            geom_point(data=model[ !vals$keeprows, , drop = FALSE], aes(.hat, .std.resid), shape = 21, fill = "red", color = "black", alpha = 0.25) +
            stat_smooth(method="loess", na.rm=TRUE) +
            xlab("Leverage") +
            ylab("Standardized Residuals") +
            ggtitle("Residual vs Leverage Plot") +
            scale_size_continuous("Cook's Distance", range=c(1,5)) +
            theme_light() +
            theme(legend.position="bottom")
            
            p5
            
        })
        
        output$residualleverage <- renderPlot({
            
            diagResidualLeverage()
            
        })
        
        
        diagCooksLeverage <- reactive({
            
            model <- modelFrame()
            
            
            p6 <- ggplot(model[ vals$keeprows, , drop = FALSE], aes(.hat, .cooksd)) +
            stat_smooth(method="loess", na.rm=TRUE) +
            xlab("Leverage hii") +
            ylab("Cook's Distance") +
            ggtitle("Cook's dist vs Leverage hii/(1-hii)") +
            geom_abline(slope=seq(0,3,0.5), color="gray", linetype="dashed") +
            theme_light() +
            geom_point(na.rm=TRUE) +
            geom_point(data=model[ vals$keeprows, , drop = FALSE], aes(.hat, .cooksd), shape = 21, fill = "red", color = "black", alpha = 0.25)
            
            p6
            
        })
        
        
        output$cooksleverage <- renderPlot({
            
            diagCooksLeverage()
            
        })
        
        
        diagPlotDownload <- reactive({
            
            grid.arrange(diagResidualsFitted(), diagQQ(),
            diagScaleLocation(), diagCooksDistance(),
            diagResidualLeverage(), diagCooksLeverage(),
            ncol=2, nrow=3)
            
        })
        
        
        output$diagplots <- downloadHandler(
        filename = function() { paste(input$calname, "_", input$calcurveelement, "_diag", ".tiff", sep='') },
        content = function(file) {
            ggsave(file,diagPlotDownload(), width=10, height=10, device="tiff", compression="lzw", type="cairo", dpi=300, )
        }
        )
        
        #########Diagnostic Plot Controls#######
        ####Residuals Fitted
        # Float over info
        output$hover_inforesidualsfitted <- renderUI({
            
            point.table <- modelFrame()
            
            
            hover <- input$plot_hoverresidualsfitted
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Spectrum
            
            )))
            )
        })
        
        # Toggle points that are clicked
        observeEvent(input$plot_residualsfitted_click, {
            
            predict.frame <- modelFrame()
            
            res <- nearPoints(predict.frame, input$plot_residualsfitted_click, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        # Toggle points that are brushed, when button is clicked
        observeEvent(input$exclude_toggle_diag, {
            
            predict.frame <- modelFrame()
            
            res <- brushedPoints(predict.frame, input$plot_residualsfitted_brush, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        ####QQ Norm
        # Float over info
        output$hover_infoqq <- renderUI({
            
            point.table <- modelFrame()
            
            
            hover <- input$plot_hoverqq
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Spectrum
            
            )))
            )
        })
        
        # Toggle points that are clicked
        observeEvent(input$plot_qq_click, {
            
            predict.frame <- modelFrame()
            
            res <- nearPoints(predict.frame, input$plot_qq_click, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        # Toggle points that are brushed, when button is clicked
        observeEvent(input$exclude_toggle_diag, {
            
            predict.frame <- modelFrame()
            
            res <- brushedPoints(predict.frame, input$plot_qq_brush, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        ####Scaled Residuals
        # Float over info
        output$hover_infoscalelocation <- renderUI({
            
            point.table <- modelFrame()
            
            
            hover <- input$plot_hoverscalelocation
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Spectrum
            
            )))
            )
        })
        
        # Toggle points that are clicked
        observeEvent(input$plot_scalelocation_click, {
            
            predict.frame <- modelFrame()
            
            res <- nearPoints(predict.frame, input$plot_scalelocation_click, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        # Toggle points that are brushed, when button is clicked
        observeEvent(input$exclude_toggle_diag, {
            
            predict.frame <- modelFrame()
            
            res <- brushedPoints(predict.frame, input$plot_scalelocation_brush, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        ####Residuals Leverage
        # Float over info
        output$hover_inforesidualleverage <- renderUI({
            
            point.table <- modelFrame()
            
            
            hover <- input$plot_hoverresidualleverage
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Spectrum
            
            )))
            )
        })
        
        # Toggle points that are clicked
        observeEvent(input$plot_residualleverage_click, {
            
            predict.frame <- modelFrame()
            
            res <- nearPoints(predict.frame, input$plot_residualleverage_click, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        # Toggle points that are brushed, when button is clicked
        observeEvent(input$exclude_toggle_diag, {
            
            predict.frame <- modelFrame()
            
            res <- brushedPoints(predict.frame, input$plot_residualleverage_brush, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        ####Cooks Leverage
        # Float over info
        output$hover_infocooksleverage <- renderUI({
            
            point.table <- modelFrame()
            
            
            hover <- input$plot_hovercooksleverage
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Spectrum
            
            )))
            )
        })
        
        # Toggle points that are clicked
        observeEvent(input$plot_cooksleverage_click, {
            
            predict.frame <- modelFrame()
            
            res <- nearPoints(predict.frame, input$plot_cooksleverage_click, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        # Toggle points that are brushed, when button is clicked
        observeEvent(input$exclude_toggle_diag, {
            
            predict.frame <- modelFrame()
            
            res <- brushedPoints(predict.frame, input$plot_cooksleverage_brush, allRows = TRUE)
            
            vals$keeprows <- xor(vals$keeprows, res$selected_)
        })
        
        
        
        # Reset all points
        observeEvent(input$exclude_reset_diag, {
            
            predict.frame <- modelFrame()
            
            vals$keeprows <- rep(TRUE, nrow(predict.frame))
        })
        
        
        
        
        #output$downloadcal <- downloadHandler(
        #filename = function() { paste(input$dataset, '.csv', sep=',') },
        #content = function(file
        #) {
        #  write.csv(metadataForm(), file)
        # }
        #)
        
        
        
        ####alternative take
        
        nullList <- reactive({
            
            spectra.line.table <- spectraData()
            
            cal.vector <- elementallinestouse()
            cal.vector2 <- cal.vector[2:length(cal.vector)]
            cal.list <- as.list(cal.vector2)
            setNames(cal.list, cal.vector2)
            cal.list <- pblapply(cal.list, function(x) return(NULL))
            nullList <- cal.list
            
            
        })
        
        
        
        
        #rf2 <- reactiveValues()
        #observe({
        #    if(input$createcalelement > 0){
        #    calList[[input$calcurveelement]] <- elementModel()
        #    }
        #    rf2 <<- calList
        #})
        
        emptyList <- reactive({
            a.list <- list()
            a.list
        })
        
        
        
        
        
        calList <- reactiveValues()
        
        observeEvent(input$actionprocess, {
            isolate(calList <- emptyList())
            calList <<- calList
        })
        
        
        
        
        observeEvent(input$createcalelement, {
            
            cal.condition <- input$radiocal
            norm.condition <- input$normcal
            
            norm.min <- print(input$comptonmin)
            norm.max <- print(input$comptonmax)
            
            cal.table <- data.frame(cal.condition, norm.condition, norm.min, norm.max)
            colnames(cal.table) <- c("CalType", "NormType", "Min", "Max")
            
            slope.corrections <- input$slope_vars
            intercept.corrections <- input$intercept_vars
            
            standards.used <- vals$keeprows
            
            cal.mode.list <- list(cal.table, slope.corrections, intercept.corrections, standards.used)
            names(cal.mode.list) <- c("CalTable", "Slope", "Intercept", "StandardsUsed")
            
            calConditons <<- cal.mode.list
            
        })
        
        
        calList <- reactiveValues()
        observeEvent(input$createcalelement, {
            
            
            calList[[input$calcurveelement]] <- list(isolate(calConditons), isolate(strip_glm(elementModel())))
            
            calList <<- calList
            
        })
        
        calPlotList <- reactiveValues()
        calPlotList <- emptyList()
        observeEvent(input$createcalelement, {
            
            
            calPlotList[[input$calcurveelement]] <- isolate(calPlotDownload())
            
            calPlotList <<- calPlotList
            
        })
        
        diagPlotList <- reactiveValues()
        diagPlotList <- emptyList()
        #observeEvent(input$createcalelement, {
        
        
        #diagPlotList[[input$calcurveelement]] <- isolate(diagPlotDownload())
        
        #diagPlotList <<- diagPlotList
        
        #})
        
        Calibration <- reactiveValues()
        observeEvent(input$createcal, {
            
            
            spectra.line.table <- if(dataType()=="Spectra"){
                spectraData()
            } else if(dataType()=="Net"){
                dataHold()
            }
            cal.intensities <- spectra.line.table[elementallinestouse()]
            cal.values <- values[["DF"]]
            cal.data <- if(dataType()=="Spectra"){
                dataHold()
            } else if(dataType()=="Net"){
                myData()
            }
            
            cal.definitions <- linevalues[["DF"]]
            
            notes <- input$notes
            
            
            dataHold()
            
            calibrationList <- NULL
            calibrationList <- list(input$filetype, input$calunits, cal.data, cal.intensities, cal.definitions, cal.values, notes, calList)
            names(calibrationList) <- c("FileType", "Units", "Spectra", "Intensities", "Definitions", "Values", "Notes", "calList")
            
            Calibration <<- calibrationList
            
            
        })
        
        CalibrationPlots <- reactiveValues()
        observeEvent(input$createcal, {
            
            CalibrationPlots$calCurves <<- calPlotList
            
            
        })
        
        #observeEvent(input$createcal, {
        
        #CalibrationPlots$diagPlots <<- diagPlotList
        
        
        #})
        
        
        
        output$downloadModel <- downloadHandler(
        filename <- function(){
            paste(input$calname, "quant", sep=".")
        },
        
        content = function(file) {
            saveRDS(Calibration, file = file, compress="xz")
        }
        )
        
        
        output$downloadReport <- downloadHandler(
        function() { paste(paste(c(input$calname), collapse=''), '.pdf',  sep='') },
        content = function(file){
            ml = marrangeGrob(grobs=CalibrationPlots$calCurves, nrow=1, ncol=1)
            ggsave(file, ml, device="pdf", dpi=300, width=12, height=7)
            
            dev.off()
        })




    })
    
    
    ####Multiplots Here
    
    
    observeEvent(input$actionprocess_multi, {
        
        
        quantNames <- reactive({
            
            inFile <- input$calfileinput_multi
            if (is.null(inFile)) return(NULL)
            
            as.vector(gsub(".quant", "", inFile$name))
            
            
            
        })


        quantLoad <- reactive({
            
            inFile <- input$calfileinput_multi
            if (is.null(inFile)) return(NULL)
            
            cal.list <- lapply(inFile$datapath, readRDS)
            names(cal.list) <- quantNames()
            
            cal.list
            
        })
        
        calListMulti <- reactiveValues()
        
        
        observeEvent(input$actionprocess_multi, {
            isolate(calListMulti <<- quantLoad())
            
        })


    

        
        
        
        
        
        nQuant <- reactive({
            
            inFile <- input$calfileinput_multi
            if (is.null(inFile)) return(NULL)
            
            length(inFile$name)
            
        })
        

        
        
        output$defaultcalui <- renderUI({
            
            inFile <- input$calfileinput_multi
            if (is.null(inFile)) return(NULL)
            
            names <- gsub(".quant", "", inFile$name)
            
            selectInput('defaultcal', "Active Cal", choices=quantNames(), selected=quantNames()[1], multiple=FALSE)
            
        })
        
        
        quantType <- reactive({
            
            filetype.vector <- sapply(calListMulti, "[[", "FileType")
            unique(filetype.vector)
            
        })
        
        
        
        dataTypeMulti <- reactive({
            if(quantType()=="CSV"){
                "Spectra"
            } else if(quantType()=="Spectra"){
                "Spectra"
            } else if(quantType()=="PDZ"){
                "Spectra"
            } else if(quantType()=="TXT"){
                "Spectra"
            } else if(quantType()=="Elio"){
                "Spectra"
            }  else if(quantType()=="MCA"){
                "Spectra"
            }  else if(quantType()=="SPX"){
                "Spectra"
            } else if(quantType()=="Net"){
                "Net"
            }
            
        })
        
        
        
        
        quantIntensities <- reactive({
            
            cal.list <- calListMulti
            
            cal.names <- quantNames()
            
            n <- length(cal.names)
            
            intensities.list <- lapply(cal.list, "[[", "Intensities")
            names(intensities.list) <- quantNames()
            
            intensities.list <- lapply(intensities.list, as.data.frame)
            names(intensities.list) <- quantNames()
            
            intensities.list
            
            #line <- seq(1, length(intensities.list), 1)
            
            #intensities.list <- lapply(line, function(x) data.frame(intensities.list[[x]], Instrument=cal.names[x]))
            #names(intensities.list) <- quantNames()
            #do.call("rbind", intensities.list)
            
        })
        
        quantValues <- reactive({
            
            cal.list <- calListMulti
            
            cal.names <- quantNames()
            
            n <- length(cal.names)
            
            values.list <- lapply(cal.list, "[[", "Values")
            names(values.list) <- quantNames()
            
            values.list <- lapply(values.list, as.data.frame)
            names(values.list) <- quantNames()
            
            values.list
            
            #line <- seq(1, length(values.list), 1)
            
            #values.list <- lapply(line, function(x) data.frame(values.list[[x]], Instrument=cal.names[x]))
            #names(values.list) <- quantNames()
            #do.call("rbind", values.list)
            
            
        })
        
        
        quantData <- reactive({
            
            cal.list <- calListMulti
            
            cal.names <- quantNames()
            
            n <- length(cal.names)
            
            data.list <- lapply(cal.list, "[[", "Spectra")
            names(data.list) <- quantNames()
            
            data.list <- lapply(data.list, as.data.frame)
            names(data.list) <- quantNames()
            
            data.list
            
            #line <- seq(1, length(data.list), 1)
            
            #data.list <- lapply(line, function(x) data.frame(data.list[[x]], Instrument=cal.names[x]))
            #names(data.list) <- quantNames()
            #do.call("rbind", data.list)
            
            data.list
            
            
        })
        
        
        quantCals <- reactive({
            
            cal.list <- calListMulti
            
            cal.names <- quantNames()
            
            n <- length(cal.names)
            
            quant.list <- lapply(cal.list, "[[", "calList")
            names(quant.list) <- quantNames()
            
            quant.list
            
            
        })
        
        
        elementallinestouseMulti <- reactive({
            
            
            colnames(quantIntensities()[[input$defaultcal]])
            
            
        })
        
        
        variablelinestouseMulti <- reactive({
            
            
            names(quantValues()[[input$defaultcal]])
            
            
        })
        
        
        outVarMulti <- reactive({
            
            myelements <- elementallinestouseMulti()
            
            result <- if(is.null(myelements)){
                "Ca.K.alpha"
            }else{
                myelements
            }
            
            result
            
            
        })
        
        outVaraltMulti <- reactive({
            
            
            myelements <- c(variablelinestouseMulti())
            
            
            if(is.null(myelements)){
                paste("Ca.K.alpha")
            }else{
                myelements
            }
            
        })
        
        outVaralt2Multi <- reactive({
            
            
            myelements <- c(variablelinestouseMulti())
            
            
            if(is.null(myelements)){
                paste("Ca.K.alpha")
            }else{
                myelements[! myelements %in% c(input$calcurveelement_multi)]
            }
            
        })
        
        output$inVar2_multi <- renderUI({
            selectInput(inputId = "calcurveelement_multi", label = h4("Element"), choices =  outVarMulti())
        })
        
        inVar3SelectedMulti <- reactive({
            
            hold <- quantValues()
            
            optionhold <- if(is.null(input$calcurveelement_multi)){
                ls(hold[[input$defaultcal]])[2]
            } else if(!is.null(input$calcurveelement_multi)){
                input$calcurveelement_multi
            }
            
            calListMulti[[input$defaultcal]][["calList"]][[optionhold]][[1]]$Intercept
            
            
            
        })
        
        
        output$inVar3_multi <- renderUI({
            
            selectInput(inputId = "intercept_vars_multi", label = h4("Intercept"), choices =  outVaralt2Multi(), selected=inVar3SelectedMulti(), multiple=TRUE)
        })
        
        inVar4SelectedMulti <- reactive({
            
            hold <- quantValues()
            
            optionhold <- if(is.null(input$calcurveelement_multi)){
                ls(hold[[input$defaultcal]])[2]
            }else  if(!is.null(input$calcurveelement_multi)){
                input$calcurveelement_multi
            }
            
            
            calListMulti[[input$defaultcal]][["calList"]][[optionhold]][[1]]$Slope
            
        })
        
        output$inVar4_multi <- renderUI({
            selectInput(inputId = "slope_vars_multi", label = h4("Slope"), choices =  outVaraltMulti(), selected=inVar4SelectedMulti(), multiple=TRUE)
        })
        
        
        
        calTypeSelectionMulti <- reactive({
            
            hold <- quantValues()
            
            optionhold <- if(is.null(input$calcurveelement_multi)){
                ls(hold[[input$defaultcal]])[2]
            } else if(!is.null(input$calcurveelement_multi)){
                input$calcurveelement_multi
            }
            
            calListMulti[[input$defaultcal]][["calList"]][[optionhold]][[1]]$CalTable$CalType
            
            
        })
        
        calNormSelectionMulti <- reactive({
            
            hold <- quantValues()
            
            optionhold <- if(is.null(input$calcurveelement_multi)){
                ls(hold[[input$defaultcal]])[2]
            }else{
                input$calcurveelement_multi
            }
            
            
            calListMulti[[input$defaultcal]][["calList"]][[optionhold]][[1]]$CalTable$NormType
            
            
        })
        
        normMinSelectionMulti <- reactive({
            
            hold <- quantValues()
            
            optionhold <- if(is.null(input$calcurveelement_multi)){
                ls(hold[[input$defaultcal]])[2]
            }else{
                input$calcurveelement_multi
            }
            
            
            calListMulti[[input$defaultcal]][["calList"]][[optionhold]][[1]]$CalTable$Min
            
            
        })
        
        normMaxSelectionMulti <- reactive({
            
            hold <- quantValues()
            
            optionhold <- if(is.null(input$calcurveelement_multi)){
                ls(hold[[input$defaultcal]])[2]
            }else if(!is.null(input$calcurveelement_multi)){
                input$calcurveelement_multi
            }
            
            calListMulti[[input$defaultcal]][["calList"]][[optionhold]][[1]]$CalTable$Max
            
        })
        
        
        
        
        
        output$calTypeInput_multi <- renderUI({
            
            selectInput("radiocal_multi", label = "Calibration Curve",
            choices = list("Linear" = 1, "Non-Linear" = 2, "Lucas-Tooth" = 3, "Forest" = 4, "Rainforest" = 5),
            selected = calTypeSelectionMulti())
            
            
        })
        
        
        output$normTypeInput_multi <- renderUI({
            
            selectInput("normcal_multi", label = "Normalization",
            choices = list("Time" = 1, "Total Counts" = 2, "Compton" = 3),
            selected = calNormSelectionMulti())
            
            
        })
        
        
        output$comptonMinInput_multi <- renderUI({
            
            numericInput('comptonmin_multi', label=h6("Min"), step=0.001, value=normMinSelectionMulti(), min=0, max=50, width='30%')
            
        })
        
        output$comptonMaxInput_multi <- renderUI({
            
            numericInput('comptonmax_multi', label=h6("Max"), step=0.001, value=normMaxSelectionMulti(), min=0, max=50, width='30%')
            
        })
        
        

observeEvent(input$actionprocess2_multi, {

        
        calTypeMulti <- reactive({
            
            
            if(input$radiocal_multi==1){
                1
            } else if(input$radiocal_multi==2){
                2
            } else if(input$radiocal_multi==3){
                3
            } else if(input$radiocal_multi==4){
                3
            } else if(input$radiocal_multi==5){
                5
            }
            
        })
        
        
        
        
        elementHoldMulti <- reactive({
            
            if(is.null(input$calcurveelement_multi)==TRUE){
                 elementallinestouseMulti()[1]
            } else{
                input$calcurveelement_multi
            }
            
        })
        
        
        calFileStandardsMulti <- reactive({
            
            calstandardmulticheck <- function(cal, vals, element){
                if(is.null(cal[[element]][[1]][["StandardsUsed"]])){
                    as.vector(rep(TRUE, length(vals[element])))
                } else if(!is.null(cal[[element]][[1]][["StandardsUsed"]])){
                   cal[[element]][[1]][["StandardsUsed"]]
                }
                
            }
            
           cal.list <- quantCals()
           
           val.list <- quantValues()
           
           cal.names <- quantNames()
           
           n <- length(cal.names)
           
           #element.cal.meta <- lapply(cal.list, "[[", elementHoldMulti())
           #element.cal.meta2 <- lapply(element.cal.meta, "[[", 1)
           #element.cal.meta3 <- lapply(element.cal.meta2, "[[", "StandardsUsed")
           #element.cal.meta4 <- lapply(element.cal.meta3, as.vector)

           
           #names(element.cal.meta4) <- quantNames()
           #element.cal.meta4
           
           element.cal.meta <- lapply(quantNames(), function(x) calstandardmulticheck(cal=quantCals()[[x]], vals=quantValues()[[x]], element=input$calcurveelement_multi))
           names(element.cal.meta) <- quantNames()
           element.cal.meta
           
        })
        
        
        
        
        
        
        vals_multi <- reactiveValues()
        
        
        vals_multi$keeprows <- calFileStandardsMulti()
        
        
        
        
        concentrationTableMulti <- reactive({
            
            concentration.table <- quantValues()
            cal.names <- names(quantValues())
            
            index <- seq(from=1, to=length(cal.names), by=1)

            
            concentration.list <- lapply(quantNames(),function(x) as.data.frame(concentration.table[[x]][order(as.character(concentration.table[[x]][,"Spectrum"])),]))
            names(concentration.list) <- quantNames()
            
            #concentration.list <- lapply(concentration.list, as.data.frame)
            #names(concentration.list) <- quantNames()
            
            concentration.list
            
        })




        
        spectraLineTableMulti <- reactive({
            
            spectra.line.table <- quantIntensities()
            cal.names <- names(spectra.line.table)
            
            index <- seq(from=1, to=length(cal.names), by=1)
            
            spectra.line.list <- lapply(quantNames(),function(x) data.frame(Spectrum=quantValues()[[x]][,"Spectrum"], spectra.line.table[[x]]))
            
                names(spectra.line.list) <- quantNames()
            
            #spectra.line.table$Spectrum <- quantValues()$Spectrum
            
            spectra.line.list <- lapply(quantNames(), function(x) as.data.frame(spectra.line.list[[x]][order(as.character(spectra.line.list[[x]][,"Spectrum"])),]))
            names(spectra.line.list) <- quantNames()
            
            spectra.line.list

            
        })
        

        
        
        holdFrameMulti <- reactive({
            
            spectra.line.table <- spectraLineTableMulti()
            
            cal.names <- names(spectra.line.table)
            
            index <- seq(from=1, to=length(cal.names), by=1)
            
            concentration <- lapply(quantNames(),function(x) as.vector(as.numeric(unlist(concentrationTableMulti()[[x]][input$calcurveelement_multi]))))
            names(concentration) <- quantNames()

            
            
            intensity <- lapply(quantNames(),function(x) as.vector(as.numeric(unlist(spectraLineTableMulti()[[x]][input$calcurveelement_multi]))))
            names(intensity) <- quantNames()

            
            spectra.names <- lapply(quantNames(),function(x) as.vector(spectra.line.table[[x]][,"Spectrum"]))
            names(spectra.names) <- quantNames()
            
            hold.list <- lapply(quantNames(),function(x) data.frame(Spectrum=spectra.names[[x]], Concentration=concentration[[x]], Intensity=intensity[[x]]))
            names(hold.list) <- quantNames()
            
            hold.list <- lapply(quantNames(),function(x) hold.list[[x]][complete.cases(hold.list[[x]]),])
            names(hold.list) <- quantNames()
            
            hold.list <- lapply(quantNames(),function(x) na.omit(hold.list[[x]]))
            names(hold.list) <- quantNames()
            
            hold.list <- lapply(quantNames(),function(x) hold.list[[x]][order(as.character(hold.list[[x]][,"Spectrum"])),])
            names(hold.list) <- quantNames()
            

            #hold.frame <- data.frame(spectra.names, concentration, intensity, instrument.names)
            #colnames(hold.frame) <- c("Spectrum", "Concentration", "Intensity", "Instrument")
            #hold.frame <- na.omit(hold.frame)
            
            #hold.frame <- hold.frame[order(as.character(hold.frame$Instrument), as.character(hold.frame$Spectrum)),]
            
            
            #hold.frame
            hold.list
            
        })
        


        
        
        dataNormMulti <- reactive({
            
            data <- quantData()
            cal.names <- names(data)
            
            index <- seq(from=1, to=length(cal.names), by=1)
            
            data <- lapply(quantNames(),function(x) as.data.frame(data[[x]][data[[x]][,"Spectrum"] %in% holdFrameMulti()[[x]][,"Spectrum"], ]))
            names(data) <- quantNames()
            
            data

        #data[paste0(data$Spectrum, data$Instrument) %in% paste0(holdFrameMulti()$Spectrum, holdFrameMulti()$Instrument), ]
            
            
        })



        predictFramePreMulti <- reactive({
            

            
            concentration <- lapply(holdFrameMulti(), function(x) as.vector(x[,"Concentration"]))
            names(concentration) <- quantNames()
            
            intensity <- lapply(holdFrameMulti(), function(x) as.vector(x[,"Intensity"]))
            names(intensity) <- quantNames()

            predict.frame <- lapply(quantNames(),function(x) data.frame(Concentration=concentration[[x]], Intensity==intensity[[x]]))
            names(predict.frame) <- quantNames()

            #data.frame(concentration, intensity)
            #colnames(predict.frame) <- c("Concentration", "Intensity")
            
            
            predict.frame
            
            
        })
        


        predictIntensityMulti <- reactive({
            

            
            spectra.line.table <- spectraLineTableMulti()
            data <- dataNormMulti()
            
            
            #spectra.line.table <- spectra.line.table[paste0(spectra.line.table$Spectrum, spectra.line.table$Instrument) %in% paste0(holdFrameMulti()$Spectrum, holdFrameMulti()$Instrument), ]
            
            spectra.line.table <- lapply(quantNames(),function(x) spectra.line.table[[x]][spectra.line.table[[x]][,"Spectrum"] %in% holdFrameMulti()[[x]][,"Spectrum"], ])
            names(spectra.line.table) <- quantNames()

            
            
            if (input$radiocal_multi==1){
                
                if(input$normcal_multi==1){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) general_prep_xrf(spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) general_prep_xrf_net(spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi))
                    }
                }
                
                if(input$normcal_multi==2){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) simple_tc_prep_xrf(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) simple_tc_prep_xrf_net(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi))
                    }
                }
                
                if(input$normcal_multi==3){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) simple_comp_prep_xrf(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, norm.min=input$comptonmin_multi, norm.max=input$comptonmax_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) simple_comp_prep_xrf_net(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, norm.min=input$comptonmin_multi, norm.max=input$comptonmax_multi))
                    }
                }
                
            }
            
            
            if (input$radiocal_multi==2){
                
                if(input$normcal_multi==1){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) general_prep_xrf(spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) general_prep_xrf_net(spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi))
                    }
                }
                
                if(input$normcal_multi==2){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) simple_tc_prep_xrf(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) simple_tc_prep_xrf_net(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi))
                    }
                }
                
                if(input$normcal_multi==3){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) simple_comp_prep_xrf(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, norm.min=input$comptonmin_multi, norm.max=input$comptonmax_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) simple_comp_prep_xrf_net(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, norm.min=input$comptonmin_multi, norm.max=input$comptonmax_multi))
                    }
                }
                
            }
            
            
            if (input$radiocal_multi==3){
                
                if(input$normcal_multi==1){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) lucas_simp_prep_xrf(spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) lucas_simp_prep_xrf_net(spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi))
                    }
                }
                
                if(input$normcal_multi==2){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) lucas_tc_prep_xrf(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) lucas_tc_prep_xrf_net(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi))
                    }
                }
                
                if(input$normcal_multi==3){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) lucas_comp_prep_xrf(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi, norm.min=input$comptonmin_multi, norm.max=input$comptonmax_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) lucas_comp_prep_xrf_net(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi, norm.min=input$comptonmin_multi, norm.max=input$comptonmax_multi))
                    }
                }
                
            }
            
            if (input$radiocal_multi==4){
                
                if(input$normcal_multi==1){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) lucas_simp_prep_xrf(spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) lucas_simp_prep_xrf_net(spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi))
                    }
                }
                
                if(input$normcal_multi==2){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) lucas_tc_prep_xrf(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) lucas_tc_prep_xrf_net(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi))
                    }
                }
                
                if(input$normcal_multi==3){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(),function(x) lucas_comp_prep_xrf(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi, norm.min=input$comptonmin_multi, norm.max=input$comptonmax_multi))
                    } else if(dataTypeMulti()=="Net"){
                        lapply(quantNames(),function(x) lucas_comp_prep_xrf_net(data=data[[x]], spectra.line.table=spectra.line.table[[x]], element.line=input$calcurveelement_multi, slope.element.lines=input$slope_vars_multi, intercept.element.lines=input$intercept_vars_multi, norm.min=input$comptonmin_multi, norm.max=input$comptonmax_multi))
                    }
                }
                
            }
            
            
            if (input$radiocal_multi==5){
                
                if(input$normcal_multi==1){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                        lapply(quantNames(), function(x) spectra_simp_prep_xrf(spectra=data[[x]])[,-1])
                    } else if(dataTypeMulti()=="Net"){
                        NULL
                    }
                }
                
                if(input$normcal_multi==2){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                         lapply(quantNames(), function(x) spectra_tc_prep_xrf(spectra=data[[x]])[,-1])
                    } else if(dataTypeMulti()=="Net"){
                        NULL
                    }
                }
                
                if(input$normcal_multi==3){
                    predict.intensity <- if(dataTypeMulti()=="Spectra"){
                         lapply(quantNames(), function(x) spectra_comp_prep_xrf(spectra=data[[x]], norm.min=input$comptonmin_multi, norm.max=input$comptonmax_multi)[,-1])
                    } else if(dataTypeMulti()=="Net"){
                        NULL
                    }
                }
                
            }
            
            
            
            
            predict.intensity <- lapply(predict.intensity, as.data.frame)
            
            names(predict.intensity) <- quantNames()
            predict.intensity
            
            
        })
        
        output$moretesting <- renderDataTable({
            predictIntensityMulti()[[1]]
            
        })
        

        
        predictFrameMulti <- reactive({
            

            
            predict.frame <- predictFramePreMulti()
            predict.intensity <- predictIntensityMulti()
            
            
            
            predict.frame <- lapply(quantNames(),function(x) data.frame(predict.intensity[[x]], Concentration=predict.frame[[x]][,"Concentration"]))
            predict.frame <- lapply(predict.frame, as.data.frame)

            names(predict.frame) <- quantNames()
            
            predict.frame
            
            #data.frame(predict.intensity, predict.frame$Concentration)
            #colnames(predict.frame) <- c(names(predict.intensity), "Concentration")
            
            #predict.frame$Instrument <- holdFrameMulti()$Instrument
            
            #predict.frame
            
            
        })
        
        
    
        
        
        
        calCurveFrameMulti <- reactive({
            

            
            cal.frame <- predictFrameMulti()
            
            cal.frame <- lapply(quantNames(),function(x) data.frame(cal.frame[[x]], Instrument=x))
            names(cal.frame) <- quantNames()
            
            do.call("rbind", cal.frame)
            
            
        })
        
        
        
        elementModelMulti <- reactive({
            
            

            
            predict.list <- predictFrameMulti()
            
            
            
            if (input$radiocal_multi==1){
                cal.lm <- lapply(quantNames(),function(x) lm(Concentration~Intensity, data=predict.list[[x]][ vals_multi$keeprows[[x]], , drop = FALSE]))
            }
            
            
            if (input$radiocal_multi==2){
                cal.lm <- lapply(quantNames(), function(x) lm(Concentration~Intensity + I(Intensity^2), data=predict.list[[x]][ vals_multi$keeprows[[x]], , drop = FALSE]))
            }
            
            if (input$radiocal_multi==3){
                cal.lm <- lapply(quantNames(),function(x) lm(Concentration~., data=predict.list[[x]][ vals_multi$keeprows[[x]], , drop = FALSE]))
            }
            
            if (input$radiocal_multi==4){
                cal.lm <- lapply(quantNames(),function(x) randomForest(Concentration~., data=predict.list[[x]][ vals_multi$keeprows[[x]], , drop = FALSE], na.action=na.omit))
            }
            
            if (input$radiocal_multi==5){
                cal.lm <- lapply(quantNames(),function(x) randomForest(Concentration~., data=predict.list[[x]][ vals_multi$keeprows[[x]], , drop = FALSE], na.action=na.omit))
            }
            
            names(cal.lm) <- quantNames()

            
            cal.lm
            
        })
        
        
        
        
        valFrameMulti <- reactive({
            
            

            
            predict.intensity <- predictIntensityMulti()
            predict.frame <- predictFrameMulti()
            element.model <- elementModelMulti()
            
            
            if (input$radiocal_multi==1){
                cal.est.conc.pred <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred) <- quantNames()
                cal.est.conc.tab <- lapply(cal.est.conc.pred, data.frame)
                names(cal.est.conc.tab) <- quantNames()
                cal.est.conc <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"fit"]))
                names(cal.est.conc) <- quantNames()
                
                
                val.frame <- lapply(quantNames(),function(x) data.frame(Concentration=predict.frame[[x]][,"Concentration"], Prediction=cal.est.conc[[x]]))
                names(val.frame) <- quantNames()
                
                #data.frame(predict.frame$Concentration, cal.est.conc)
                #colnames(val.frame) <- c("Concentration", "Prediction")
            }
            
            if (input$radiocal_multi==2){
                cal.est.conc.pred <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred) <- quantNames()
                cal.est.conc.tab <- lapply(cal.est.conc.pred, data.frame)
                names(cal.est.conc.tab) <- quantNames()
                cal.est.conc <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"fit"]))
                names(cal.est.conc) <- quantNames()
                
                
                val.frame <- lapply(quantNames(),function(x) data.frame(Concentration=predict.frame[[x]][,"Concentration"], Prediction=cal.est.conc[[x]]))
                names(val.frame) <- quantNames()
                
                #data.frame(predict.frame$Concentration, cal.est.conc)
                #colnames(val.frame) <- c("Concentration", "Prediction")
            }
            
            if (input$radiocal_multi==3){
        
                yv=lapply(quantNames(),function(x) predict(element.model[[x]], newdata=predict.intensity[[x]]))
                names(yv) <- quantNames()

                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred.luc) <- quantNames()
                cal.est.conc.tab <- lapply(cal.est.conc.pred.luc, data.frame)
                names(cal.est.conc.tab) <- quantNames()
                cal.est.conc.luc <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"fit"]))
                names(cal.est.conc.luc) <- quantNames()
                cal.est.conc.luc.up <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"upr"]))
                names(cal.est.conc.luc.up) <- quantNames()
                cal.est.conc.luc.low <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"lwr"]))
                names(cal.est.conc.luc.low) <- quantNames()

                
                val.frame <- lapply(quantNames(),function(x)
                data.frame(
                Concentration=predict.frame[[x]][,"Concentration"],
                Intensity=predict.intensity[[x]],
                IntensityNorm=lucas.x[[x]],
                Prediction=cal.est.conc.luc[[x]],
                Upper=cal.est.conc.luc.up[[x]],
                Lower=cal.est.conc.luc.low[[x]]
                ))
                names(val.frame) <- quantNames()
                
                
                #val.frame <- data.frame(predict.frame$Concentration, predict.intensity$Intensity, lucas.x, cal.est.conc.luc, cal.est.conc.luc.up, cal.est.conc.luc.low)
                #colnames(val.frame) <- c("Concentration", "Intensity", "IntensityNorm", "Prediction", "Upper", "Lower")
            }
            
            
            
            
        
            
            if (input$radiocal_multi==4){
                
                yv=lapply(quantNames(),function(x) predict(element.model[[x]], newdata=predict.intensity[[x]]))
                names(yv) <- quantNames()
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred.luc) <- quantNames()
                cal.est.conc.luc <- lapply(cal.est.conc.pred.luc, function(x) as.vector(x))
                names(cal.est.conc.luc) <- quantNames()

                
                val.frame <- lapply(quantNames(),function(x)
                data.frame(
                Concentration=predict.frame[[x]][,"Concentration"],
                Intensity=predict.intensity[[x]],
                IntensityNorm=lucas.x[[x]],
                Prediction=cal.est.conc.luc[[x]]
                ))
                names(val.frame) <- quantNames()
                
                
                #val.frame <- data.frame(predict.frame$Concentration, predict.intensity$Intensity, lucas.x, cal.est.conc.luc, cal.est.conc.luc.up, cal.est.conc.luc.low)
                #colnames(val.frame) <- c("Concentration", "Intensity", "IntensityNorm", "Prediction", "Upper", "Lower")
            }
            
            
            if (input$radiocal_multi==5){
                
                yv=lapply(quantNames(),function(x) predict(element.model[[x]], newdata=predict.intensity[[x]]))
                names(yv) <- quantNames()
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred.luc) <- quantNames()
                cal.est.conc.luc <- lapply(cal.est.conc.pred.luc, function(x) as.vector(x))
                names(cal.est.conc.luc) <- quantNames()
                
                
                val.frame <- lapply(quantNames(),function(x)
                data.frame(
                Concentration=predict.frame[[x]][,"Concentration"],
                IntensityNorm=lucas.x[[x]],
                Prediction=cal.est.conc.luc[[x]]
                ))
                names(val.frame) <- quantNames()
                
                
                #val.frame <- data.frame(predict.frame$Concentration, predict.intensity$Intensity, lucas.x, cal.est.conc.luc, cal.est.conc.luc.up, cal.est.conc.luc.low)
                #colnames(val.frame) <- c("Concentration", "Intensity", "IntensityNorm", "Prediction", "Upper", "Lower")
            }
            
            
            
            val.frame <- lapply(quantNames(),function(x) data.frame(val.frame[[x]], Instrument=x))
            names(val.frame) <- quantNames()
            do.call("rbind", val.frame)
            
            #val.frame$Instrument <- holdFrameMulti()$Instrument
            
            
        })
        

        
        
        calValFrameMulti <- reactive({
            
            valFrameMulti()
            
        })
        
        
        rangescalcurve_multi <- reactiveValues(x = NULL, y = NULL)
        
        output$tabletest <- renderDataTable({
            as.data.frame(calCurveFrameMulti())
            
        })
        
        calCurvePlotMulti <- reactive({
            
            predict.frame <- calCurveFrameMulti()
            element.model <- elementModelMulti()[[input$defaultcal]]
            val.frame <- valFrameMulti()
            
            element.name <- gsub("[.]", "", substr(input$calcurveelement_multi, 1, 2))
            intens <- " Counts per Second"
            norma <- " Normalized"
            norma.comp <- " Compton Normalized"
            norma.tc <- " Valid Counts Normalized"
            conen <- " (%)"
            predi <- " Estimate (%)"
            log <- "Log "
            
            
            intensity.name <- c(element.name, intens)
            concentration.name <- c(element.name, conen)
            prediction.name <- c(element.name, predi)
            
            
            if(input$radiocal_multi==1){
                calcurve.plot <- ggplot(data=predict.frame[ unlist(vals_multi$keeprows), , drop = FALSE], aes(Intensity, Concentration, colour=Instrument, shape=Instrument)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~Intensity, predict.frame[ unlist(unlist(vals_multi$keeprows), use.names=FALSE), , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(data = predict.frame[!unlist(vals_multi$keeprows), , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
                stat_smooth(method="lm", fullrange = TRUE, aes(fill=Instrument), alpha=0.1) +
                scale_x_continuous(paste(element.name, intens)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurve_multi$x, ylim = rangescalcurve_multi$y, expand = TRUE)
                
            }
            
            if(input$radiocal_multi==2){
                calcurve.plot <- ggplot(data=predict.frame[ unlist(vals_multi$keeprows), , drop = FALSE], aes(Intensity, Concentration, colour=Instrument, shape=Instrument)) +
                theme_light() +
                annotate("text", label=lm_eqn_poly(lm(Concentration~Intensity + I(Intensity^2), predict.frame[ unlist(vals_multi$keeprows), , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(data = predict.frame[!unlist(vals_multi$keeprows), , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
                stat_smooth(method="lm", formula=y~poly(x,2), aes(fill=Instrument), alpha=0.1) +
                scale_x_continuous(paste(element.name, intens)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurve_multi$x, ylim = rangescalcurve_multi$y, expand = TRUE)
                
            }
            
            if(input$radiocal_multi==3){
                calcurve.plot <- ggplot(data=val.frame[ unlist(vals_multi$keeprows), , drop = FALSE], aes(IntensityNorm, Concentration, colour=Instrument, shape=Instrument)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~., val.frame[ unlist(vals_multi$keeprows), , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(aes(IntensityNorm, Concentration), data = val.frame[!unlist(vals_multi$keeprows), , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
                geom_smooth(aes(x=IntensityNorm, y=Concentration, ymin = Lower, ymax = Upper, fill=Instrument), alpha=0.1) +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurve_multi$x, ylim = rangescalcurve_multi$y, expand = TRUE)
                
            }
            
            
            if(input$radiocal_multi==4){
                calcurve.plot <- ggplot(data=val.frame[ unlist(vals_multi$keeprows), , drop = FALSE], aes(IntensityNorm, Concentration, colour=Instrument, shape=Instrument)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~., val.frame[ unlist(vals_multi$keeprows), , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(aes(IntensityNorm, Concentration), data = val.frame[!unlist(vals_multi$keeprows), , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
                geom_smooth(alpha=0.1) +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurve_multi$x, ylim = rangescalcurve_multi$y, expand = TRUE)
                
            }
            
            
            if(input$radiocal_multi==5){
                calcurve.plot <- ggplot(data=val.frame[ unlist(vals_multi$keeprows), , drop = FALSE], aes(IntensityNorm, Concentration, colour=Instrument, shape=Instrument)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~., val.frame[ unlist(vals_multi$keeprows), , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_point(aes(IntensityNorm, Concentration), data = val.frame[!unlist(vals_multi$keeprows), , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
                geom_smooth(alpha=0.1) +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurve_multi$x, ylim = rangescalcurve_multi$y, expand = TRUE)
                
            }
            
            
            
            calcurve.plot
            
            
        })
        
        
        
        observeEvent(input$cropcalmulti, {
            brush <- input$plot_cal_brush_multi
            if (!is.null(brush)) {
                rangescalcurve_multi$x <- c(brush$xmin, brush$xmax)
                rangescalcurve_multi$y <- c(brush$ymin, brush$ymax)
                
            } else {
                rangescalcurve_multi$x <- NULL
                rangescalcurve_multi$y <- NULL
            }
        })
        
        output$calcurveplots_multi <- renderPlot({
            calCurvePlotMulti()
        })
        
        
        rangesvalcurve_multi <- reactiveValues(x = NULL, y = NULL)
        
        
        valCurvePlotMulti <- reactive({
            
            predict.intensity <- predictIntensityMulti()
            predict.frame <- predictFrameMulti()
            element.model <- elementModelMulti()[[input$defaultcal]]
            
            
            
            element.name <- gsub("[.]", "", substr(input$calcurveelement_multi, 1, 2))
            intens <- " Counts per Second"
            norma <- " Normalized"
            norma.comp <- " Compton Normalized"
            norma.tc <- " Valid Counts Normalized"
            conen <- " (%)"
            predi <- " Estimate (%)"
            log <- "Log "
            
            intensity.name <- c(element.name, intens)
            concentration.name <- c(element.name, conen)
            prediction.name <- c(element.name, predi)
            val.frame <- valFrameMulti()
            
            
            valcurve.plot <- ggplot(data=val.frame[ unlist(vals_multi$keeprows), , drop = FALSE], aes(Prediction, Concentration, colour=Instrument, shape=Instrument)) +
            theme_bw() +
            annotate("text", label=lm_eqn_val(lm(Concentration~Prediction, val.frame[ unlist(vals_multi$keeprows), , drop = FALSE])), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
            geom_abline(intercept=0, slope=1, lty=2) +
            stat_smooth(method="lm", aes(fill=Instrument), alpha=0.1) +
            geom_point() +
            geom_point(aes(Prediction, Concentration),  data = val.frame[!unlist(vals_multi$keeprows), , drop = FALSE], shape = 21, fill = "red", color = "black", alpha = 0.25) +
            scale_x_continuous(paste(element.name, predi)) +
            scale_y_continuous(paste(element.name, conen)) +
            coord_cartesian(xlim = rangesvalcurve_multi$x, ylim = rangesvalcurve_multi$y, expand = TRUE)
            
            
            
            
            
            valcurve.plot
            
        })
        
        
        observeEvent(input$cropvalmulti, {
            brush <- input$plot_val_brush_multi
            if (!is.null(brush)) {
                rangesvalcurve_multi$x <- c(brush$xmin, brush$xmax)
                rangesvalcurve_multi$y <- c(brush$ymin, brush$ymax)
                
            } else {
                rangesvalcurve_multi$x <- NULL
                rangesvalcurve_multi$y <- NULL
            }
        })
        
        
        output$valcurveplots_multi <- renderPlot({
            valCurvePlotMulti()
        })
        
        

        
        
        calValTableMulti <- reactive({
            
            standard.table <- valFrameMulti()
            
            concentration.table <- do.call("rbind", holdFrameMulti())
            
            hold.table <- as.vector(concentration.table[,"Spectrum"])
            
            
            standard.table$Spectrum <- hold.table

            
            standard.table.summary <- data.frame(standard.table$Instrument, standard.table$Spectrum, standard.table$Concentration, standard.table$Prediction, standard.table$Concentration-standard.table$Prediction, ((standard.table$Concentration-standard.table$Prediction)/standard.table$Concentration))
            colnames(standard.table.summary) <- c("Instrument", "Standard", "Concentration", "Prediction", "Difference", "Relative")
            
            standard.table.summary[,3:length(standard.table.summary)] <-round(standard.table.summary[,3:length(standard.table.summary)],4)
            standard.table.summary[,5] <- as.character(percent(standard.table.summary[,5]))
            
            this.table <- DT::datatable(standard.table.summary)
            this.table
            
        })
        
        
        output$standardsperformance_multi <- DT::renderDataTable({
            
            
            standard.table <- if(input$switchmulti==FALSE){
                calValTableMulti()
            } else if(input$switchmulti==TRUE){
                calValTableRandomMulti()
            }
            
            standard.table
            
        }, options =list(aoColumnDefs = list(list(sClass="alignRight",aTargets=c(list(2), list(3),list(4),list(5))))  ))
        
        
        randomizeDataMulti <- reactive({
            
            cal.frame <- spectraLineTableMulti()[[input$defaultcal]]
            cal.frame <- cal.frame[ vals_multi$keeprows[[input$defaultcal]], , drop = FALSE]
            total.number <- length(cal.frame[,1])
            sample.number <- total.number-round(input$percentrandom_multi*total.number, 0)
            
            hold <- cal.frame[sample(nrow(cal.frame), sample.number),]
            cal.frame$Spectrum %in% hold$Spectrum
            
        })
        
        
        
        standardNamesRandomized <- reactive({
            
            cal.frame <- spectraLineTableMulti()[[input$defaultcal]]
            cal.frame <- cal.frame[ vals_multi$keeprows[[input$defaultcal]], , drop = FALSE]
            total.number <- length(cal.frame[,1])
            sample.number <- total.number-round(input$percentrandom_multi*total.number, 0)
            
            hold <- cal.frame[randomizeDataMulti(),]
            cal.frame$Spectrum[cal.frame$Spectrum %in% hold$Spectrum]
            
        })
        
        
        
        

        
        
        
        predictFrameRandomMulti <- reactive({
            
            predict.frame <- predictFrameMulti()
            

            
            
            predict.frame <- lapply(quantNames(),function(x) as.data.frame(predict.frame[[x]][ vals_multi$keeprows[[x]], ]))
            names(predict.frame) <- quantNames()
            
            predict.frame <- lapply(quantNames(),function(x) as.data.frame(predict.frame[[x]][randomizeDataMulti(),]))
            names(predict.frame) <- quantNames()
            
            
            predict.frame
            
        })

        
        calCurveFrameRandomizedMulti <- reactive({
            

            
            predict.frame <- predictFrameRandomMulti()
            
            predict.frame <- lapply(quantNames(),function(x) data.frame(predict.frame[[x]], Instrument=x))
            names(predict.frame) <- quantNames()
            
            
            do.call("rbind", predict.frame)

            
        })
        

        
        
        elementModelRandomMulti <- reactive({
            
            predict.frame <- predictFrameRandomMulti()
            
            

            
            predict.list <- predict.frame
            
            
            
            if (input$radiocal_multi==1){
                cal.lm <- lapply(quantNames(),function(x) lm(Concentration~Intensity, data=predict.list[[x]]))
            }
            
            
            if (input$radiocal_multi==2){
                cal.lm <- lapply(quantNames(), function(x) lm(Concentration~Intensity + I(Intensity^2), data=predict.list[[x]]))
            }
            
            if (input$radiocal_multi==3){
                cal.lm <- lapply(quantNames(),function(x) lm(Concentration~., data=predict.list[[x]]))
            }
            
            if (input$radiocal_multi==4){
                cal.lm <- lapply(quantNames(),function(x) randomForest(Concentration~., data=predict.list[[x]], na.action=na.omit))
            }
            
            if (input$radiocal_multi==5){
                cal.lm <- lapply(quantNames(),function(x) randomForest(Concentration~., data=predict.list[[x]], na.action=na.omit))
            }
            
            names(cal.lm) <- quantNames()
            
            
            cal.lm
            
        })
        
        
        
        
        valFrameRandomizedMulti <- reactive({
            
            concentration.table.rev <- predictFrameRandomMulti()
            


            
            predict.intensity <- lapply(quantNames(),function(x) data.frame(predictIntensityMulti()[[x]][ vals_multi$keeprows[[x]], , drop = FALSE]))
            names(predict.intensity) <- quantNames()

            
            predict.frame <- lapply(quantNames(),function(x) data.frame(predictFrameMulti()[[x]][ vals_multi$keeprows[[x]], , drop = FALSE]))
            names(predict.frame) <- quantNames()

            
            predict.intensity <- lapply(quantNames(),function(x) data.frame( predict.intensity[[x]][!(randomizeDataMulti()), , drop = FALSE]))
            names(predict.intensity) <- quantNames()

            
            predict.frame <- lapply(quantNames(),function(x) data.frame( predict.frame[[x]][!(randomizeDataMulti()), , drop = FALSE]))
            names(predict.frame) <- quantNames()
            

            element.model <- elementModelRandomMulti()
            
            
            
            if (input$radiocal_multi==1){
                cal.est.conc.pred <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred) <- quantNames()

                
                cal.est.conc.tab <- lapply(cal.est.conc.pred, data.frame)
                names(cal.est.conc.tab) <- quantNames()

                cal.est.conc <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"fit"]))
                names(cal.est.conc) <- quantNames()


                val.frame <- lapply(quantNames(),function(x) data.frame(Concentration=predict.frame[[x]][,"Concentration"], Prediction=cal.est.conc[[x]]))
                names(val.frame) <- quantNames()
            }
            
            if (input$radiocal_multi==2){
                cal.est.conc.pred <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred) <- quantNames()
                
                
                cal.est.conc.tab <- lapply(cal.est.conc.pred, data.frame)
                names(cal.est.conc.tab) <- quantNames()
                
                cal.est.conc <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"fit"]))
                names(cal.est.conc) <- quantNames()
                
                
                val.frame <- lapply(quantNames(),function(x) data.frame(Concentration=predict.frame[[x]][,"Concentration"], Prediction=cal.est.conc[[x]]))
                names(val.frame) <- quantNames()
            }
            
            if (input$radiocal_multi==3){
                
                yv=lapply(quantNames(),function(x) predict(element.model[[x]], newdata=predict.intensity[[x]]))
                names(yv) <- quantNames()

                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred.luc) <- quantNames()

                cal.est.conc.tab <- lapply(cal.est.conc.pred.luc, data.frame)
                names(cal.est.conc.tab) <- quantNames()

                cal.est.conc.luc <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"fit"]))
                names(cal.est.conc.luc) <- quantNames()

                cal.est.conc.luc.up <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"upr"]))
                names(cal.est.conc.luc.up) <- quantNames()

                cal.est.conc.luc.low <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"lwr"]))
                names(cal.est.conc.luc.low) <- quantNames()

                
                val.frame <- lapply(quantNames(),function(x)
                data.frame(
                Concentration=predict.frame[[x]][,"Concentration"],
                Intensity=predict.intensity[[x]],
                IntensityNorm=lucas.x[[x]],
                Prediction=cal.est.conc.luc[[x]],
                Upper=cal.est.conc.luc.up[[x]],
                Lower=cal.est.conc.luc.low[[x]]
                ))
                
                names(val.frame) <- quantNames()
            }
            
            
            
            if (input$radiocal_multi==4){
                
                yv=lapply(quantNames(),function(x) predict(element.model[[x]], newdata=predict.intensity[[x]]))
                names(yv) <- quantNames()
                
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred.luc) <- quantNames()
                
                cal.est.conc.luc <- lapply(cal.est.conc.pred.luc, function(x) as.vector(x))
                names(cal.est.conc.luc) <- quantNames()


                
                
                val.frame <- lapply(quantNames(),function(x)
                data.frame(
                Concentration=predict.frame[[x]][,"Concentration"],
                Intensity=predict.intensity[[x]],
                IntensityNorm=lucas.x[[x]],
                Prediction=cal.est.conc.luc[[x]]
                ))
                
                names(val.frame) <- quantNames()
            }
            
            
            
            if (input$radiocal_multi==5){
                
                yv=lapply(quantNames(),function(x) predict(element.model[[x]], newdata=predict.intensity[[x]]))
                names(yv) <- quantNames()
                
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred.luc) <- quantNames()
                
                cal.est.conc.luc <- lapply(cal.est.conc.pred.luc, function(x) as.vector(x))
                names(cal.est.conc.luc) <- quantNames()
                
                
                
                
                val.frame <- lapply(quantNames(),function(x)
                data.frame(
                Concentration=predict.frame[[x]][,"Concentration"],
                IntensityNorm=lucas.x[[x]],
                Prediction=cal.est.conc.luc[[x]]
                ))
                
                names(val.frame) <- quantNames()
            }
            
            
            
            
            val.frame <- lapply(quantNames(),function(x) data.frame(val.frame[[x]], Instrument=x))
            names(val.frame) <- quantNames()
            
            val.frame <- lapply(quantNames(), function(x) as.data.frame(
            val.frame[[x]][val.frame[[x]][, "Concentration"] > min(concentration.table.rev[[x]][,"Concentration"], na.rm = TRUE) & val.frame[[x]][, "Concentration"] < max(concentration.table.rev[[x]][,"Concentration"], na.rm = TRUE), ]))
            names(val.frame) <- quantNames()

            
            do.call("rbind", val.frame)

        })
        
        
        
        valFrameRandomizedRevMulti <- reactive({
            

            
            
            predict.intensity <- lapply(quantNames(),function(x) data.frame(predictIntensityMulti()[[x]][ vals_multi$keeprows[[x]], , drop = FALSE]))
            names(predict.intensity) <- quantNames()

            
            predict.frame <- lapply(quantNames(),function(x) data.frame(predictFrameMulti()[[x]][ vals_multi$keeprows[[x]], , drop = FALSE]))
            names(predict.frame) <- quantNames()

            
            predict.intensity <- lapply(quantNames(),function(x) data.frame( predict.intensity[[x]][(randomizeDataMulti()), , drop = FALSE]))
            names(predict.intensity) <- quantNames()

            predict.frame <- lapply(quantNames(),function(x) data.frame( predict.frame[[x]][(randomizeDataMulti()), , drop = FALSE]))
            names(predict.frame) <- quantNames()

            element.model <- elementModelRandomMulti()
            
            
            
            if (input$radiocal_multi==1){
                cal.est.conc.pred <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred) <- quantNames()

                cal.est.conc.tab <- lapply(cal.est.conc.pred, data.frame)
                names(cal.est.conc.tab) <- quantNames()

                cal.est.conc <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"fit"]))
                names(cal.est.conc) <- quantNames()

                
                val.frame <- lapply(quantNames(),function(x) data.frame(Concentration=predict.frame[[x]][,"Concentration"], Prediction=cal.est.conc[[x]]))
                names(val.frame) <- quantNames()
            }
            
            if (input$radiocal_multi==2){
                cal.est.conc.pred <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred) <- quantNames()
                
                cal.est.conc.tab <- lapply(cal.est.conc.pred, data.frame)
                names(cal.est.conc.tab) <- quantNames()
                
                cal.est.conc <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"fit"]))
                names(cal.est.conc) <- quantNames()
                
                
                val.frame <- lapply(quantNames(),function(x) data.frame(Concentration=predict.frame[[x]][,"Concentration"], Prediction=cal.est.conc[[x]]))
                names(val.frame) <- quantNames()
            }
            
            if (input$radiocal_multi==3){
                
                yv=lapply(quantNames(),function(x) predict(element.model[[x]], newdata=predict.intensity[[x]]))
                names(yv) <- quantNames()

                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred.luc) <- quantNames()

                cal.est.conc.tab <- lapply(cal.est.conc.pred.luc, data.frame)
                names(cal.est.conc.tab) <- quantNames()

                cal.est.conc.luc <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"fit"]))
                names(cal.est.conc.luc) <- quantNames()

                cal.est.conc.luc.up <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"upr"]))
                names(cal.est.conc.luc.up) <- quantNames()

                cal.est.conc.luc.low <- lapply(cal.est.conc.tab, function(x) as.vector(x[,"lwr"]))
                names(cal.est.conc.luc.low) <- quantNames()

                
                val.frame <- lapply(quantNames(),function(x)
                data.frame(
                Concentration=predict.frame[[x]][,"Concentration"],
                Intensity=predict.intensity[[x]],
                IntensityNorm=lucas.x[[x]],
                Prediction=cal.est.conc.luc[[x]],
                Upper=cal.est.conc.luc.up[[x]],
                Lower=cal.est.conc.luc.low[[x]]
                ))
                
                names(val.frame) <- quantNames()
            }
            
            
            if (input$radiocal_multi==4){
                
                yv=lapply(quantNames(),function(x) predict(element.model[[x]], newdata=predict.intensity[[x]]))
                names(yv) <- quantNames()
                
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred.luc) <- quantNames()
                
                cal.est.conc.luc <- lapply(cal.est.conc.pred.luc, function(x) as.vector(x))
                names(cal.est.conc.luc) <- quantNames()


                
                
                val.frame <- lapply(quantNames(),function(x)
                data.frame(
                Concentration=predict.frame[[x]][,"Concentration"],
                Intensity=predict.intensity[[x]],
                IntensityNorm=lucas.x[[x]],
                Prediction=cal.est.conc.luc[[x]]
                ))
                
                names(val.frame) <- quantNames()
            }
            
            
            if (input$radiocal_multi==5){
                
                yv=lapply(quantNames(),function(x) predict(element.model[[x]], newdata=predict.intensity[[x]]))
                names(yv) <- quantNames()
                
                
                
                lucas.x <- yv
                
                cal.est.conc.pred.luc <- lapply(quantNames(),function(x) predict(object=element.model[[x]], newdata=predict.intensity[[x]], interval='confidence'))
                names(cal.est.conc.pred.luc) <- quantNames()
                
                cal.est.conc.luc <- lapply(cal.est.conc.pred.luc, function(x) as.vector(x))
                names(cal.est.conc.luc) <- quantNames()
                
                
                
                
                val.frame <- lapply(quantNames(),function(x)
                data.frame(
                Concentration=predict.frame[[x]][,"Concentration"],
                IntensityNorm=lucas.x[[x]],
                Prediction=cal.est.conc.luc[[x]]
                ))
                
                names(val.frame) <- quantNames()
            }
            
            
            
            
            val.frame <- lapply(quantNames(),function(x) data.frame(val.frame[[x]], Instrument=x))
            names(val.frame) <- quantNames()
            do.call("rbind", val.frame)
            
        })
        
        
        rangescalcurverandom_multi <- reactiveValues(x = NULL, y = NULL)
        
        
        calCurvePlotRandomMulti <- reactive({
            
            predict.frame <- calCurveFrameRandomizedMulti()
            element.model <- elementModelRandomMulti()[[input$defaultcal]]
            
            
            
            element.name <- gsub("[.]", "", substr(input$calcurveelement_multi, 1, 2))
            intens <- " Counts per Second"
            norma <- " Normalized"
            norma.comp <- " Compton Normalized"
            norma.tc <- " Valid Counts Normalized"
            conen <- " (%)"
            predi <- " Estimate (%)"
            
            intensity.name <- c(element.name, intens)
            concentration.name <- c(element.name, conen)
            prediction.name <- c(element.name, predi)
            
            
            if(input$radiocal_multi==1){
                calcurve.plot <- ggplot(data=predict.frame, aes(Intensity, Concentration, colour=Instrument, shape=Instrument)) +
                theme_light() +
                annotate("text", label=lm_eqn(element.model), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                stat_smooth(method="lm", fullrange = TRUE, aes(fill=Instrument), alpha=0.1) +
                scale_x_continuous(paste(element.name, intens)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurverandom_multi$x, ylim = rangescalcurverandom_multi$y, expand = TRUE)
                
            }
            
            if(input$radiocal_multi==2){
                
                calcurve.plot <- ggplot(data=predict.frame, aes(Intensity, Concentration, colour=Instrument, shape=Instrument)) +
                theme_light() +
                annotate("text", label=lm_eqn_poly(element.model), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                stat_smooth(method="lm", formula=y~poly(x,2), aes(fill=Instrument), alpha=0.1) +
                scale_x_continuous(paste(element.name, intens)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurverandom_multi$x, ylim = rangescalcurverandom_multi$y, expand = TRUE)
            }
            
            if(input$radiocal_multi==3){
                val.frame <- valFrameRandomizedRevMulti()

                calcurve.plot <- ggplot(data=val.frame, aes(IntensityNorm, Concentration, colour=Instrument, shape=Instrument)) +
                theme_light() +
                annotate("text", label=lm_eqn(element.model), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_smooth(aes(x=IntensityNorm, y=Concentration, ymin = Lower, ymax = Upper, fill=Instrument), alpha=0.1) +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurverandom_multi$x, ylim = rangescalcurverandom_multi$y, expand = TRUE)
            }
            
            
            if(input$radiocal_multi==4){
                val.frame <- valFrameRandomizedRevMulti()
                
                calcurve.plot <- ggplot(data=val.frame, aes(IntensityNorm, Concentration, colour=Instrument, shape=Instrument)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~., val.frame)), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_smooth(alpha=0.1) +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurverandom_multi$x, ylim = rangescalcurverandom_multi$y, expand = TRUE)
            }
            
            
            if(input$radiocal_multi==5){
                val.frame <- valFrameRandomizedRevMulti()
                
                calcurve.plot <- ggplot(data=val.frame, aes(IntensityNorm, Concentration, colour=Instrument, shape=Instrument)) +
                theme_light() +
                annotate("text", label=lm_eqn(lm(Concentration~., val.frame)), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
                geom_point() +
                geom_smooth(alpha=0.1) +
                scale_x_continuous(paste(element.name, norma)) +
                scale_y_continuous(paste(element.name, conen)) +
                coord_cartesian(xlim = rangescalcurverandom_multi$x, ylim = rangescalcurverandom_multi$y, expand = TRUE)
            }
            
            calcurve.plot
            
            
        })
        
        
        
        calValTableRandomMulti <- reactive({
            
            standard.table <- valFrameRandomizedMulti()
            
            cal.frame <- spectraLineTableMulti()[[input$defaultcal]]
            cal.frame <- cal.frame[ vals_multi$keeprows[[input$defaultcal]], , drop = FALSE]
            total.number <- length(cal.frame[,1])
            sample.number <- total.number-round(input$percentrandom_multi*total.number, 0)
            
            
            concentration.table.rev <- predictFrameRandomMulti()
            
            
            predict.frame <- lapply(quantNames(),function(x) data.frame(holdFrameMulti()[[x]][ vals_multi$keeprows[[x]], , drop = FALSE]))
            names(predict.frame) <- quantNames()
            
            
            predict.frame <- lapply(quantNames(),function(x) data.frame( predict.frame[[x]][!(randomizeDataMulti()), , drop = FALSE]))
            names(predict.frame) <- quantNames()
            
            predict.frame <- lapply(quantNames(), function(x) as.data.frame(
            predict.frame[[x]][predict.frame[[x]][, "Concentration"] > min(concentration.table.rev[[x]][,"Concentration"], na.rm = TRUE) & predict.frame[[x]][, "Concentration"] < max(concentration.table.rev[[x]][,"Concentration"], na.rm = TRUE), ]))
            names(predict.frame) <- quantNames()
            
            hold.table <- do.call("rbind", predict.frame)
            
            
            standard.table$Spectrum <- hold.table[,"Spectrum"]
            
            standard.table.summary <- data.frame(standard.table$Instrument, standard.table$Spectrum, standard.table$Concentration, standard.table$Prediction, standard.table$Concentration-standard.table$Prediction, ((standard.table$Concentration-standard.table$Prediction)/standard.table$Concentration))
            colnames(standard.table.summary) <- c("Instrument", "Standard", "Concentration", "Prediction", "Difference", "Relative")
            
            standard.table.summary[,3:length(standard.table.summary)] <-round(standard.table.summary[,3:length(standard.table.summary)],4)
            standard.table.summary[,5] <- as.character(percent(standard.table.summary[,5]))
            
            this.table <- DT::datatable(standard.table.summary)
            this.table
            
        })
        
        
        observeEvent(input$cropcalmultirandom, {
            brush <- input$plot_cal_brush_random_multi
            if (!is.null(brush)) {
                rangescalcurverandom_multi$x <- c(brush$xmin, brush$xmax)
                rangescalcurverandom_multi$y <- c(brush$ymin, brush$ymax)
                
            } else {
                rangescalcurverandom_multi$x <- NULL
                rangescalcurverandom_multi$y <- NULL
            }
        })
        
        
        output$calcurveplotsrandom_multi <- renderPlot({
            calCurvePlotRandomMulti()
        })
        
        
        rangesvalcurverandom_multi <- reactiveValues(x = NULL, y = NULL)
        
        
        valCurvePlotRandomMulti <- reactive({
            
            
            
            element.name <- gsub("[.]", "", substr(input$calcurveelement_multi, 1, 2))
            intens <- " Counts per Second"
            norma <- " Normalized"
            norma.comp <- " Compton Normalized"
            norma.tc <- " Valid Counts Normalized"
            conen <- " (%)"
            predi <- " Estimate (%)"
            
            intensity.name <- c(element.name, intens)
            concentration.name <- c(element.name, conen)
            prediction.name <- c(element.name, predi)
            
            val.frame <- valFrameRandomizedMulti()
            
            valcurve.plot <- ggplot(data=val.frame, aes(Prediction, Concentration, colour=Instrument, shape=Instrument)) +
            theme_bw() +
            annotate("text", label=lm_eqn_val(lm(Concentration~Prediction, val.frame)), x=0, y=Inf, hjust=0, vjust=1, parse=TRUE)+
            geom_abline(intercept=0, slope=1, lty=2) +
            stat_smooth(method="lm", aes(fill=Instrument), alpha=0.1) +
            geom_point() +
            scale_x_continuous(paste(element.name, predi)) +
            scale_y_continuous(paste(element.name, conen)) +
            coord_cartesian(xlim = rangesvalcurverandom_multi$x, ylim = rangesvalcurverandom_multi$y, expand = TRUE)
            
            valcurve.plot
            
        })
        
        observeEvent(input$cropvalmultirandom, {
            brush <- input$plot_val_brush_random_multi
            if (!is.null(brush)) {
                rangesvalcurverandom_multi$x <- c(brush$xmin, brush$xmax)
                rangesvalcurverandom_multi$y <- c(brush$ymin, brush$ymax)
                
            } else {
                rangesvalcurverandom_multi$x <- NULL
                rangesvalcurverandom_multi$y <- NULL
            }
        })
        
        output$valcurveplotsrandom_multi <- renderPlot({
            valCurvePlotRandomMulti()
        })
        
        
        
        ####CalCurves
        
        # Float over info
        output$hover_infocal_multi <- renderUI({
            
            point.table <- if(calTypeMulti()==1){
                calCurveFrameMulti()
            } else if(calTypeMulti()==2){
                calCurveFrameMulti()
            } else if(calTypeMulti()==3) {
                calValFrameMulti()
            } else if(calTypeMulti()==5) {
                calValFrameMulti()
            }
            
            
            concentration.table <- do.call("rbind", holdFrameMulti())
            
            hold.table <- as.vector(concentration.table[,"Spectrum"])
            
            
            point.table$Spectrum <- hold.table
           
            
            #point.table$Spectrum <- hold.table["Spectrum"]
            
            hover <- input$plot_hovercal_multi
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Instrument))),
            p(HTML(paste0(point$Spectrum)))

            )
        })
        
        
        
        # Float over info
        output$hover_infocal_random_multi <- renderUI({
            

            
            point.table <- if(calTypeMulti()==1){
                calCurveFrameRandomizedMulti()
            } else if(calTypeMulti()==2){
                calCurveFrameRandomizedMulti()
            } else if(calTypeMulti()==3) {
                valFrameRandomizedRevMulti()
            } else if(calTypeMulti()==5) {
                valFrameRandomizedRevMulti()
            }
            
            
            
            predict.frame <- lapply(quantNames(),function(x) data.frame(spectraLineTableMulti()[[x]][ vals_multi$keeprows[[x]], ]))
            names(predict.frame) <- quantNames()
            
            
            predict.frame <- lapply(quantNames(),function(x) data.frame( predict.frame[[x]][(randomizeDataMulti()), ]))
            names(predict.frame) <- quantNames()
            
           
            
            concentration.table <- do.call("rbind", predict.frame)

            #concentration.table <- concentration.table[ unlist(vals_multi$keeprows), , drop = FALSE]
            #concentration.table <- concentration.table[(randomizeDataMulti()),]
            
            hold.table <- as.vector(concentration.table[,"Spectrum"])
            
            
            point.table$Spectrum <- hold.table

            
            
            
            #point.table$Spectrum <- hold.table["Spectrum"]
            
            hover <- input$plot_hovercal_random_multi
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Instrument))),
            p(HTML(paste0(point$Spectrum)))
            
            )
        })
        
        
        
        # Toggle points that are clicked
        observeEvent(input$plot_cal_click_multi, {
            
            predict.frame <- if(calTypeMulti()==1){
                calCurveFrameMulti()
            } else if(calTypeMulti()==2){
                calCurveFrameMulti()
            } else if(calTypeMulti()==3) {
                calValFrameMulti()
            } else if(calTypeMulti()==5) {
                calValFrameMulti()
            }
            
            res <- nearPoints(predict.frame, input$plot_cal_click_multi, allRows = TRUE)
            
            temprows <- xor(unlist(vals_multi$keeprows), res$selected_)
            
            vals_multi$keeprows <- relist(flesh=temprows, skeleton=vals_multi$keeprows)
        })
        
        
        
        # Toggle points that are brushed, when button is clicked
        observeEvent(input$exclude_toggle_multi, {
            
            predict.frame <- if(calTypeMulti()==1){
                calCurveFrameMulti()
            } else if(calTypeMulti()==2){
                calCurveFrameMulti()
            } else if(calTypeMulti()==3) {
                calValFrameMulti()
            } else if(calTypeMulti()==5) {
                calValFrameMulti()
            }
            res <- brushedPoints(predict.frame, input$plot_cal_brush_multi, allRows = TRUE)
            
            temprows <- xor(unlist(vals_multi$keeprows), res$selected_)
            
            vals_multi$keeprows <- relist(flesh=temprows, skeleton=vals_multi$keeprows)        })
        
        
        
        # Reset all points
        observeEvent(input$exclude_reset_multi, {
            
            predict.frame <- if(calTypeMulti()==1){
                calCurveFrameMulti()
            } else if(calTypeMulti()==2){
                calCurveFrameMulti()
            } else if(calTypeMulti()==3) {
                calValFrameMulti()
            } else if(calTypeMulti()==5) {
                calValFrameMulti()
            }
            vals_multi$keeprows <- calFileStandardsMulti()
        })
        
        
        
        # Reset all points on element change
        observeEvent(input$calcurveelement_multi, {
            
            
            
            vals_multi$keeprows <- calFileStandardsMulti()
            
        })
        
        
        
        ####ValCurves Multiplot
        
        
        
        
        # Float over info
        output$hover_infoval_multi <- renderUI({
            
            point.table <- valFrameMulti()
            concentration.table.rev <- predictFrameRandomMulti()

            
            concentration.table <- do.call("rbind", holdFrameMulti())
            
            hold.table <- as.vector(concentration.table[,"Spectrum"])
            
            
            point.table$Spectrum <- hold.table

            
            
            hover <- input$plot_hoverval_multi
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Instrument))),
            p(HTML(paste0(point$Spectrum)))
            
            )
        })
        
        
        
        
        output$hover_infoval_random_multi <- renderUI({
            
            point.table <- valFrameRandomizedMulti()
            
            concentration.table.rev <- predictFrameRandomMulti()

            
            predict.frame <- lapply(quantNames(),function(x) data.frame(holdFrameMulti()[[x]][ vals_multi$keeprows[[x]], , drop = FALSE]))
            names(predict.frame) <- quantNames()
            
            
            predict.frame <- lapply(quantNames(),function(x) data.frame( predict.frame[[x]][!(randomizeDataMulti()), , drop = FALSE]))
            names(predict.frame) <- quantNames()
            
            predict.frame <- lapply(quantNames(), function(x) as.data.frame(
            predict.frame[[x]][predict.frame[[x]][, "Concentration"] > min(concentration.table.rev[[x]][,"Concentration"], na.rm = TRUE) & predict.frame[[x]][, "Concentration"] < max(concentration.table.rev[[x]][,"Concentration"], na.rm = TRUE), ]))
            names(predict.frame) <- quantNames()
            
            hold.table <- do.call("rbind", predict.frame)
            
            
            point.table$Spectrum <- hold.table[,"Spectrum"]
            
            
            hover <- input$plot_hoverval_random_multi
            point <- nearPoints(point.table,  coordinfo=hover,   threshold = 5, maxpoints = 1, addDist = TRUE)
            if (nrow(point) == 0) return(NULL)
            
            
            
            
            # calculate point position INSIDE the image as percent of total dimensions
            # from left (horizontal) and from top (vertical)
            left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
            top_pct <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
            
            # calculate distance from left and bottom side of the picture in pixels
            left_px <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
            top_px <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)
            
            
            # create style property fot tooltip
            # background color is set so tooltip is a bit transparent
            # z-index is set so we are sure are tooltip will be on top
            style <- paste0("position:absolute; z-index:100; background-color: rgba(245, 245, 245, 0.85); ",
            "left:", left_px + 2, "px; top:", top_px + 2, "px;")
            
            # actual tooltip created as wellPanel
            wellPanel(
            style = style,
            p(HTML(paste0(point$Instrument))),
            p(HTML(paste0(point$Spectrum)))
            
            )
        })
        
        
        
        # Toggle points that are clicked
        observeEvent(input$plot_val_click_multi, {
            
            predict.frame <- calValFrameMulti()
            
            res <- nearPoints(predict.frame, input$plot_val_click_multi, allRows = TRUE)
            
            temprows <- xor(unlist(vals_multi$keeprows), res$selected_)
            
            vals_multi$keeprows <- relist(flesh=temprows, skeleton=vals_multi$keeprows)        })
        
        
        
        # Toggle points that are brushed, when button is clicked
        observeEvent(input$exclude_toggle_multi, {
            predict.frame <- calValFrameMulti()
            
            res <- brushedPoints(predict.frame, input$plot_val_brush_multi, allRows = TRUE)
            
            temprows <- xor(unlist(vals_multi$keeprows), res$selected_)
            
            vals_multi$keeprows <- relist(flesh=temprows, skeleton=vals_multi$keeprows)        })
        
        
        
        # Reset all points
        observeEvent(input$exclude_reset_multi, {
            predict.frame <- calValFrameMulti()
            
            vals_multi$keeprows <- calFileStandardsMulti()
        })
        
        
        calPlotDownloadMulti <- reactive({
            
            if(input$switchmulti==FALSE){
                grid.arrange(calCurvePlotMulti(), valCurvePlotMulti(), ncol=2)
            }else if(input$switchmulti==TRUE){
                grid.arrange(calCurvePlotRandomMulti(), valCurvePlotRandomMulti(), ncol=2)
                
            }
            
            
        })
        
        
        output$downloadcloudplot_multi <- downloadHandler(
        filename = function() { paste(paste(c(input$calname, "_", input$calcurveelement_multi), collapse=''), '.tiff',  sep='') },
        content = function(file) {
            ggsave(file,calPlotDownloadMulti(), device="tiff", compression="lzw", type="cairo", dpi=300, width=18, height=7)
        }
        )





calConditonsMulti <- reactiveValues()


observeEvent(input$createcalelement_multi, {
    
    cal.condition <- input$radiocal_multi
    norm.condition <- input$normcal_multi
    
    norm.min <- print(input$comptonmin_multi)
    norm.max <- print(input$comptonmax_multi)
    
    cal.table <- data.frame(cal.condition, norm.condition, norm.min, norm.max)
    colnames(cal.table) <- c("CalType", "NormType", "Min", "Max")
    
    slope.corrections <- input$slope_vars_multi
    intercept.corrections <- input$intercept_vars_multi
    
    standards.used <- 1
    
    cal.mode.list <- list(cal.table, slope.corrections, intercept.corrections, standards.used)
    names(cal.mode.list) <- c("CalTable", "Slope", "Intercept", "StandardsUsed")
    
    calConditonsMulti <<- cal.mode.list
    
})




        


        
        observeEvent(input$createcalelement_multi, {
            
           lapply(quantNames(), function(x)
            calListMulti[[x]][["calList"]][[input$calcurveelement_multi]] <<- list(isolate(calConditonsMulti), isolate(strip_glm(elementModelMulti()[[x]]))))
            
            
            
        })

observeEvent(input$createcalelement_multi, {
    
    lapply(quantNames(), function(x)
           calListMulti[[x]][["calList"]][[input$calcurveelement_multi]][["CalTable"]][["StandardsUsed"]] <<- isolate(as.vector(vals_multi$keeprows[[x]])))
      
    
    
})



emptyList <- reactive({
    a.list <- list()
    a.list
})

calPlotListMulti <- reactiveValues()
calPlotListMulti <- emptyList()
observeEvent(input$createcalelement_multi, {
    
        calPlotListMulti[[input$calcurveelement_multi]] <- isolate(calPlotDownloadMulti())

    
    calPlotListMulti <<- calPlotListMulti
    
})

CalibrationPlotsMulti <- reactiveValues()
observeEvent(input$createcal_multi, {
    
    CalibrationPlotsMulti$calCurves <<- calPlotListMulti
    
    
})

#observeEvent(input$createcal, {

#CalibrationPlots$diagPlots <<- diagPlotList


#})



output$downloadModel_multi <- downloadHandler(
filename <- function(){
    paste(input$defaultcal, "quant", sep=".")
},

content = function(file) {
    saveRDS(calListMulti[[input$defaultcal]], file = file, compress="xz")
}
)


output$downloadReport_multi <- downloadHandler(
function() { paste("summarycurves", '.pdf',  sep='') },
content = function(file){
    ml = marrangeGrob(grobs=CalibrationPlotsMulti$calCurves, nrow=1, ncol=1)
    ggsave(file, ml, device="pdf", dpi=300, width=16, height=7)
    
    dev.off()
})






        

})
        
        
    })
    
    
    output$filevalgrab <- renderUI({
        
        if(input$valfiletype=="CSV") {
            fileInput('loadvaldata', 'Choose CSV', multiple=TRUE,
            accept=c(".csv"))
        } else if(input$valfiletype=="TXT") {
            fileInput('loadvaldata', 'Choose TXT', multiple=TRUE,
            accept=c(".txt"))
        } else if(input$valfiletype=="Net") {
            fileInput('loadvaldata', 'Choose Net Counts', multiple=TRUE,
            accept=c(".csv"))
        } else if(input$valfiletype=="Elio") {
            fileInput('loadvaldata', 'Choose Elio Spectra', multiple=TRUE,
            accept=c(".spt"))
        } else if(input$valfiletype=="MCA") {
            fileInput('loadvaldata', 'Choose MCA File', multiple=TRUE,
            accept=c(".mca"))
        } else if(input$valfiletype=="SPX") {
            fileInput('loadvaldata', 'Choose Artax File', multiple=TRUE,
            accept=c(".spx"))
        } else if(input$valfiletype=="PDZ") {
            fileInput('loadvaldata', 'Choose PDZ File', multiple=TRUE,
            accept=c(".pdz"))
        }
        
    })
    
    

    
    
    observeEvent(input$processvalspectra, {
        
        fullValSpectra <- reactive({
            
            
            withProgress(message = 'Processing Data', value = 0, {
                
                inFile <- input$loadvaldata
                if (is.null(inFile)) return(NULL)
                temp = inFile$name
                temp <- gsub(".csv", "", temp)
                id.seq <- seq(1, 2048,1)
                
                n <- length(temp)*id.seq
                
                n.seq <- seq(1, length(inFile$name), 1)
                
                
                data <- pblapply(n.seq, function(x) csvFrame(filepath=inFile$datapath[x], filename=inFile$name[x]))
                data <- do.call("rbind", data)
                
                
                incProgress(1/n)
                Sys.sleep(0.1)
            })
            
            data$Energy <- data$Energy + gainshiftHold()
            
            data
        })
        
        readValTXT <- reactive({
            
            withProgress(message = 'Processing Data', value = 0, {
                
                inFile <- input$loadvaldata
                if (is.null(inFile)) return(NULL)
                
                n <- length(inFile$datapath)
                names <- inFile$name
                
                myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readTXTData(filepath=inFile$datapath[x], filename=inFile$name[x]))))
                
                incProgress(1/n)
                Sys.sleep(0.1)
            })
            
            myfiles.frame$Energy <- myfiles.frame$Energy + gainshiftHold()
            
            myfiles.frame
            
        })
        
        
        netValCounts <- reactive({
            
            withProgress(message = 'Processing Data', value = 0, {
                
                
                inFile <- input$loadvaldata
                if (is.null(inFile)) return(NULL)
                
                #inName <- inFile$name
                #inPath <- inFile$datapath
                
                #inList <- list(inName, inPath)
                #names(inList) <- c("inName", "inPath")
                
                
                n <- length(inFile$name)
                net.names <- gsub("\\@.*","",inFile$name)
                
                myfiles = pblapply(inFile$datapath,  read_csv_net)
                
                
                myfiles.frame.list <- pblapply(myfiles, data.frame, stringsAsFactors=FALSE)
                nms = unique(unlist(pblapply(myfiles.frame.list, names)))
                myfiles.frame <- as.data.frame(do.call(rbind, lapply(myfiles.frame.list, "[", nms)))
                myfiles.frame <- as.data.frame(sapply(myfiles.frame, as.numeric))
                
                
                #myfiles.frame$Spectrum <- net.names
                
                united.frame <- data.frame(net.names, myfiles.frame)
                colnames(united.frame) <- c("Spectrum", names(myfiles.frame))
                #united.frame$None <- rep(1, length(united.frame$Spectrum))
                
                
                incProgress(1/n)
                Sys.sleep(0.1)
            })
            
            united.frame <- as.data.frame(united.frame)
            united.frame
            
        })
        
        readValElio <- reactive({
            
            withProgress(message = 'Processing Data', value = 0, {
                
                inFile <- input$loadvaldata
                if (is.null(inFile)) return(NULL)
                
                n <- length(inFile$datapath)
                names <- inFile$name
                
                myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readSPTData(filepath=inFile$datapath[x], filename=inFile$name[x]))))
                
                
                incProgress(1/n)
                Sys.sleep(0.1)
            })
            
            
            myfiles.frame
            
            
        })
        
        readValMCA <- reactive({
            
            withProgress(message = 'Processing Data', value = 0, {
                
                inFile <- input$loadvaldata
                if (is.null(inFile)) return(NULL)
                
                n <- length(inFile$datapath)
                names <- inFile$name
                
                myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readMCAData(filepath=inFile$datapath[x], filename=inFile$name[x]))))
                
                
                incProgress(1/n)
                Sys.sleep(0.1)
            })
            
            
            myfiles.frame
            
            
        })
        
        readValSPX <- reactive({
            
            withProgress(message = 'Processing Data', value = 0, {
                
                inFile <- input$loadvaldata
                if (is.null(inFile)) return(NULL)
                
                n <- length(inFile$datapath)
                names <- inFile$name
                
                myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readSPXData(filepath=inFile$datapath[x], filename=inFile$name[x]))))
                
                
                incProgress(1/n)
                Sys.sleep(0.1)
            })
            
            myfiles.frame
            
            
        })
            
            
            readvalPDZ <- reactive({
                
                withProgress(message = 'Processing Data', value = 0, {
                    
                    inFile <- input$loadvaldata
                    if (is.null(inFile)) return(NULL)
                    
                    n <- length(inFile$datapath)
                    names <- inFile$name
                    
                    myfiles.frame <- as.data.frame(do.call(rbind, lapply(seq(1, n, 1), function(x) readPDZData(filepath=inFile$datapath[x], filename=inFile$name[x]))))
                    
                    
                    incProgress(1/n)
                    Sys.sleep(0.1)
                })
                
                myfiles.frame$Energy <- myfiles.frame$Energy + gainshiftHold()
                
                myfiles.frame
                
                
            })
            
            

        
        
        myValData <- reactive({
            
            data <- if(input$valfiletype=="CSV"){
                fullValSpectra()
            } else if(input$valfiletype=="TXT"){
                readValTXT()
            } else if(input$valfiletype=="Net"){
                netValCounts()
            } else if(input$valfiletype=="Elio") {
                readValElio()
            }  else if(input$valfiletype=="MCA") {
                readValMCA()
            }  else if(input$valfiletype=="SPX") {
                readValSPX()
            }  else if(input$valfiletype=="PDZ") {
                readvalPDZ()
            }
            
            data
            
        })
        
        
        
        calFileContents2 <- reactive({
            
            existingCalFile <- input$calfileinput2
            
            if (is.null(existingCalFile)) return(NULL)
            
            
            Calibration <- readRDS(existingCalFile$datapath)
            
            Calibration
            
        })
        
        valdata <- myValData()

        
        
        output$contents2 <- renderTable({
            
            
            
            myValData()
            
        })
        
        calValHold <- reactive({
            

            calFileContents2()[["calList"]]
            
            
            


        })
        
        calVariables <- reactive({
            

                calFileContents2()$Intensities

            
            
        })
        
        calValElements <- reactive({
            calList <- calValHold()
            valelements <- ls(calList)
            
            valelements.simp <- gsub(".K.alpha", "", valelements)
            valelements.simp <- gsub(".K.beta", "", valelements.simp)
            valelements.simp <- gsub(".L.alpha", "", valelements.simp)
            valelements.simp <- gsub(".L.beta", "", valelements.simp)
            valelements.simp <- gsub(".M.line", "", valelements.simp)

            
            valelements <- as.vector(as.character(valelements[match(as.character(fluorescence.lines$Symbol), valelements.simp)]))
            
            valelements <- c(valelements, ls(calList)[!(ls(calList) %in% valelements)])

            
            valelements
        })
        
        calVariableElements <- reactive({
            variables <- calVariables()
            variableelements <- ls(variables)
            
            #variableelements.simp <- gsub(".K.alpha", "", variableelements)
            #variableelements.simp <- gsub(".K.beta", "", variableelements)
            #variableelements.simp <- gsub(".L.alpha", "", variableelements)
            #variableelements.simp <- gsub(".L.beta", "", variableelements)
            #variableelements.simp <- gsub(".M.line", "", variableelements)
            
            #variableelements <- as.vector(as.character(na.omit(variableelements[match(as.character(fluorescence.lines$Symbol), variableelements.simp)])))

            variableelements
        })
        
        
        
        calDefinitions <- reactive({
            
            if(!is.null(calFileContents2()[["Definitions"]])){
                calFileContents2()[["Definitions"]]
            } else if(is.null(calFileContents2()[["Definitions"]])){
                NULL
            }
            
        })
        
        valDataType <- reactive({
            
            if(input$valfiletype=="CSV"){
                "Spectra"
            } else if(input$valfiletype=="TXT"){
                "Spectra"
            } else if(input$valfiletype=="Net"){
                "Net"
            } else if(input$valfiletype=="Elio") {
                "Spectra"
            } else if(input$valfiletype=="SPX") {
                "Spectra"
            } else if(input$valfiletype=="MCA") {
                "Spectra"
            } else if(input$valfiletype=="PDZ") {
                "Spectra"
            }
            
        })
        
        
  
  
        
        
        tableInputValCounts <- reactive({
            valelements <- calValElements()
            variableelements <- calVariableElements()
            val.data <- myValData()
            
            if(valDataType()=="Spectra"){spectra.line.list <- lapply(valelements, function(x) elementGrab(element.line=x, data=val.data, range.table=calDefinitions()))}
            if(valDataType()=="Spectra"){element.count.list <- lapply(spectra.line.list, '[', 2)}
            
            
            
            if(valDataType()=="Spectra"){spectra.line.vector <- as.numeric(unlist(element.count.list))}
            
            if(valDataType()=="Spectra"){dim(spectra.line.vector) <- c(length(spectra.line.list[[1]]$Spectrum), length(valelements))}
            
            if(valDataType()=="Spectra"){spectra.line.frame <- data.frame(spectra.line.list[[1]]$Spectrum, spectra.line.vector)}
            
            if(valDataType()=="Spectra"){colnames(spectra.line.frame) <- c("Spectrum", valelements)}
            
            if(valDataType()=="Spectra"){spectra.line.frame <- as.data.frame(spectra.line.frame)}
            
            if(valDataType()=="Spectra"){spectra.line.frame}
            
            if(valDataType()=="Spectra"){val.line.table <- spectra.line.frame[, c("Spectrum", valelements), drop = FALSE]}
            
            
            if(valDataType()=="Net"){val.line.table <- val.data[c("Spectrum", valelements), drop=FALSE]}
                
                
                val.line.table


        })
        

        
        
        fullInputValCounts <- reactive({
            valelements <- calValElements()
            variableelements <- calVariableElements()
            val.data <- myValData()
            
            if(valDataType()=="Spectra" ){spectra.line.list <- lapply(variableelements, function(x) elementGrab(element.line=x, data=val.data, range.table=calDefinitions()))}
            if(valDataType()=="Spectra"){element.count.list <- lapply(spectra.line.list, `[`, 2)}
            
            
            if(valDataType()=="Spectra"){spectra.line.vector <- as.numeric(unlist(element.count.list))}
            
            if(valDataType()=="Spectra"){dim(spectra.line.vector) <- c(length(spectra.line.list[[1]]$Spectrum), length(variableelements))}
            
            if(valDataType()=="Spectra"){spectra.line.frame <- data.frame(spectra.line.list[[1]]$Spectrum, spectra.line.vector)}
            
            if(valDataType()=="Spectra"){colnames(spectra.line.frame) <- c("Spectrum", variableelements)}
            
            if(valDataType()=="Spectra"){spectra.line.frame <- as.data.frame(spectra.line.frame)}
            
            if(valDataType()=="Spectra"){val.line.table <- spectra.line.frame[c("Spectrum", variableelements)]}
            
            if(valDataType()=="Net"){val.line.table <- val.data}
            
            
            val.line.table
        })

        
        
        output$myvaltable1 <- renderDataTable({
            
            fullInputValCounts()
            
        })
        
        
        
        
        tableInputValQuant <- reactive({
            
            
            
        count.table <- data.frame(fullInputValCounts())
        the.cal <- calValHold()
        elements.cal <- calValElements()
        elements <- elements.cal[!is.na(match(elements.cal, ls(count.table)))]
        variables <- calVariableElements()
        valdata <- myValData()
        
        #elements <- fluorescence.lines$Symbol[sort(order(fluorescence.lines$Symbol)[elements])]

        cal_type <- function(element){
    
    
            if(the.cal[[element]][[1]]$CalTable$CalType==1){
                    1
                } else if(the.cal[[element]][[1]]$CalTable$CalType==2){
                        1
                    } else if(the.cal[[element]][[1]]$CalTable$CalType==3){
                            3
                        } else if(the.cal[[element]][[1]]$CalTable$CalType==4){
                                4
                            } else if(the.cal[[element]][[1]]$CalTable$CalType==5){
                                5
                                }
    
        }

            
            
        predicted.list <- lapply(elements, function (x)
            if(valDataType()=="Spectra" && cal_type(x)==1 && the.cal[[x]][[1]]$CalTable$NormType==1){
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=general_prep_xrf(
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x)
                )
            } else if(valDataType()=="Spectra" && cal_type(x)==1 && the.cal[[x]][[1]]$CalTable$NormType==2) {
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=simple_tc_prep_xrf(
                        data=valdata,
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x
                    )
                )
            } else if(valDataType()=="Spectra" && cal_type(x)==1 && the.cal[[x]][[1]]$CalTable$NormType==3) {
                predict(
                    object=the.cal[[x]][[2]],
                        newdata=simple_comp_prep_xrf(
                            data=valdata,
                            spectra.line.table=as.data.frame(
                                count.table
                                ),
                            element.line=x,
                            norm.min=the.cal[[x]][[1]][1]$CalTable$Min,
                            norm.max=the.cal[[x]][[1]][1]$CalTable$Max
                            )
                )
            } else if(valDataType()=="Spectra" && cal_type(x)==3 && the.cal[[x]][[1]]$CalTable$NormType==1){
                 predict(
                    object=the.cal[[x]][[2]],
                    newdata=lucas_simp_prep_xrf(
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x,
                        slope.element.lines=the.cal[[x]][[1]][2]$Slope,
                        intercept.element.lines=the.cal[[x]][[1]][3]$Intercept
                        )
                 )
            } else if(valDataType()=="Spectra" && cal_type(x)==3 && the.cal[[x]][[1]]$CalTable$NormType==2){
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=lucas_tc_prep_xrf(
                        data=valdata,
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x,
                        slope.element.lines=the.cal[[x]][[1]][2]$Slope,
                        intercept.element.lines=the.cal[[x]][[1]][3]$Intercept
                    )
                )
            } else if(valDataType()=="Spectra" && cal_type(x)==3 && the.cal[[x]][[1]]$CalTable$NormType==3){
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=lucas_comp_prep_xrf(
                        data=valdata,
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x,
                        slope.element.lines=the.cal[[x]][[1]][2]$Slope,
                        intercept.element.lines=the.cal[[x]][[1]][3]$Intercept,
                        norm.min=the.cal[[x]][[1]][1]$CalTable$Min,
                        norm.max=the.cal[[x]][[1]][1]$CalTable$Max
                        )
                )
            } else if(valDataType()=="Spectra" && cal_type(x)==4 && the.cal[[x]][[1]]$CalTable$NormType==1){
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=lucas_simp_prep_xrf(
                        spectra.line.table=as.data.frame(
                        count.table
                        ),
                    element.line=x,
                    slope.element.lines=variables,
                    intercept.element.lines=the.cal[[x]][[1]][3]$Intercept
                    )
                )
            } else if(valDataType()=="Spectra" && cal_type(x)==4 && the.cal[[x]][[1]]$CalTable$NormType==2){
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=lucas_tc_prep_xrf(
                        data=valdata,
                        spectra.line.table=as.data.frame(
                        count.table
                        ),
                    element.line=x,
                    slope.element.lines=variables,
                    intercept.element.lines=the.cal[[x]][[1]][3]$Intercept
                    )
                )
            } else if(valDataType()=="Spectra" && cal_type(x)==4 && the.cal[[x]][[1]]$CalTable$NormType==3){
                predict(
                object=the.cal[[x]][[2]],
                    newdata=lucas_comp_prep_xrf(
                        data=valdata,
                        spectra.line.table=as.data.frame(
                        count.table
                        ),
                    element.line=x,
                    slope.element.lines=variables,
                    intercept.element.lines=the.cal[[x]][[1]][3]$Intercept,
                    norm.min=the.cal[[x]][[1]][1]$CalTable$Min,
                    norm.max=the.cal[[x]][[1]][1]$CalTable$Max
                    )
                )
            } else if(valDataType()=="Spectra" && cal_type(x)==5 && the.cal[[x]][[1]]$CalTable$NormType==1){
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=spectra_simp_prep_xrf(valdata)[,-1]
                )
            } else if(valDataType()=="Spectra" && cal_type(x)==5 && the.cal[[x]][[1]]$CalTable$NormType==2){
                predict(
                object=the.cal[[x]][[2]],
                newdata=spectra_tc_prep_xrf(valdata)[,-1]
                )
            } else if(valDataType()=="Spectra" && cal_type(x)==5 && the.cal[[x]][[1]]$CalTable$NormType==3){
                predict(
                object=the.cal[[x]][[2]],
                newdata=spectra_comp_prep_xrf(valdata,
                    norm.min=the.cal[[x]][[1]][1]$CalTable$Min,
                    norm.max=the.cal[[x]][[1]][1]$CalTable$Max)[,-1]
                )
            } else if(valDataType()=="Net" && cal_type(x)==1 && the.cal[[x]][[1]]$CalTable$NormType==1){
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=general_prep_xrf_net(
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x)
                )
            } else if(valDataType()=="Net" && cal_type(x)==1 && the.cal[[x]][[1]]$CalTable$NormType==2) {
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=simple_tc_prep_xrf_net(
                        data=valdata,
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                            element.line=x
                            )
                )
            } else if(valDataType()=="Net" && cal_type(x)==1 && the.cal[[x]][[1]]$CalTable$NormType==3) {
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=simple_comp_prep_xrf_net(
                        data=valdata,
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x,
                        norm.min=the.cal[[x]][[1]][1]$CalTable$Min,
                        norm.max=the.cal[[x]][[1]][1]$CalTable$Max
                        )
                )
            } else if(valDataType()=="Net" && cal_type(x)==3 && the.cal[[x]][[1]]$CalTable$NormType==1){
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=lucas_simp_prep_xrf_net(
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x,
                        slope.element.lines=the.cal[[x]][[1]][2]$Slope,
                        intercept.element.lines=the.cal[[x]][[1]][3]$Intercept
                        )
                )
            } else if(valDataType()=="Net" && cal_type(x)==3 && the.cal[[x]][[1]]$CalTable$NormType==2){
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=lucas_tc_prep_xrf_net(
                        data=valdata,
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x,
                        slope.element.lines=the.cal[[x]][[1]][2]$Slope,
                        intercept.element.lines=the.cal[[x]][[1]][3]$Intercept
                        )
                )
            } else if(valDataType()=="Net" && cal_type(x)==3 && the.cal[[x]][[1]]$CalTable$NormType==3){
                predict(
                    object=the.cal[[x]][[2]],
                    newdata=lucas_comp_prep_xrf_net(
                        data=valdata,
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x,
                        slope.element.lines=the.cal[[x]][[1]][2]$Slope,
                        intercept.element.lines=the.cal[[x]][[1]][3]$Intercept,
                        norm.min=the.cal[[x]][[1]][1]$CalTable$Min,
                        norm.max=the.cal[[x]][[1]][1]$CalTable$Max
                        )
                )
        } else if(valDataType()=="Net" && cal_type(x)==4 && the.cal[[x]][[1]]$CalTable$NormType==1){
            predict(
                object=the.cal[[x]][[2]],
                newdata=lucas_simp_prep_xrf_net(
                    spectra.line.table=as.data.frame(
                        count.table
                        ),
                    element.line=x,
                    slope.element.lines=variables,
                    intercept.element.lines=the.cal[[x]][[1]][3]$Intercept
                    )
            )
        } else if(valDataType()=="Net" && cal_type(x)==4 && the.cal[[x]][[1]]$CalTable$NormType==2){
            predict(
                object=the.cal[[x]][[2]],
                newdata=lucas_tc_prep_xrf_net(
                    data=valdata,
                        spectra.line.table=as.data.frame(
                            count.table
                            ),
                        element.line=x,
                        slope.element.lines=variables,
                        intercept.element.lines=the.cal[[x]][[1]][3]$Intercept
                        )
            )
        } else if(valDataType()=="Net" && cal_type(x)==4 && the.cal[[x]][[1]]$CalTable$NormType==3){
            predict(
                object=the.cal[[x]][[2]],
                newdata=lucas_comp_prep_xrf_net(
                data=valdata,
                    spectra.line.table=as.data.frame(
                        count.table
                        ),
                    element.line=x,
                    slope.element.lines=variables,
                    intercept.element.lines=the.cal[[x]][[1]][3]$Intercept,
                    norm.min=the.cal[[x]][[1]][1]$CalTable$Min,
                    norm.max=the.cal[[x]][[1]][1]$CalTable$Max
                    )
            )
        }
            )
            
        predicted.vector <- round(unlist(predicted.list), input$resultrounding)
        
        dim(predicted.vector) <- c(length(count.table$Spectrum), length(elements))
        
        predicted.frame <- data.frame(count.table$Spectrum, predicted.vector)
        
        colnames(predicted.frame) <- c("Spectrum", elements)
        #elements <- elements[order(match(fluorescence.lines$Symbol, elements))]

        

        predicted.data.table <- predicted.frame

        #predicted.values <- t(predicted.values)
        predicted.data.table
            
            
        })
        
        output$myvaltable2 <- renderDataTable({
            
            tableInputValQuant()
            
        })
        
        
        # valtest <- lapply(valelements, function(x) predict(calsList[[x]], as.data.frame(val.line.table[x])))
        
        output$downloadValData <- downloadHandler(
        filename = function() { paste(input$calname, "_ValData", '.csv', sep='', collapse='') },
        content = function(file
        ) {
            write.csv(tableInputValQuant(), file)
        }
        )
        
        
  


})

 })
